#!/bin/sh
# Script to run the EXAMPLE_AER_FILE_FWD code
#
# The result is compared to a reference file. See users guide
# to note what are the "normal" differences.
# 
# This script runs only ONE test for NOAA-16 AVHRR

# Set BIN directory if supplied
BIN=`perl -e 'for(@ARGV){m/^BIN=(\S+)$/o && print "$1";}' $*`
if [ "x$BIN" = "x" ]
then
  BIN=bin
fi

######## Edit this section for your pathnames and test case input ######

# Path relative to the rttov_test directory:
TEST=./test_example_aer_file_fwd.1

# Paths relative to the rttov_test/${TEST} directory:
BINDIR=../../$BIN                           # BIN directory (may be set with BIN= argument)
REF_TEST=../test_example_aer_file_fwd.2     # Test reference data
DATART=../../rtcoef_rttov11/rttov7pred54L   # Coefficients directory
DATASC=../../rtcoef_rttov11/cldaer          # Scattering coefficients directory

# Test case input data
COEF_FILENAME="rtcoef_msg_3_seviri.dat"
AER_COEF_FILENAME="scaercoef_msg_3_seviri.dat"
PROF_FILENAME="prof.dat"
AER_PROF_FILENAME="aer_prof.dat"
NPROF=2         # Number of profiles defined in prof.dat
NLEVELS=51      # Number of profile levels
DO_SOLAR=0      # 0 = solar off / 1 = solar on

########################################################################

ARG_ARCH=`perl -e 'for(@ARGV){m/^ARCH=(\S+)$/o && print "$1";}' $*`
if [ ! "x$ARG_ARCH" = "x" ]; then
  ARCH=$ARG_ARCH
fi
if [ "x$ARCH" = "x" ];
then
  echo 'Please supply ARCH'
  exit 1
fi

CWD=`pwd`
cd $TEST

echo " "
echo " "
echo " Test forward with aerosol simulations using pre-defined particle types"
echo " "

echo  "Coef filename:           ${COEF_FILENAME}"
echo  "Aerosol coef filename:   ${AER_COEF_FILENAME}"
echo  "Input profile file:      ${PROF_FILENAME}"
echo  "Input aerosol prof file: ${AER_PROF_FILENAME}"
echo  "Number of profiles:      ${NPROF}"
echo  "Number of levels:        ${NLEVELS}"
echo  "Do solar:                ${DO_SOLAR}"


# Coefficient files
rm -f $COEF_FILENAME
if [ -s $DATART/$COEF_FILENAME ]; then
  ln -s $DATART/$COEF_FILENAME
else
  echo "Coef file $DATART/$COEF_FILENAME not found, aborting..."
  exit 1
fi

rm -f $AER_COEF_FILENAME
if [ -s $DATASC/$AER_COEF_FILENAME ]; then
  ln -s $DATASC/$AER_COEF_FILENAME
else
  echo "Coef file $DATASC/$AER_COEF_FILENAME not found, aborting..."
  exit 1
fi

$BINDIR/example_aer_file_fwd.exe << EOF
${COEF_FILENAME}    , Coefficient filename
${AER_COEF_FILENAME}, Aerosol coefficient filename
${PROF_FILENAME}    , Input profile filename
${AER_PROF_FILENAME}, Input aerosol profile filename
${NPROF}            , Number of profiles
${NLEVELS}          , Number of levels
${DO_SOLAR}         , Turn solar radiation on/off
 1, 1, 0.0, 0.0     , channel 1, valid, emissivity=0.0, BRDF=0.0
 2, 1, 0.0, 0.0     , channel 2, valid, emissivity=0.0, BRDF=0.0
 3, 1, 0.0, 0.0     , channel 3, valid, emissivity=0.0, BRDF=0.0
 4, 1, 0.0, 0.0     , channel 4, valid, emissivity=0.0, BRDF=0.0
 5, 1, 0.0, 0.0     , channel 5, valid, emissivity=0.0, BRDF=0.0
 6, 1, 0.0, 0.0     , channel 6, valid, emissivity=0.0, BRDF=0.0
 7, 1, 0.0, 0.0     , channel 7, valid, emissivity=0.0, BRDF=0.0
 8, 1, 0.0, 0.0     , channel 8, valid, emissivity=0.0, BRDF=0.0
EOF

if [ $? -ne 0 ]; then
  echo " "
  echo "TEST FAILED"
  echo " "
  exit 1
fi

OUT_FILE=output_example_aer_file_fwd.dat
DIFF_FILE=diff_example_aer_file_fwd.${ARCH}

mv ${OUT_FILE} ${OUT_FILE}.${ARCH}

if [ $? -ne 0 ]; then
  echo "Expected output file not found"
  exit 1
fi

echo
echo "Output is in the file ${TEST}/${OUT_FILE}.${ARCH}"

if [ -f ${REF_TEST}/${OUT_FILE} ]; then

  diff -biw ${OUT_FILE}.${ARCH} ${REF_TEST}/${OUT_FILE} > $DIFF_FILE

  if [ `stat -c %s $DIFF_FILE` -eq 0 ]; then
    echo " "
    echo "Diff file has zero size: TEST SUCCESSFUL"
    echo " "
  else
    echo "--- Diff file contents: ---"
    cat $DIFF_FILE
    echo "---------------------------"
  fi
else
  echo "Test reference output not found"
fi
echo

rm -f $COEF_FILENAME $AER_COEF_FILENAME

cd $CWD

exit
