
SUBROUTINE rttov_ad( &
            & errorstatus,       &
            & chanprof,          &
            & opts,              &
            & profiles,          &
            & profiles_ad,       &
            & coefs,             &
            & transmission,      &
            & transmission_ad,   &
            & radiancedata,      &
            & radiancedata_ad,   &
            & calcemis,          &
            & emissivity,        &
            & emissivity_ad,     &
            & calcrefl,          &
            & reflectance,       &
            & reflectance_ad,    &
            & aer_opt_param,     &
            & cld_opt_param,     &
            & traj,              &
            & traj_ad,           &
            & pccomp,            &
            & pccomp_ad,         &
            & channels_rec)
!
! Description:
! Adjoint of rttov_direct
! to compute multi-channel level to space transmittances,
! top of atmosphere and level to space radiances and brightness
! temperatures and optionally surface emissivities, for many
! profiles in a single call, for satellite
! infrared or microwave sensors. The code requires a coefficient file
! for each sensor for which simulated radiances are requested.
!
! Note that radiancedata_ad can be used for all its structure elements
! In normal case the element total or bt is the only one initialised but
! for some particular cases like for rttov_cld_ad some other elements
! have been already init.
! According to the argument opts%rt_all%switchrad the main input total or bt is used
! opts%rt_all%switchrad == true    bt is the input, brightness temperature
! opts%rt_all%switchrad == false   total is the input, radiance
!
! The AD outputs profiles_ad and emissivity_ad should be allocated and
! initialised before calling the subroutine
!
! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2002, EUMETSAT, All Rights Reserved.
!
! Method: The methodology is described in the following:
!
! Eyre J.R. and H.M. Woolf  1988 Transmittance of atmospheric gases
! in the microwave region: a fast model. Applied Optics 27  3244-3249
!
! Eyre J.R. 1991 A fast radiative transfer model for satellite sounding
! systems.  ECMWF Research Dept. Tech. Memo. 176 (available from the
! librarian at ECMWF).
!
! Saunders R.W., M. Matricardi and P. Brunel 1999 An Improved Fast Radiative
! Transfer Model for Assimilation of Satellite Radiance Observations.
! QJRMS, 125, 1407-1425.
!
! Matricardi, M., F. Chevallier and S. Tjemkes 2001 An improved general
! fast radiative transfer model for the assimilation of radiance
! observations. ECMWF Research Dept. Tech. Memo. 345
! (available from the librarian at ECMWF).
!
! Matricardi, M. 2003 RTIASI-4, a new version of the ECMWF fast radiative
! transfer model for the infrared atmospheric sounding interferometer.
! ECMWF Research Dept. Tech. Memo. 425 (available from the librarian at ECMWF)
!
! Current Code Owner: SAF NWP
!
! History:
! Version   Date     Comment
! -------   ----     -------
!  1.1   01/12/2002  New F90 code with structures (P Brunel A Smith)
!  1.2   02/01/2003  More comments added (R Saunders)
!  1.3   24/01/2003  Error return code by input profile (P Brunel)
!  1.4               Add WV Continuum and CO2 capability
!  1.5   04/12/2003  Optimisation (J Hague and D Salmond ECMWF)
!  1.6   02/06/2004  Change tests on id_comp_lvl == 7 by tests on fmv_model_ver (P. Brunel)
!  1.7   08/12/2005  Added surface humidity to lowest mean layer q (R Saunders)
!  1.8   01/06/2005  Marco Matricardi (ECMWF):
!              --    IASI capability added.
!              --    Variable trace gases added for IASI and AIRS.
!              --    Solar radiation added for IASI and AIRS
!              --    Altitude dependent local zenith angle added.
!              --    Linear in tau approximation for RT equation added.
!  1.9    06/12/2005 Minor mods to initialise of traj0_ad%sunglint (R Saunders)
!  1.10   08/02/2007 Removed polarisation indexing (R Saunders)
!         29/06/2007 Introduce interpolator (P.J.Rayer)
!              --    Profiles on USER levs -> COEF levs
!              --    Duplicate profaux and setgeometry (USER and COEF levs)
!              --    Predict atm/sol optical depths on COEF levs -> USER levs
!              --    Calculate transmittance and integrate RT on USER levs
!  1.11   11/10/2007 Move iaernum & iaertyp profile members to traj0%aux_prof P.Marguinaud
!  1.12   16/01/2008 Facility to apply regression limits (N. Bormann)
!  1.13   26/09/2008 Disable vectorization of the pointer association
!                    loops and nullify temp pointers; this is for the NEC
!                    under -Chopt. P. Marguinaud
!
!  1.13   03/10/2008 Fix bug in checking addsolar (A. Geer)
!  1.14   15/08/2009 User defined ToA. Layers distinct from levels (P.Rayer)
!  1.15   03/11/2009 Transmittances / optical depths on levels (A Geer)
!  1.16   02/12/2009 Introduced principal component capability. Marco Matricardi. ECMWF
!  1.17   17/06/2010 Introduced optional spacetop flag to zero opdeps at user's
!                    model-top in Zeeman channels (P Rayer)
!  1.28   05/07/2010 Remove addsolar flag from profiles structure (J Hocking)
!  1.29   14/10/2010 Remove rt8_mode (J Hocking)
!
! 2010/03 Code cleaning Pascal Brunel, Philippe Marguinaud
!
!
! Code Description:
!   Language:           Fortran 90.
!   Software Standards: "European Standards for Writing and
!     Documenting Exchangeable Fortran 90 Code".
! A user guide and technical documentation is available at
! http://www.metoffice.com/research/interproj/nwpsaf/rtm/index.html
!
! Declarations:
! Modules used:
! Imported Parameters:
!INTF_OFF
#include "throw.h"
!INTF_ON
! Imported Type Definitions:
  USE rttov_types, ONLY :  &
       & rttov_coefs,       &
       & rttov_pccomp,      &
       & profile_Type,      &
       & transmission_Type, &
       & radiance_Type,     &
       & rttov_options,     &
       & rttov_chanprof,    &
       & rttov_emissivity,  &
       & rttov_reflectance, &
       & rttov_opt_param,   &
       & rttov_traj
  USE parkind1, ONLY : jpim, jplm
!INTF_OFF
  USE parkind1, ONLY : jprb
  USE rttov_const, ONLY :  &
         sensor_id_mw, &
         sensor_id_ir, &
         sensor_id_po, &
         sensor_id_hi, &
         ncldtyp
  USE yomhook, ONLY : LHOOK, DR_HOOK
  USE rttov_types, ONLY :  &
       & rttov_traj_sta,    &
       & rttov_traj_dyn      
!INTF_ON
  IMPLICIT NONE
!subroutine arguments:
  INTEGER(KIND=jpim)     , INTENT(OUT)                     :: errorstatus
  TYPE(profile_Type  )   , INTENT(IN)                      :: profiles(:)
  TYPE(rttov_chanprof)   , INTENT(IN)                      :: chanprof(:)
  TYPE(rttov_options )   , INTENT(IN)                      :: opts
  TYPE(profile_Type  )   , INTENT(INOUT)                   :: profiles_ad(size(profiles))
  TYPE(rttov_coefs   )   , INTENT(IN)   , TARGET           :: coefs
  TYPE(transmission_Type), INTENT(INOUT)                   :: transmission        ! in because of mem allocation
  TYPE(transmission_Type), INTENT(INOUT)                   :: transmission_ad     ! in because of mem allocation
  TYPE(radiance_Type    ), INTENT(INOUT)                   :: radiancedata        ! in because of mem allocation
  TYPE(radiance_Type    ), INTENT(INOUT)                   :: radiancedata_ad     ! in because of mem allocation
  LOGICAL(KIND=jplm)     , INTENT(IN)   , OPTIONAL         :: calcemis(size(chanprof))
  TYPE(rttov_emissivity) , INTENT(INOUT), OPTIONAL         :: emissivity(size(chanprof))
  TYPE(rttov_emissivity) , INTENT(INOUT), OPTIONAL         :: emissivity_ad(size(chanprof))
  LOGICAL(KIND=jplm)     , INTENT(IN)   , OPTIONAL         :: calcrefl(size(chanprof))
  TYPE(rttov_reflectance), INTENT(INOUT), OPTIONAL         :: reflectance(size(chanprof))
  TYPE(rttov_reflectance), INTENT(INOUT), OPTIONAL         :: reflectance_ad(size(chanprof))
  TYPE(rttov_opt_param)  , INTENT(IN)   , OPTIONAL         :: aer_opt_param
  TYPE(rttov_opt_param)  , INTENT(IN)   , OPTIONAL         :: cld_opt_param
  TYPE(rttov_traj       ), INTENT(INOUT), OPTIONAL, TARGET :: traj, traj_ad       ! target is needed here
  TYPE(rttov_pccomp     ), INTENT(INOUT), OPTIONAL         :: pccomp
  TYPE(rttov_pccomp     ), INTENT(INOUT), OPTIONAL         :: pccomp_ad
  INTEGER(KIND=jpim)     , INTENT(IN)   , OPTIONAL         :: channels_rec(:)
!INTF_END
#include "rttov_apply_reg_limits_ad.interface"
#include "rttov_errorreport.interface"
#include "rttov_setgeometry_ad.interface"
#include "rttov_calcemis_ir_ad.interface"
#include "rttov_calcsurfrefl_ad.interface"
#include "rttov_profaux_ad.interface"
#include "rttov_setpredictors_7_ad.interface"
#include "rttov_setpredictors_8_ad.interface"
#include "rttov_setpredictors_9_ad.interface"
#include "rttov_opdep_ad.interface"
#include "rttov_opdep_9_ad.interface"
#include "rttov_transmit_ad.interface"
#include "rttov_transmit_9_solar_ad.interface"
#include "rttov_calcemis_mw_ad.interface"
#include "rttov_integrate_ad.interface"
#include "rttov_intavg_prof_ad.interface"
#include "rttov_intavg_chan_ad.interface"
#include "rttov_refsun_ad.interface"
#include "rttov_init_prof.interface"
#include "rttov_check_traj.interface"
#include "rttov_init_predictor.interface"
#include "rttov_init_raytracing.interface"
#include "rttov_cldstr_ad.interface"
#include "rttov_opdpscattir_ad.interface"
#include "rttov_fresnel_ad.interface"
#include "rttov_pcscores_ad.interface"
#include "rttov_reconstruct_ad.interface"
#include "rttov_calcbt_pc_ad.interface"
#include "rttov_add_raytracing.interface"
#include "rttov_add_aux_prof.interface"
#include "rttov_add_prof.interface"
#include "rttov_init_sunglint.interface"
#include "rttov_init_ircld.interface"
#include "rttov_init_opdp_path.interface"
#include "rttov_init_transmission_aux.interface"
#include "rttov_add_opdp_path.interface"
#include "rttov_init_aux_prof.interface"
#include "rttov_init_auxrad_stream.interface"
#include "rttov_init_trans_scatt_ir.interface"
#include "rttov_alloc_traj_dyn.interface"
#include "rttov_alloc_traj_sta.interface"
#include "rttov_direct.interface"
#include "rttov_nlte_bias_correction_ad.interface"
#include "rttov_calcbt_ad.interface"
!#include "rttov_calcsatrefl_ad.interface"

!local variables:
  LOGICAL(KIND=jplm) :: addcosmic! switch for adding temp of cosmic background

  TYPE(rttov_traj_dyn) :: traj0_dyn
  TYPE(rttov_traj_sta) :: traj0_sta
  TYPE(rttov_traj_dyn) :: traj0_ad_dyn

  LOGICAL(KIND=jplm)   :: ltraj_ad_dyn_dealloc

  TYPE(rttov_traj), TARGET  :: traj1
  TYPE(rttov_traj), POINTER :: traj0
  TYPE(rttov_traj), TARGET  :: traj1_ad
  TYPE(rttov_traj), POINTER :: traj0_ad

  TYPE(rttov_options) :: opts_coef
  INTEGER(KIND=jpim)  :: nlevels
  INTEGER(KIND=jpim)  :: nprofiles                    ! Number of profiles
  INTEGER(KIND=jpim)  :: nchannels                    ! Number of radiances computed (channels used * profiles)
  INTEGER(KIND=jpim)  :: npcscores
  INTEGER(KIND=jpim)  :: ERR
  REAL(KIND=JPRB)     :: ZHOOK_HANDLE

!- End of header --------------------------------------------------------
  TRY
  IF (LHOOK) CALL DR_HOOK('RTTOV_AD', 0_jpim, ZHOOK_HANDLE)
!-------------
!0. initialize
!-------------
  nprofiles                 = size(profiles)
  nchannels                 = size(chanprof)
  opts_coef                 = opts
  opts_coef%rt_ir%addaerosl = .FALSE.
  opts_coef%rt_ir%addclouds = .FALSE.
  nlevels                   = profiles(1)%nlevels
  errorstatus               = errorstatus_success

  ltraj_ad_dyn_dealloc = .FALSE.

  IF (opts%rt_ir%pc%addpc) THEN
    npcscores = size(pccomp%pcscores)
  ENDIF

  NULLIFY (traj0)
  NULLIFY (traj0_ad)
!-----------------------------------------------------------------------
!1. interpolator first call - input profiles from USER levs to COEF levs
!-----------------------------------------------------------------------
! done immediately in direct code - for quick escape if target values are out of bounds
  CALL rttov_check_traj( &
        & ERR,                 &
        & nprofiles,           &
        & nchannels,           &
        & opts,                &
        & nlevels,             &
        & coefs,               &
        & 1_jpim,              &
        & traj0 = traj0,       &
        & traj0_ad = traj0_ad, &
        & traj1 = traj1,       &
        & traj1_ad = traj1_ad, &
        & traj2 = traj,        &
        & traj2_ad = traj_ad)
  THROWM( ERR .NE. 0 , "rttov_check_traj fatal error")

  CALL rttov_direct( &
            & err,                           &
            & chanprof,                      &
            & opts,                          &
            & profiles,                      &
            & coefs,                         &
            & transmission,                  &
            & radiancedata,                  &
            & calcemis      = calcemis,      &
            & emissivity    = emissivity,    &
            & calcrefl      = calcrefl,      &
            & reflectance   = reflectance,   &
            & aer_opt_param = aer_opt_param, &
            & cld_opt_param = cld_opt_param, &
            & traj          = traj0,         &
            & traj_dyn      = traj0_dyn,     &
            & traj_sta      = traj0_sta,     &
            & pccomp        = pccomp,        &
            & channels_rec  = channels_rec)
  THROW(err.ne.0)

  IF (traj0_sta%dothermal .AND. .NOT. PRESENT(emissivity_ad)) THEN
    err = errorstatus_fatal
    THROWM(err.ne.0, "emissivity_ad parameter required")
  END IF
  IF (traj0_sta%dosolar .AND. .NOT. PRESENT(reflectance_ad)) THEN
    err = errorstatus_fatal
    THROWM(err.ne.0, "reflectance_ad parameter required")
  END IF

  CALL rttov_alloc_traj_dyn (err, traj0_ad_dyn, opts, nchannels, profiles(1)%nlayers, &
                             traj0_dyn%nstreams, ncldtyp, 1_jpim)
  THROW(err.ne.0)
  ltraj_ad_dyn_dealloc = .TRUE.
!
! Adjoint
!----------------
!
!---------------------------------------------
!0. allocate and initialize local AD variables
!---------------------------------------------
!0.1 profile AD-variables on COEF levels (USER levels done in tstrad_ad)
!------------------------------------------------------------------------
  CALL rttov_init_opdp_path(opts, traj0_ad%opdp_path)
  CALL rttov_init_opdp_path(opts, traj0_ad%opdp_path_coef)

  CALL rttov_init_ircld(traj0_ad%ircld)
  IF (opts%rt_ir%addaerosl .OR. opts%rt_ir%addclouds) THEN
    CALL rttov_init_trans_scatt_ir(traj0_ad%transmission_scatt_ir)
    CALL rttov_init_trans_scatt_ir(traj0_ad_dyn%transmission_scatt_ir_stream)
  ENDIF

  CALL rttov_init_auxrad_stream (traj0_ad_dyn%auxrad_stream)

  CALL rttov_init_transmission_aux (opts, traj0_ad_dyn%transmission_aux)

  CALL rttov_init_prof(traj0_ad%profiles_coef)
!0.2 auxillary profile AD-variables
!----------------------------------
! on USER levels
  CALL rttov_init_aux_prof(traj0_ad%aux_prof)
! on COEF levels
  CALL rttov_init_aux_prof(traj0_ad%aux_prof_coef)
!0.3 predictor AD-variables on COEF levels
!------------------------------------------
  CALL rttov_init_predictor(opts%rt_ir%addsolar, traj0_ad%predictors)
!0.4 raytracing AD-variables
!---------------------------
  CALL rttov_init_raytracing(traj0_sta%dosolar, traj0_ad%raytracing)
  CALL rttov_init_raytracing(traj0_sta%dosolar, traj0_ad%raytracing_coef)
!0.5 path opdep AD-variables on COEF levels
!--------------------------------------------------
! allocated with the direct arrays
! on USER levels
!0.6 transmission AD-variables on USER levels
!--------------------------------------------
! allocated with the direct arrays
!0.7 emissivity and traj0%thermrefl AD-variables - SURFACE
!-------------------------------------------------------
! emissivity_ad is init before calling
  traj0_ad%thermrefl(:) = 0._JPRB

  IF (traj0_sta%dosolar) THEN
    traj0_ad%fresnrefl(:) = 0._JPRB
    CALL rttov_init_sunglint(traj0_ad%sunglint)
  ENDIF

!--------------------------------------------------
! Set up radiance/BT inputs
!--------------------------------------------------

  IF (opts%rt_ir%pc%addpc) THEN

    IF (opts%rt_ir%pc%addradrec) THEN

      IF (opts%rt_all%switchrad) THEN
        CALL rttov_calcbt_pc_ad(         &
              & traj0_sta%chanprof_in,   &
              & coefs%coef_pccomp,       &
              & pccomp,                  &
              & pccomp_ad)
      ENDIF

      CALL rttov_reconstruct_ad( &
            & opts                 ,   &
            & traj0_sta%chanprof_in,   &
            & traj0_sta%chanprof_pc,   &
            & pccomp,                  &
            & pccomp_ad,               &
            & coefs%coef_pccomp)

    ENDIF

    CALL rttov_pcscores_ad( &
          & opts,                   &
          & chanprof,               &
          & traj0_sta%chanprof_pc,  &
          & pccomp,                 &
          & pccomp_ad,              &
          & coefs%coef_pccomp,      &
          & radiancedata_ad)

  ELSE

    ! Classical RTTOV
    IF (opts%rt_all%switchrad) THEN
      IF (traj0_sta%dothermal) &
        CALL rttov_calcbt_ad(chanprof, coefs%coef, traj0_sta%thermal, radiancedata, radiancedata_ad)
!       Input AD perturbation is always in radiance for pure-solar channels, never in reflectance
!       IF (traj0_sta%dosolar) &
!         CALL rttov_calcsatrefl_ad(chanprof, profiles, traj0_sta%solar_spec_esd, &
!                                   traj0_sta%thermal, traj0_sta%solar, radiancedata_ad)
    ELSE
      radiancedata_ad%clear(:) = 0._jprb
    ENDIF

  ENDIF


  ! Do NLTE bias correction for hyperspectral instruments (not PC-RTTOV)
  IF (.NOT. opts%rt_ir%pc%addpc) THEN
    IF (coefs%coef%nltecoef) THEN
      IF (opts%rt_ir%do_nlte_correction .AND. &
          coefs%coef%id_sensor == sensor_id_hi) THEN
        CALL rttov_nlte_bias_correction_ad(coefs%coef, profiles, profiles_ad, &
                                           traj0_sta%angles, chanprof, radiancedata_ad)
      ENDIF
    ENDIF
  ENDIF

!--------------------------------------------------
!1. AD of radiative transfer integration - USER levs
!--------------------------------------------------
  addcosmic = (coefs%coef%id_sensor == sensor_id_mw .OR. coefs%coef%id_sensor == sensor_id_po)

  CALL rttov_integrate_ad( &
        & addcosmic,                                 &
        & opts,                                      &
        & traj0_dyn%nstreams,                        &
        & chanprof,                                  &
        & emissivity,                                &
        & emissivity_ad,                             &
        & reflectance,                               &
        & reflectance_ad,                            &
        & traj0_sta%refl_norm,                       &
        & traj0%thermrefl,                           &
        & traj0_ad%thermrefl,                        &
        & traj0_sta%do_lambertian,                   &
        & traj0_sta%thermal,                         &
        & traj0_sta%dothermal,                       &
        & traj0_sta%solar,                           &
        & traj0_sta%dosolar,                         &
        & traj0_sta%solar_spec_esd,                  &
        & traj0_dyn%transmission_aux,                &
        & traj0_ad_dyn%transmission_aux,             &
        & traj0%transmission_scatt_ir,               &
        & traj0_ad%transmission_scatt_ir,            &
        & profiles,                                  &
        & profiles_ad,                               &
        & traj0%aux_prof,                            &
        & traj0_ad%aux_prof,                         &
        & coefs%coef,                                &
        & traj0%raytracing,                          &
        & traj0_ad%raytracing,                       &
        & traj0%ircld,                               &
        & traj0_ad%ircld,                            &
        & radiancedata,                              &
        & traj0_sta%auxrad,                          &
        & traj0_dyn%auxrad_stream,                   &
        & traj0_ad_dyn%auxrad_stream,                &
        & radiancedata_ad)! inout  (output if converstion Bt -> rad)

!--------------------------------------------------------
!2. AD of channel reflectances and emissivities - SURFACE
!--------------------------------------------------------

  IF (traj0_sta%dosolar) THEN
  
    IF (ANY(calcrefl)) THEN
    
      CALL rttov_calcsurfrefl_ad(     &
            & coefs%coef,             &
            & profiles,               &
            & traj0%sunglint,         &
            & traj0_ad%sunglint,      &
            & traj0%fresnrefl,        &
            & traj0_ad%fresnrefl,     &
            & traj0_sta%solar,        &
            & chanprof,               &
            & traj0_sta%refl_norm,    &
            & calcrefl,               &
            & emissivity,             &
            & emissivity_ad,          &
            & reflectance_ad%refl_out)
  
      CALL rttov_fresnel_ad( &
            & chanprof,           &
            & profiles,           &
            & traj0_sta%solar,    &
            & coefs%coef,         &
            & traj0%sunglint,     &
            & traj0_ad%sunglint,  &
            & traj0%fresnrefl,    &
            & traj0_ad%fresnrefl)
      CALL rttov_refsun_ad( &
            & profiles,            &
            & profiles_ad,         &
            & coefs%coef,          &
            & traj0%aux_prof,      &
            & traj0%sunglint,      &
            & traj0_ad%sunglint,   &
            & traj0%raytracing,    &
            & traj0_ad%raytracing)
    
    ENDIF
    
    reflectance_ad%refl_in = reflectance_ad%refl_in + reflectance_ad%refl_out
    
  ENDIF


  IF (traj0_sta%dothermal) THEN
  
    IF (ANY(calcemis) .OR. opts%rt_ir%pc%addpc) THEN
      ! Calculate surface emissivity and traj0%thermrefl for selected channels
  
      IF (coefs%coef%id_sensor == sensor_id_ir) THEN
        ! Infrared
        emissivity_ad(:)%emis_out = emissivity_ad(:)%emis_out - traj0_ad%thermrefl(:)
      ELSE IF (coefs%coef%id_sensor == sensor_id_mw .OR. coefs%coef%id_sensor == sensor_id_po) THEN
        ! Microwave
        CALL rttov_calcemis_mw_ad( &
              & opts,                               &
              & profiles,                           &
              & profiles_ad,                        &
              & traj0_sta%angles,                   &
              & coefs%coef,                         &
              & chanprof,                           &
              & traj0_dyn%transmission_aux,         &
              & traj0_ad_dyn%transmission_aux,      &
              & calcemis,                           &
              & emissivity_ad%emis_out,             &
              & traj0_ad%thermrefl)
      ELSE
        ! Hires
        emissivity_ad(:)%emis_out = emissivity_ad(:)%emis_out - traj0_ad%thermrefl(:)
        CALL rttov_calcemis_ir_ad(    &
              & profiles,             &
              & profiles_ad,          &
              & coefs%coef,           &
              & opts%rt_ir%pc%addpc,  &
              & coefs%coef_pccomp,    &
              & traj0_sta%thermal,    &
              & chanprof,             &
              & calcemis,             &
              & emissivity_ad%emis_out)
      ENDIF

    ENDIF

    IF (.NOT. opts%rt_ir%pc%addpc) THEN
      WHERE (.NOT. calcemis)
        emissivity_ad%emis_out = emissivity_ad%emis_out - traj0_ad%thermrefl
      ENDWHERE
    ENDIF

    emissivity_ad%emis_in = emissivity_ad%emis_in + emissivity_ad%emis_out

  ENDIF

!------------------------------------
!3. AD of transmittances - USER levs
!------------------------------------

  IF (traj0_sta%dosolar) THEN
    CALL rttov_transmit_9_solar_ad( &
          & opts%rt_ir%addaerosl,                      &
          & opts%rt_ir%addclouds,                      &
          & profiles(1)%nlayers,                       &
          & nprofiles,                                 &
          & chanprof,                                  &
          & traj0_sta%solar,                           &
          & traj0%aux_prof,                            &
          & traj0_ad%aux_prof,                         &
          & coefs%coef,                                &
          & traj0%raytracing,                          &
          & traj0_ad%raytracing,                       &
          & traj0%ircld,                               &
          & traj0%opdp_path,                           &
          & traj0_ad%opdp_path,                        &
          & traj0_sta%solar_path2,                     &
          & traj0_sta%solar_path1,                     &
          & transmission,                              &
          & transmission_ad,                           &
          & traj0_dyn%transmission_aux,                &
          & traj0_ad_dyn%transmission_aux,             &
          & traj0%transmission_scatt_ir,               &
          & traj0_ad%transmission_scatt_ir,            &
          & traj0_dyn%transmission_scatt_ir_stream,    &
          & traj0_ad_dyn%transmission_scatt_ir_stream)
  ENDIF

  IF (traj0_sta%dothermal) THEN
    CALL rttov_transmit_ad( &
          & opts%rt_ir%addaerosl,                        &
          & opts%rt_ir%addclouds,                        &
          & traj0_sta%do_lambertian,                     &
          & profiles(1)%nlayers,                         &
          & chanprof,                                    &
          & traj0_sta%thermal,                           &
          & traj0%aux_prof,                              &
          & traj0_ad%aux_prof,                           &
          & coefs%coef,                                  &
          & traj0%ircld,                                 &
          & traj0_sta%angles,                            &
          & traj0%opdp_path%atm_level,                   &
          & traj0_ad%opdp_path%atm_level,                &
          & traj0_sta%thermal_path1%od_level,            &
          & transmission%tau_levels,                     &
          & transmission%tau_total,                      &
          & transmission_ad%tau_levels,                  &
          & transmission_ad%tau_total,                   &
          & traj0_dyn%transmission_aux,                  &
          & traj0_dyn%transmission_aux%thermal_path1,    &
          & traj0_ad_dyn%transmission_aux%thermal_path1, &
          & traj0%transmission_scatt_ir,                 &
          & traj0_ad%transmission_scatt_ir,              &
          & traj0_dyn%transmission_scatt_ir_stream,      &
          & traj0_ad_dyn%transmission_scatt_ir_stream,   &
          & traj0_sta%thermal_path1%tau_ref,             &
          & traj0_sta%thermal_path1%tau_ref_surf,        &
          & traj0_sta%thermal_path1%tau_surf,            &
          & traj0_sta%thermal_path1%tau_level)
  ENDIF
!-------------------------------------------------------------
!4. AD of optical depths of aerosols and/or clouds - USER levs
!-------------------------------------------------------------

  IF (opts%rt_ir%addaerosl .OR. opts%rt_ir%addclouds) THEN
    CALL rttov_opdpscattir_ad( &
          & profiles(1)%nlayers,                       &
          & chanprof,                                  &
          & opts,                                      &
          & traj0%aux_prof,                            &
          & traj0_ad%aux_prof,                         &
          & profiles,                                  &
          & profiles_ad,                               &
          & aer_opt_param,                             &
          & cld_opt_param,                             &
          & traj0_sta%dosolar,                         &
          & traj0_sta%solar,                           &
          & coefs%coef,                                &
          & coefs%coef_scatt_ir,                       &
          & traj0%raytracing,                          &
          & traj0_ad%raytracing,                       &
          & traj0%transmission_scatt_ir,               &
          & traj0_ad%transmission_scatt_ir,            &
          & traj0_dyn%transmission_scatt_ir_stream,    &
          & traj0_ad_dyn%transmission_scatt_ir_stream, &
          & coefs%optp,                                &
          & traj0%ircld,                               &
          & traj0_ad%ircld)
  ENDIF

!----------------------------------------------------
!5. AD of cloud streams and distributions - USER levs
!----------------------------------------------------

  IF (opts%rt_ir%addclouds) THEN
    CALL rttov_cldstr_ad( &
          & opts%rt_ir,     &
          & profiles,       &
          & profiles_ad,    &
          & traj0%ircld,    &
          & traj0_ad%ircld)
  ENDIF

!------------------------------------
!6. AD of optical depth interpolation
!------------------------------------
 
  IF (opts%interpolation%spacetop) THEN
    traj0_ad%opdp_path%atm_level(1,:) = 0._jprb
    IF (opts%rt_ir%addsolar) THEN
      traj0_ad%opdp_path%sun_level_path2(1,:) = 0._jprb
    ENDIF
  ENDIF

  IF (opts%interpolation%addinterp) THEN
    CALL rttov_intavg_chan_ad( &
          & opts,                           &
          & traj0_sta%thermal,              &
          & traj0_sta%solar,                &
          & coefs%coef%nlevels,             &
          & nlevels,                        &
          & chanprof,                       &
          & traj0%profiles_coef,            &
          & profiles,                       &
          & profiles_ad,                    &
          & traj0%opdp_path_coef,           &
          & traj0_ad%opdp_path_coef,        &
          & traj0%opdp_path,                &
          & traj0_ad%opdp_path)
  ELSE
    CALL rttov_add_opdp_path(opts, traj0_ad%opdp_path_coef, traj0_ad%opdp_path_coef, traj0_ad%opdp_path)
  ENDIF

!! testop omitted
!---------------------------------------------------------
!7. AD of atmospheric and solar optical depths - COEF levs
!---------------------------------------------------------
! optical depth arrays allocated at start of AD-code

  IF (coefs%coef%fmv_model_ver == 9) THEN
    IF (traj0_sta%dosolar) THEN
      ! Calculate solar path2 optical depths for solar channels
      CALL rttov_opdep_9_ad( &
            & coefs%coef%nlayers,                      &
            & chanprof,                                &
            & traj0_sta%solar,                         &
            & traj0%predictors,                        &
            & traj0%predictors%path2,                  &
            & traj0_ad%predictors%path2,               &
            & traj0%aux_prof_coef,                     &
            & traj0_ad%aux_prof_coef,                  &
            & coefs%coef,                              &
            & coefs%coef%solar,                        &
            & traj0%opdp_path_coef%sun_level_path2,    &
            & traj0_ad%opdp_path_coef%sun_level_path2, &
            & traj0_sta%solar_path2%opdp_ref_coef)
    ENDIF
    IF (traj0_sta%dothermal) THEN
      ! Calculate thermal path1 optical depths for thermal channels
      CALL rttov_opdep_9_ad( &
            & coefs%coef%nlayers,                  &
            & chanprof,                            &
            & traj0_sta%thermal,                   &
            & traj0%predictors,                    &
            & traj0%predictors%path1,              &
            & traj0_ad%predictors%path1,           &
            & traj0%aux_prof_coef,                 &
            & traj0_ad%aux_prof_coef,              &
            & coefs%coef,                          &
            & coefs%coef%thermal,                  &
            & traj0%opdp_path_coef%atm_level,      &
            & traj0_ad%opdp_path_coef%atm_level,   &
            & traj0_sta%thermal_path1%opdp_ref_coef)
    ENDIF
  ELSE
    CALL rttov_opdep_ad( &
          & coefs%coef%nlayers,        &
          & chanprof,                  &
          & traj0%predictors,          &
          & traj0%predictors%path1,    &
          & traj0_ad%predictors%path1, &
          & traj0%aux_prof_coef,       &
          & traj0_ad%aux_prof_coef,    &
          & coefs%coef,                &
          & coefs%coef%thermal,        &
          & traj0%opdp_path_coef,      &
          & traj0_ad%opdp_path_coef,   &
          & traj0_sta%thermal_path1%opdp_ref_coef)
  ENDIF

!-------------------------------------------------------
!8. AD of RTTOV-7 RTTOV-8 RTTOV-9 predictors - COEF levs
!-------------------------------------------------------

  IF (coefs%coef%fmv_model_ver == 7) THEN
    CALL rttov_setpredictors_7_ad( &
          & opts,                      &
          & traj0%profiles_coef,       &
          & traj0_ad%profiles_coef,    &
          & traj0_sta%angles_coef,     &
          & coefs%coef,                &
          & traj0%aux_prof_coef,       &
          & traj0_ad%aux_prof_coef,       &
          & traj0%predictors%path1,    &
          & traj0_ad%predictors%path1, &
          & traj0%raytracing_coef,     &
          & traj0_ad%raytracing_coef)
  ELSE IF (coefs%coef%fmv_model_ver == 8) THEN
    CALL rttov_setpredictors_8_ad( &
          & opts,                      &
          & traj0%profiles_coef,       &
          & traj0_ad%profiles_coef,    &
          & traj0_sta%angles_coef,     &
          & coefs%coef,                &
          & traj0%aux_prof_coef,       &
          & traj0%predictors%path1,    &
          & traj0_ad%predictors%path1, &
          & traj0%raytracing_coef,     &
          & traj0_ad%raytracing_coef)
  ELSE IF (coefs%coef%fmv_model_ver == 9) THEN
    IF (traj0_sta%dosolar) THEN
      CALL rttov_setpredictors_9_ad(            &
            & opts,                             &
            & traj0%profiles_coef,              &
            & traj0_ad%profiles_coef,           &
            & traj0_sta%angles_coef,            &
            & traj0%raytracing_coef%patheff,    &
            & traj0_ad%raytracing_coef%patheff, &
            & coefs%coef_pccomp,                &
            & coefs%coef,                       &
            & traj0%predictors%path2,           &
            & traj0_ad%predictors%path2)
    ENDIF
    IF (traj0_sta%dothermal) THEN
      CALL rttov_setpredictors_9_ad(            &
            & opts,                             &
            & traj0%profiles_coef,              &
            & traj0_ad%profiles_coef,           &
            & traj0_sta%angles_coef,            &
            & traj0%raytracing_coef%pathsat,    &
            & traj0_ad%raytracing_coef%pathsat, &
            & coefs%coef_pccomp,                &
            & coefs%coef,                       &
            & traj0%predictors%path1,           &
            & traj0_ad%predictors%path1)
    ENDIF
  ENDIF

!-------------------------------------------------------
!9. AD of common geometric set-up for the RT integration
!-------------------------------------------------------

!9.1 USER levs
!-------------
  IF (opts%interpolation%addinterp) THEN
    CALL rttov_setgeometry_ad( &
      & opts,                &
      & traj0_sta%dosolar,   &
      & profiles,            &
      & profiles_ad,         &
      & traj0%aux_prof,      &
      & coefs%coef,          &
      & traj0_sta%angles,    &
      & traj0%raytracing,    &
      & traj0_ad%raytracing)
  ELSE
    CALL rttov_add_raytracing(traj0_sta%dosolar, traj0_ad%raytracing_coef, &
                              traj0_ad%raytracing_coef, traj0_ad%raytracing)
  ENDIF

!9.1 COEF levs
!-------------
  CALL rttov_setgeometry_ad( &
    & opts,                     &
    & traj0_sta%dosolar,        &
    & traj0%profiles_coef,      &
    & traj0_ad%profiles_coef,   &
    & traj0%aux_prof_coef,      &
    & coefs%coef,               &
    & traj0_sta%angles_coef,    &
    & traj0%raytracing_coef,    &
    & traj0_ad%raytracing_coef)

!---------------------------------------------------------
!10. AD of cloud top, surface levels, ice cloud parameters
!---------------------------------------------------------
!10.2 USER levs
!-------------
  IF (opts%interpolation%addinterp .or. opts%rt_ir%addclouds) THEN
    CALL rttov_profaux_ad( &
      & opts,              &
      & profiles,          &
      & profiles_ad,       &
      & coefs%coef,        &
      & traj0%aux_prof,    &
      & traj0_ad%aux_prof)
  ELSE
    CALL rttov_add_aux_prof(traj0_ad%aux_prof_coef, traj0_ad%aux_prof_coef, traj0_ad%aux_prof)
  ENDIF

!10.1 COEF levs
!-------------
    CALL rttov_profaux_ad( &
          & opts_coef,              &
          & traj0%profiles_coef,    &
          & traj0_ad%profiles_coef, &
          & coefs%coef,             &
          & traj0%aux_prof_coef,    &
          & traj0_ad%aux_prof_coef)


!------------------------------------------------------------------
!11 AD of check input data is within suitable physical limits - COEF levs
!------------------------------------------------------------------

  IF (opts%config%apply_reg_limits .OR. opts%interpolation%reg_limit_extrap) THEN
    CALL rttov_apply_reg_limits_ad( &
          & opts,                        &
          & profiles,                    &
          & traj0_sta%profiles_coef_ref, &
          & traj0_ad%profiles_coef,      &
          & coefs%coef,                  &
          & coefs%coef_pccomp)
  ENDIF

!-----------------------------------------------------------------
!12. AD of profile variable interpolation - COEF levs to USER levs
!-----------------------------------------------------------------
! intvar logical array set by direct code

  IF (opts%interpolation%addinterp) THEN
    CALL rttov_intavg_prof_ad( &
          & opts,                           &
          & nlevels,                        &
          & coefs%coef%nlevels,             &
          & profiles,                       &
          & profiles_ad,                    &
          & traj0%profiles_coef,            &
          & traj0_ad%profiles_coef,         &
          & coefs%coef,                     &
          & coefs%coef_pccomp)
  ELSE
! COEF levs same as USER levs
    CALL rttov_add_prof( &
          & profiles_ad,            &
          & profiles_ad,            &
          & traj0_ad%profiles_coef, &
          & lair = .TRUE._jplm,     &
          & lground = .FALSE._jplm)
  ENDIF

  CALL rttov_add_prof( &
        & profiles_ad,            &
        & profiles_ad,            &
        & traj0_ad%profiles_coef, &
        & lair = .FALSE._jplm,    &
        & lground = .TRUE._jplm)
!-----------------------------------------
!13. deallocate memory for local variables
!-----------------------------------------

  CALL cleanup()

  IF (LHOOK) CALL DR_HOOK('RTTOV_AD', 1_jpim, ZHOOK_HANDLE)
  CATCH_C
  errorstatus = ERR
  IF (LHOOK) CALL DR_HOOK('RTTOV_AD', 1_jpim, ZHOOK_HANDLE)

CONTAINS

  SUBROUTINE cleanup()
    INTEGER(KIND=jpim) :: error

    IF ( ASSOCIATED(traj0_dyn%transmission_aux%fac1) ) THEN
      CALL rttov_alloc_traj_dyn (error, traj0_dyn, opts, nchannels, profiles(1)%nlayers, &
                                 traj0_dyn%nstreams, ncldtyp, 0_jpim)
    ENDIF

    IF ( ltraj_ad_dyn_dealloc ) THEN
      CALL rttov_alloc_traj_dyn (error, traj0_ad_dyn, opts, nchannels, profiles(1)%nlayers, &
                                 traj0_dyn%nstreams, ncldtyp, 0_jpim)
    ENDIF

    IF ( ASSOCIATED(traj0_sta%thermal) ) THEN
      CALL rttov_alloc_traj_sta (error, traj0_sta, opts, coefs%coef, nlevels, nchannels, nprofiles, &
                                 0_jpim, npcscores, channels_rec)
    ENDIF

    IF ( ASSOCIATED(traj0) ) THEN
      CALL rttov_check_traj( &
            & error,               &
            & nprofiles,           &
            & nchannels,           &
            & opts,                &
            & nlevels,             &
            & coefs,               &
            & 0_jpim,              &
            & traj0 = traj0,       &
            & traj0_ad = traj0_ad, &
            & traj1 = traj1,       &
            & traj1_ad = traj1_ad, &
            & traj2 = traj,        &
            & traj2_ad = traj_ad)
    ENDIF
  END SUBROUTINE cleanup

END SUBROUTINE rttov_ad
