!
Subroutine rttov_user_options_checkinput( &
       & err,                &
       & opts,               &
       & coefs               )
  ! Description:
  ! Check input options for inconsistencies
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method: Check input profiles with fixed limits specified
  !         in constants and coeff file.
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0      21/11/2011  J Hocking
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:

!INTF_OFF
#include "throw.h"
!INTF_ON

  ! Imported Type Definitions:
  Use rttov_types, Only :      &
         & rttov_coefs       , &
         & rttov_options
  
  Use parkind1, Only : &
      jpim

!INTF_OFF

Use yomhook, Only : &
    LHOOK,&
    DR_HOOK
Use parkind1, Only : &
    jprb
Use rttov_const, Only : &
    sensor_id_ir, &
    sensor_id_hi, &
    sensor_id_mw, &
    sensor_id_po, &
    ninterp_modes
!INTF_ON

  Implicit None

  ! subroutine arguments
  ! scalar arguments with intent(in):
  Type(rttov_options),Intent (in) :: opts                            ! rttov options
  Type(rttov_coefs),  Intent (in) :: coefs                           ! coefficients

  ! scalar arguments with intent(out):
  Integer(Kind=jpim), Intent (out) :: ERR                            ! return code

!INTF_END

#include "rttov_errorreport.interface"


  !local variables:
  Character(len=256) :: msg
  
REAL(KIND=JPRB) :: ZHOOK_HANDLE

  !- End of header --------------------------------------------------------
TRY

IF (LHOOK) CALL DR_HOOK('RTTOV_USER_OPTIONS_CHECKINPUT',0_jpim,ZHOOK_HANDLE)

IF (opts%interpolation%addinterp) THEN
  IF (opts%interpolation%interp_mode < 1 .OR. &
      opts%interpolation%interp_mode > ninterp_modes) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'Error in specified interpolation mode'
    THROWM( ERR .NE. 0 , msg)
  ENDIF
ELSE
  IF (opts%interpolation%lgradp) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'Interpolation should be enabled if LGRADP is set to TRUE'
    THROWM( ERR .NE. 0 , msg)
  ENDIF
ENDIF

IF (opts%rt_ir%ozone_data .AND. coefs%coef%nozone == 0_jpim) THEN
  ERR = ERRORSTATUS_FATAL
  msg = 'Coefficient file does not allow variable O3'
  THROWM( ERR .NE. 0 , msg)
ENDIF

IF (opts%rt_ir%co2_data .AND. coefs%coef%nco2 == 0_jpim) THEN
  ERR = ERRORSTATUS_FATAL
  msg = 'Coefficient file does not allow variable CO2'
  THROWM( ERR .NE. 0 , msg)
ENDIF

IF (opts%rt_ir%co_data .AND. coefs%coef%nco == 0_jpim) THEN
  ERR = ERRORSTATUS_FATAL
  msg = 'Coefficient file does not allow variable CO'
  THROWM( ERR .NE. 0 , msg)
ENDIF

IF (opts%rt_ir%n2o_data .AND. coefs%coef%nn2o == 0_jpim) THEN
  ERR = ERRORSTATUS_FATAL
  msg = 'Coefficient file does not allow variable N2O'
  THROWM( ERR .NE. 0 , msg)
ENDIF

IF (opts%rt_ir%ch4_data .AND. coefs%coef%nch4 == 0_jpim) THEN
  ERR = ERRORSTATUS_FATAL
  msg = 'Coefficient file does not allow variable CH4'
  THROWM( ERR .NE. 0 , msg)
ENDIF


IF (opts%rt_ir%addsolar .AND. coefs%coef%fmv_model_ver /= 9) THEN
  ERR = ERRORSTATUS_FATAL
  msg = 'Coefficient file does not support solar calculations'
  THROWM( ERR .NE. 0 , msg)
ENDIF


! All IR instruments
IF (coefs%coef%id_sensor == sensor_id_ir .OR. coefs%coef%id_sensor == sensor_id_hi) THEN
  IF (opts%rt_mw%clw_data) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'Cloud liquid water only applicable to MW instruments'
    THROWM( ERR .NE. 0 , msg)
  ENDIF

  IF (opts%rt_mw%do_lambertian) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'do_lambertian option only applicable to MW instruments'
    THROWM( ERR .NE. 0 , msg)
  ENDIF
ENDIF


! MW instruments
IF (coefs%coef%id_sensor == sensor_id_mw .OR. coefs%coef%id_sensor == sensor_id_po) THEN
  IF (opts%rt_ir%addsolar) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'Solar calculations not applicable to MW instruments'
    THROWM( ERR .NE. 0 , msg)
  ENDIF
  IF (opts%rt_ir%addclouds) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'addclouds not applicable to MW instruments: use rttov_scatt interface instead'
    THROWM( ERR .NE. 0 , msg)
  ENDIF
  IF (opts%rt_ir%addaerosl) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'Aerosol calculations not applicable to MW instruments'
    THROWM( ERR .NE. 0 , msg)
  ENDIF
ENDIF


! NLTE
IF (opts%rt_ir%do_nlte_correction) THEN
  IF (.NOT. coefs%coef%nltecoef) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'Coefficient file does not support NLTE correction or channel selection does not include any NLTE-affected channels'
    THROWM( ERR .NE. 0 , msg)
  ENDIF
ENDIF


! PC-RTTOV
IF (opts%rt_ir%pc%addpc) THEN
  IF (coefs%coef%id_comp_pc == 0_jpim) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'PC-RTTOV: optical depth coefficient file is not compatible with PCs'
    THROWM( ERR .NE. 0 , msg)
  ENDIF

  IF (opts%rt_ir%addclouds .AND. coefs%coef_pccomp%fmv_pc_cld == 0_jpim) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'PC-RTTOV: addclouds is TRUE, but PC coef file is not compatible with cloudy simulations'
    THROWM( ERR .NE. 0 , msg)
  ENDIF

  IF (opts%rt_ir%pc%ipcbnd < 1_jpim .OR. &
    & opts%rt_ir%pc%ipcbnd > coefs%coef_pccomp%fmv_pc_bands) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'PC-RTTOV: invalid spectral band (opts%rt_ir%pc%ipcbnd)'
    THROWM( ERR .NE. 0 , msg)
  ENDIF

  IF (opts%rt_ir%pc%ipcreg < 1_jpim .OR. &
    & opts%rt_ir%pc%ipcreg > coefs%coef_pccomp%fmv_pc_sets(opts%rt_ir%pc%ipcbnd)) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'PC-RTTOV: invalid predictor set (opts%rt_ir%pc%ipcreg) for selected spectral band (opts%rt_ir%pc%ipcbnd)'
    THROWM( ERR .NE. 0 , msg)
  ENDIF

  IF (opts%rt_all%switchrad .AND. .NOT. opts%rt_ir%pc%addradrec) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'PC-RTTOV: switchrad will have no effect unless addradrec is TRUE'
    THROWM( ERR .NE. 0 , msg)
  ENDIF

  IF (opts%rt_ir%addaerosl) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'PC-RTTOV: cannot be run with aerosol profiles'
    THROWM( ERR .NE. 0 , msg)
  ENDIF

  IF (opts%rt_ir%addclouds .AND. opts%rt_ir%user_cld_opt_param) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'PC-RTTOV: cannot be run with user input cloud optical parameters'
    THROWM( ERR .NE. 0 , msg)
  ENDIF

  IF (opts%rt_ir%addsolar) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'PC-RTTOV: cannot be run with addsolar TRUE'
    THROWM( ERR .NE. 0 , msg)
  ENDIF

  IF (opts%rt_ir%do_nlte_correction) THEN
    ERR = ERRORSTATUS_FATAL
    msg = 'PC-RTTOV: cannot be run with do_nlte_correction TRUE'
    THROWM( ERR .NE. 0 , msg)
  ENDIF
ENDIF


IF (LHOOK) CALL DR_HOOK('RTTOV_USER_OPTIONS_CHECKINPUT',1_jpim,ZHOOK_HANDLE)

CATCH

IF (LHOOK) CALL DR_HOOK('RTTOV_USER_OPTIONS_CHECKINPUT',1_jpim,ZHOOK_HANDLE)
End Subroutine rttov_user_options_checkinput
