!
MODULE mod_brdf_atlas
  ! Description:
  !   Data and routines for visible to nir brdf atlas.
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2012, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0      03/01/2012  Based on UW IR atlas code
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  !
#include "throw.h"

  USE parkind1, Only : jpim, jprb, jplm

  USE mod_rttov_brdf_atlas, Only : brdf_atlas_single_inst
  
  ! Disable implicit typing
  IMPLICIT NONE

#include "rttov_errorreport.interface"

  INCLUDE 'netcdf.inc'

  ! Specify kinds for NetCDF interface
  INTEGER, PARAMETER :: ncint32  = SELECTED_INT_KIND(9)
  INTEGER, PARAMETER :: ncint16  = SELECTED_INT_KIND(4)
  INTEGER, PARAMETER :: ncint8   = SELECTED_INT_KIND(2)
  INTEGER, PARAMETER :: ncreal32 = SELECTED_REAL_KIND(6,37)
  
  ! User can specify atlas version at set-up. Version 100 is the default.
  INTEGER(KIND=jpim) :: vn_atlas_version=100   ! Version of atlas need to be verify
  
  ! Atlas constants
    
  INTEGER(KIND=jpim), PARAMETER :: numpcs=6
  INTEGER(KIND=jpim), PARAMETER :: hngpnts=7
  INTEGER(KIND=jpim), PARAMETER :: numwave=2101

  INTEGER(KIND=jpim), PARAMETER :: nb_lats=1800
  INTEGER(KIND=jpim), PARAMETER :: nb_lons=3600
  INTEGER(KIND=jpim), PARAMETER :: nb_pack=2250931  !ver2.1  2007

  INTEGER(KIND=jpim), PARAMETER :: db_ver_year=2007
  
  REAL(KIND=jprb),    PARAMETER :: default_std = 0.05_JPRB ! default standard deviation

  INTEGER(KIND=jpim), PARAMETER :: brdfmodis_gridres = 100_JPRB     ! 0.1 deg
  INTEGER(KIND=jpim), PARAMETER :: brdfmodis_ygrid1 = 89950_JPRB    ! 89.95 deg
  INTEGER(KIND=jpim), PARAMETER :: brdfmodis_xgrid1 = -179950_JPRB  ! -179.95 deg
  
  ! Atlas data loaded by initialisation routine
  
  INTEGER(KIND=ncint16), ALLOCATABLE :: brdfmodis_flag(:,:)  ! dims are (nb_lats,nb_lons)
  INTEGER(KIND=jpim),    ALLOCATABLE :: brdfmodis_lut(:,:)   ! dims are (nb_lats,nb_lons)
  Integer(Kind=ncint16), ALLOCATABLE :: brdfmodis(:,:,:)     ! dims are (nb_kernel,nb_pack,hngpnts)

  REAL(KIND=jprb)     :: D(hngpnts,numpcs)
  REAL(KIND=ncreal32) :: pcu(numpcs,numwave)              ! pcu need to be read in from flat file
  
  ! Data to allow more efficient memory usage (convert integers to reals only at point of use)
  
  REAL(KIND=ncreal32) :: sfac_fiso(hngpnts) ! iso parameter scale factors
  REAL(KIND=ncreal32) :: offs_fiso(hngpnts) ! iso parameter offsets
  REAL(KIND=ncreal32) :: sfac_fvol(hngpnts) ! vol parameter scale factors
  REAL(KIND=ncreal32) :: offs_fvol(hngpnts) ! vol parameter offsets
  REAL(KIND=ncreal32) :: sfac_fgeo(hngpnts) ! geo parameter scale factors
  REAL(KIND=ncreal32) :: offs_fgeo(hngpnts) ! geo parameter offsets
  
  ! Atlas data contained in this file
  
  INTEGER(KIND=jpim) :: i
  REAL(KIND=jprb) :: snow_ref(numwave)
  REAL(KIND=jprb) :: coastal_waters_ref(numwave),ocean_waters_ref(numwave)
  REAL(KIND=jprb) :: pcm(numwave)
  REAL(KIND=jprb) :: hsr_wavenum(numwave)
  REAL(KIND=jprb) :: pcu_modres(hngpnts,hngpnts)
  REAL(KIND=jprb) :: pcm_modres(hngpnts)

  ! Arrays to hold hsr data interpolated onto instrument wavenumbers

  INTEGER(KIND=jpim) :: ncoefchans
  REAL(KIND=jprb), ALLOCATABLE :: snow_ref_int(:)
  REAL(KIND=jprb), ALLOCATABLE :: coastal_waters_ref_int(:), ocean_waters_ref_int(:)
  REAL(KIND=jprb), ALLOCATABLE :: pcu_int(:,:)
  REAL(KIND=jprb), ALLOCATABLE :: pcm_int(:)

  DATA pcm_modres / & 
  39.8051262_JPRB,  49.6526299_JPRB,  52.5679855_JPRB,  49.9688873_JPRB,  39.3762932_JPRB,  &
  36.0293274_JPRB,  28.9127731_JPRB /

!   number of PC is the first dimension and spectra is the second for pcu_modres (eigenvectors) 
  DATA pcu_modres / &
  -0.0153509_JPRB,  -0.0174285_JPRB,  -0.0550101_JPRB,  -0.0456830_JPRB,  -0.0492308_JPRB,  &
  -0.0334872_JPRB,   0.0433589_JPRB,  -0.0207704_JPRB,   0.0020112_JPRB,  -0.0397478_JPRB,  &
  -0.0045270_JPRB,   0.0223504_JPRB,   0.0344748_JPRB,   0.0374810_JPRB,  -0.0216471_JPRB,  &
   0.0288548_JPRB,  -0.0150199_JPRB,   0.0039704_JPRB,   0.0347403_JPRB,  -0.0112829_JPRB,  &
   0.0245785_JPRB,  -0.0213760_JPRB,   0.0332810_JPRB,   0.0133695_JPRB,  -0.0124754_JPRB,  &
  -0.0246968_JPRB,   0.0084204_JPRB,  -0.0061599_JPRB,  -0.0256441_JPRB,  -0.0138075_JPRB,  &
  -0.0049740_JPRB,   0.0402420_JPRB,  -0.0040104_JPRB,  -0.0184281_JPRB,  -0.0085442_JPRB,  &
  -0.0232838_JPRB,  -0.0140538_JPRB,   0.0088426_JPRB,   0.0090562_JPRB,  -0.0227179_JPRB,  &
   0.0317618_JPRB,   0.0141060_JPRB,  -0.0217532_JPRB,  -0.0192157_JPRB,   0.0172922_JPRB,  &
  -0.0152554_JPRB,   0.0092314_JPRB,   0.0020664_JPRB,  -0.0013151_JPRB /

  DATA hsr_wavenum / &
  4000.0_JPRB,  4010.0_JPRB,  4020.0_JPRB,  4030.0_JPRB,  4040.0_JPRB,  4050.0_JPRB,  4060.0_JPRB,  &
  4070.0_JPRB,  4080.0_JPRB,  4090.0_JPRB,  4100.0_JPRB,  4110.0_JPRB,  4120.0_JPRB,  4130.0_JPRB,  &
  4140.0_JPRB,  4150.0_JPRB,  4160.0_JPRB,  4170.0_JPRB,  4180.0_JPRB,  4190.0_JPRB,  4200.0_JPRB,  &
  4210.0_JPRB,  4220.0_JPRB,  4230.0_JPRB,  4240.0_JPRB,  4250.0_JPRB,  4260.0_JPRB,  4270.0_JPRB,  &
  4280.0_JPRB,  4290.0_JPRB,  4300.0_JPRB,  4310.0_JPRB,  4320.0_JPRB,  4330.0_JPRB,  4340.0_JPRB,  &
  4350.0_JPRB,  4360.0_JPRB,  4370.0_JPRB,  4380.0_JPRB,  4390.0_JPRB,  4400.0_JPRB,  4410.0_JPRB,  &
  4420.0_JPRB,  4430.0_JPRB,  4440.0_JPRB,  4450.0_JPRB,  4460.0_JPRB,  4470.0_JPRB,  4480.0_JPRB,  &
  4490.0_JPRB,  4500.0_JPRB,  4510.0_JPRB,  4520.0_JPRB,  4530.0_JPRB,  4540.0_JPRB,  4550.0_JPRB,  &
  4560.0_JPRB,  4570.0_JPRB,  4580.0_JPRB,  4590.0_JPRB,  4600.0_JPRB,  4610.0_JPRB,  4620.0_JPRB,  &
  4630.0_JPRB,  4640.0_JPRB,  4650.0_JPRB,  4660.0_JPRB,  4670.0_JPRB,  4680.0_JPRB,  4690.0_JPRB,  &
  4700.0_JPRB,  4710.0_JPRB,  4720.0_JPRB,  4730.0_JPRB,  4740.0_JPRB,  4750.0_JPRB,  4760.0_JPRB,  &
  4770.0_JPRB,  4780.0_JPRB,  4790.0_JPRB,  4800.0_JPRB,  4810.0_JPRB,  4820.0_JPRB,  4830.0_JPRB,  &
  4840.0_JPRB,  4850.0_JPRB,  4860.0_JPRB,  4870.0_JPRB,  4880.0_JPRB,  4890.0_JPRB,  4900.0_JPRB,  &
  4910.0_JPRB,  4920.0_JPRB,  4930.0_JPRB,  4940.0_JPRB,  4950.0_JPRB,  4960.0_JPRB,  4970.0_JPRB,  &
  4980.0_JPRB,  4990.0_JPRB,  5000.0_JPRB,  5010.0_JPRB,  5020.0_JPRB,  5030.0_JPRB,  5040.0_JPRB,  &
  5050.0_JPRB,  5060.0_JPRB,  5070.0_JPRB,  5080.0_JPRB,  5090.0_JPRB,  5100.0_JPRB,  5110.0_JPRB,  &
  5120.0_JPRB,  5130.0_JPRB,  5140.0_JPRB,  5150.0_JPRB,  5160.0_JPRB,  5170.0_JPRB,  5180.0_JPRB,  &
  5190.0_JPRB,  5200.0_JPRB,  5210.0_JPRB,  5220.0_JPRB,  5230.0_JPRB,  5240.0_JPRB,  5250.0_JPRB,  &
  5260.0_JPRB,  5270.0_JPRB,  5280.0_JPRB,  5290.0_JPRB,  5300.0_JPRB,  5310.0_JPRB,  5320.0_JPRB,  &
  5330.0_JPRB,  5340.0_JPRB,  5350.0_JPRB,  5360.0_JPRB,  5370.0_JPRB,  5380.0_JPRB,  5390.0_JPRB,  &
  5400.0_JPRB,  5410.0_JPRB,  5420.0_JPRB,  5430.0_JPRB,  5440.0_JPRB,  5450.0_JPRB,  5460.0_JPRB,  &
  5470.0_JPRB,  5480.0_JPRB,  5490.0_JPRB,  5500.0_JPRB,  5510.0_JPRB,  5520.0_JPRB,  5530.0_JPRB,  &
  5540.0_JPRB,  5550.0_JPRB,  5560.0_JPRB,  5570.0_JPRB,  5580.0_JPRB,  5590.0_JPRB,  5600.0_JPRB,  &
  5610.0_JPRB,  5620.0_JPRB,  5630.0_JPRB,  5640.0_JPRB,  5650.0_JPRB,  5660.0_JPRB,  5670.0_JPRB,  &
  5680.0_JPRB,  5690.0_JPRB,  5700.0_JPRB,  5710.0_JPRB,  5720.0_JPRB,  5730.0_JPRB,  5740.0_JPRB,  &
  5750.0_JPRB,  5760.0_JPRB,  5770.0_JPRB,  5780.0_JPRB,  5790.0_JPRB,  5800.0_JPRB,  5810.0_JPRB,  &
  5820.0_JPRB,  5830.0_JPRB,  5840.0_JPRB,  5850.0_JPRB,  5860.0_JPRB,  5870.0_JPRB,  5880.0_JPRB,  &
  5890.0_JPRB,  5900.0_JPRB,  5910.0_JPRB,  5920.0_JPRB,  5930.0_JPRB,  5940.0_JPRB,  5950.0_JPRB,  &
  5960.0_JPRB,  5970.0_JPRB,  5980.0_JPRB,  5990.0_JPRB,  6000.0_JPRB,  6010.0_JPRB,  6020.0_JPRB,  &
  6030.0_JPRB,  6040.0_JPRB,  6050.0_JPRB,  6060.0_JPRB,  6070.0_JPRB,  6080.0_JPRB,  6090.0_JPRB,  &
  6100.0_JPRB,  6110.0_JPRB,  6120.0_JPRB,  6130.0_JPRB,  6140.0_JPRB,  6150.0_JPRB,  6160.0_JPRB,  &
  6170.0_JPRB,  6180.0_JPRB,  6190.0_JPRB,  6200.0_JPRB,  6210.0_JPRB,  6220.0_JPRB,  6230.0_JPRB,  &
  6240.0_JPRB,  6250.0_JPRB,  6260.0_JPRB,  6270.0_JPRB,  6280.0_JPRB,  6290.0_JPRB,  6300.0_JPRB,  &
  6310.0_JPRB,  6320.0_JPRB,  6330.0_JPRB,  6340.0_JPRB,  6350.0_JPRB,  6360.0_JPRB,  6370.0_JPRB,  &
  6380.0_JPRB,  6390.0_JPRB,  6400.0_JPRB,  6410.0_JPRB,  6420.0_JPRB,  6430.0_JPRB,  6440.0_JPRB,  &
  6450.0_JPRB,  6460.0_JPRB,  6470.0_JPRB,  6480.0_JPRB,  6490.0_JPRB,  6500.0_JPRB,  6510.0_JPRB,  &
  6520.0_JPRB,  6530.0_JPRB,  6540.0_JPRB,  6550.0_JPRB,  6560.0_JPRB,  6570.0_JPRB,  6580.0_JPRB,  &
  6590.0_JPRB,  6600.0_JPRB,  6610.0_JPRB,  6620.0_JPRB,  6630.0_JPRB,  6640.0_JPRB,  6650.0_JPRB,  &
  6660.0_JPRB,  6670.0_JPRB,  6680.0_JPRB,  6690.0_JPRB,  6700.0_JPRB,  6710.0_JPRB,  6720.0_JPRB,  &
  6730.0_JPRB,  6740.0_JPRB,  6750.0_JPRB,  6760.0_JPRB,  6770.0_JPRB,  6780.0_JPRB,  6790.0_JPRB,  &
  6800.0_JPRB,  6810.0_JPRB,  6820.0_JPRB,  6830.0_JPRB,  6840.0_JPRB,  6850.0_JPRB,  6860.0_JPRB,  &
  6870.0_JPRB,  6880.0_JPRB,  6890.0_JPRB,  6900.0_JPRB,  6910.0_JPRB,  6920.0_JPRB,  6930.0_JPRB,  &
  6940.0_JPRB,  6950.0_JPRB,  6960.0_JPRB,  6970.0_JPRB,  6980.0_JPRB,  6990.0_JPRB,  7000.0_JPRB,  &
  7010.0_JPRB,  7020.0_JPRB,  7030.0_JPRB,  7040.0_JPRB,  7050.0_JPRB,  7060.0_JPRB,  7070.0_JPRB,  &
  7080.0_JPRB,  7090.0_JPRB,  7100.0_JPRB,  7110.0_JPRB,  7120.0_JPRB,  7130.0_JPRB,  7140.0_JPRB,  &
  7150.0_JPRB,  7160.0_JPRB,  7170.0_JPRB,  7180.0_JPRB,  7190.0_JPRB,  7200.0_JPRB,  7210.0_JPRB,  &
  7220.0_JPRB,  7230.0_JPRB,  7240.0_JPRB,  7250.0_JPRB,  7260.0_JPRB,  7270.0_JPRB,  7280.0_JPRB,  &
  7290.0_JPRB,  7300.0_JPRB,  7310.0_JPRB,  7320.0_JPRB,  7330.0_JPRB,  7340.0_JPRB,  7350.0_JPRB,  &
  7360.0_JPRB,  7370.0_JPRB,  7380.0_JPRB,  7390.0_JPRB,  7400.0_JPRB,  7410.0_JPRB,  7420.0_JPRB,  &
  7430.0_JPRB,  7440.0_JPRB,  7450.0_JPRB,  7460.0_JPRB,  7470.0_JPRB,  7480.0_JPRB,  7490.0_JPRB,  &
  7500.0_JPRB,  7510.0_JPRB,  7520.0_JPRB,  7530.0_JPRB,  7540.0_JPRB,  7550.0_JPRB,  7560.0_JPRB,  &
  7570.0_JPRB,  7580.0_JPRB,  7590.0_JPRB,  7600.0_JPRB,  7610.0_JPRB,  7620.0_JPRB,  7630.0_JPRB,  &
  7640.0_JPRB,  7650.0_JPRB,  7660.0_JPRB,  7670.0_JPRB,  7680.0_JPRB,  7690.0_JPRB,  7700.0_JPRB,  &
  7710.0_JPRB,  7720.0_JPRB,  7730.0_JPRB,  7740.0_JPRB,  7750.0_JPRB,  7760.0_JPRB,  7770.0_JPRB,  &
  7780.0_JPRB,  7790.0_JPRB,  7800.0_JPRB,  7810.0_JPRB,  7820.0_JPRB,  7830.0_JPRB,  7840.0_JPRB,  &
  7850.0_JPRB,  7860.0_JPRB,  7870.0_JPRB,  7880.0_JPRB,  7890.0_JPRB,  7900.0_JPRB,  7910.0_JPRB,  &
  7920.0_JPRB,  7930.0_JPRB,  7940.0_JPRB,  7950.0_JPRB,  7960.0_JPRB,  7970.0_JPRB,  7980.0_JPRB,  &
  7990.0_JPRB,  8000.0_JPRB,  8010.0_JPRB,  8020.0_JPRB,  8030.0_JPRB,  8040.0_JPRB,  8050.0_JPRB,  &
  8060.0_JPRB,  8070.0_JPRB,  8080.0_JPRB,  8090.0_JPRB,  8100.0_JPRB,  8110.0_JPRB,  8120.0_JPRB,  &
  8130.0_JPRB,  8140.0_JPRB,  8150.0_JPRB,  8160.0_JPRB,  8170.0_JPRB,  8180.0_JPRB,  8190.0_JPRB,  &
  8200.0_JPRB,  8210.0_JPRB,  8220.0_JPRB,  8230.0_JPRB,  8240.0_JPRB,  8250.0_JPRB,  8260.0_JPRB,  &
  8270.0_JPRB,  8280.0_JPRB,  8290.0_JPRB,  8300.0_JPRB,  8310.0_JPRB,  8320.0_JPRB,  8330.0_JPRB,  &
  8340.0_JPRB,  8350.0_JPRB,  8360.0_JPRB,  8370.0_JPRB,  8380.0_JPRB,  8390.0_JPRB,  8400.0_JPRB,  &
  8410.0_JPRB,  8420.0_JPRB,  8430.0_JPRB,  8440.0_JPRB,  8450.0_JPRB,  8460.0_JPRB,  8470.0_JPRB,  &
  8480.0_JPRB,  8490.0_JPRB,  8500.0_JPRB,  8510.0_JPRB,  8520.0_JPRB,  8530.0_JPRB,  8540.0_JPRB,  &
  8550.0_JPRB,  8560.0_JPRB,  8570.0_JPRB,  8580.0_JPRB,  8590.0_JPRB,  8600.0_JPRB,  8610.0_JPRB,  &
  8620.0_JPRB,  8630.0_JPRB,  8640.0_JPRB,  8650.0_JPRB,  8660.0_JPRB,  8670.0_JPRB,  8680.0_JPRB,  &
  8690.0_JPRB,  8700.0_JPRB,  8710.0_JPRB,  8720.0_JPRB,  8730.0_JPRB,  8740.0_JPRB,  8750.0_JPRB,  &
  8760.0_JPRB,  8770.0_JPRB,  8780.0_JPRB,  8790.0_JPRB,  8800.0_JPRB,  8810.0_JPRB,  8820.0_JPRB,  &
  8830.0_JPRB,  8840.0_JPRB,  8850.0_JPRB,  8860.0_JPRB,  8870.0_JPRB,  8880.0_JPRB,  8890.0_JPRB,  &
  8900.0_JPRB,  8910.0_JPRB,  8920.0_JPRB,  8930.0_JPRB,  8940.0_JPRB,  8950.0_JPRB,  8960.0_JPRB,  &
  8970.0_JPRB,  8980.0_JPRB,  8990.0_JPRB,  9000.0_JPRB,  9010.0_JPRB,  9020.0_JPRB,  9030.0_JPRB,  &
  9040.0_JPRB,  9050.0_JPRB,  9060.0_JPRB,  9070.0_JPRB,  9080.0_JPRB,  9090.0_JPRB,  9100.0_JPRB,  &
  9110.0_JPRB,  9120.0_JPRB,  9130.0_JPRB,  9140.0_JPRB,  9150.0_JPRB,  9160.0_JPRB,  9170.0_JPRB,  &
  9180.0_JPRB,  9190.0_JPRB,  9200.0_JPRB,  9210.0_JPRB,  9220.0_JPRB,  9230.0_JPRB,  9240.0_JPRB,  &
  9250.0_JPRB,  9260.0_JPRB,  9270.0_JPRB,  9280.0_JPRB,  9290.0_JPRB,  9300.0_JPRB,  9310.0_JPRB,  &
  9320.0_JPRB,  9330.0_JPRB,  9340.0_JPRB,  9350.0_JPRB,  9360.0_JPRB,  9370.0_JPRB,  9380.0_JPRB,  &
  9390.0_JPRB,  9400.0_JPRB,  9410.0_JPRB,  9420.0_JPRB,  9430.0_JPRB,  9440.0_JPRB,  9450.0_JPRB,  &
  9460.0_JPRB,  9470.0_JPRB,  9480.0_JPRB,  9490.0_JPRB,  9500.0_JPRB,  9510.0_JPRB,  9520.0_JPRB,  &
  9530.0_JPRB,  9540.0_JPRB,  9550.0_JPRB,  9560.0_JPRB,  9570.0_JPRB,  9580.0_JPRB,  9590.0_JPRB,  &
  9600.0_JPRB,  9610.0_JPRB,  9620.0_JPRB,  9630.0_JPRB,  9640.0_JPRB,  9650.0_JPRB,  9660.0_JPRB,  &
  9670.0_JPRB,  9680.0_JPRB,  9690.0_JPRB,  9700.0_JPRB,  9710.0_JPRB,  9720.0_JPRB,  9730.0_JPRB,  &
  9740.0_JPRB,  9750.0_JPRB,  9760.0_JPRB,  9770.0_JPRB,  9780.0_JPRB,  9790.0_JPRB,  9800.0_JPRB,  &
  9810.0_JPRB,  9820.0_JPRB,  9830.0_JPRB,  9840.0_JPRB,  9850.0_JPRB,  9860.0_JPRB,  9870.0_JPRB,  &
  9880.0_JPRB,  9890.0_JPRB,  9900.0_JPRB,  9910.0_JPRB,  9920.0_JPRB,  9930.0_JPRB,  9940.0_JPRB,  &
  9950.0_JPRB,  9960.0_JPRB,  9970.0_JPRB,  9980.0_JPRB,  9990.0_JPRB, 10000.0_JPRB, 10010.0_JPRB,  &
 10020.0_JPRB, 10030.0_JPRB, 10040.0_JPRB, 10050.0_JPRB, 10060.0_JPRB, 10070.0_JPRB, 10080.0_JPRB,  &
 10090.0_JPRB, 10100.0_JPRB, 10110.0_JPRB, 10120.0_JPRB, 10130.0_JPRB, 10140.0_JPRB, 10150.0_JPRB,  &
 10160.0_JPRB, 10170.0_JPRB, 10180.0_JPRB, 10190.0_JPRB, 10200.0_JPRB, 10210.0_JPRB, 10220.0_JPRB,  &
 10230.0_JPRB, 10240.0_JPRB, 10250.0_JPRB, 10260.0_JPRB, 10270.0_JPRB, 10280.0_JPRB, 10290.0_JPRB,  &
 10300.0_JPRB, 10310.0_JPRB, 10320.0_JPRB, 10330.0_JPRB, 10340.0_JPRB, 10350.0_JPRB, 10360.0_JPRB,  &
 10370.0_JPRB, 10380.0_JPRB, 10390.0_JPRB, 10400.0_JPRB, 10410.0_JPRB, 10420.0_JPRB, 10430.0_JPRB,  &
 10440.0_JPRB, 10450.0_JPRB, 10460.0_JPRB, 10470.0_JPRB, 10480.0_JPRB, 10490.0_JPRB, 10500.0_JPRB,  &
 10510.0_JPRB, 10520.0_JPRB, 10530.0_JPRB, 10540.0_JPRB, 10550.0_JPRB, 10560.0_JPRB, 10570.0_JPRB,  &
 10580.0_JPRB, 10590.0_JPRB, 10600.0_JPRB, 10610.0_JPRB, 10620.0_JPRB, 10630.0_JPRB, 10640.0_JPRB,  &
 10650.0_JPRB, 10660.0_JPRB, 10670.0_JPRB, 10680.0_JPRB, 10690.0_JPRB, 10700.0_JPRB, 10710.0_JPRB,  &
 10720.0_JPRB, 10730.0_JPRB, 10740.0_JPRB, 10750.0_JPRB, 10760.0_JPRB, 10770.0_JPRB, 10780.0_JPRB,  &
 10790.0_JPRB, 10800.0_JPRB, 10810.0_JPRB, 10820.0_JPRB, 10830.0_JPRB, 10840.0_JPRB, 10850.0_JPRB,  &
 10860.0_JPRB, 10870.0_JPRB, 10880.0_JPRB, 10890.0_JPRB, 10900.0_JPRB, 10910.0_JPRB, 10920.0_JPRB,  &
 10930.0_JPRB, 10940.0_JPRB, 10950.0_JPRB, 10960.0_JPRB, 10970.0_JPRB, 10980.0_JPRB, 10990.0_JPRB,  &
 11000.0_JPRB, 11010.0_JPRB, 11020.0_JPRB, 11030.0_JPRB, 11040.0_JPRB, 11050.0_JPRB, 11060.0_JPRB,  &
 11070.0_JPRB, 11080.0_JPRB, 11090.0_JPRB, 11100.0_JPRB, 11110.0_JPRB, 11120.0_JPRB, 11130.0_JPRB,  &
 11140.0_JPRB, 11150.0_JPRB, 11160.0_JPRB, 11170.0_JPRB, 11180.0_JPRB, 11190.0_JPRB, 11200.0_JPRB,  &
 11210.0_JPRB, 11220.0_JPRB, 11230.0_JPRB, 11240.0_JPRB, 11250.0_JPRB, 11260.0_JPRB, 11270.0_JPRB,  &
 11280.0_JPRB, 11290.0_JPRB, 11300.0_JPRB, 11310.0_JPRB, 11320.0_JPRB, 11330.0_JPRB, 11340.0_JPRB,  &
 11350.0_JPRB, 11360.0_JPRB, 11370.0_JPRB, 11380.0_JPRB, 11390.0_JPRB, 11400.0_JPRB, 11410.0_JPRB,  &
 11420.0_JPRB, 11430.0_JPRB, 11440.0_JPRB, 11450.0_JPRB, 11460.0_JPRB, 11470.0_JPRB, 11480.0_JPRB,  &
 11490.0_JPRB, 11500.0_JPRB, 11510.0_JPRB, 11520.0_JPRB, 11530.0_JPRB, 11540.0_JPRB, 11550.0_JPRB,  &
 11560.0_JPRB, 11570.0_JPRB, 11580.0_JPRB, 11590.0_JPRB, 11600.0_JPRB, 11610.0_JPRB, 11620.0_JPRB,  &
 11630.0_JPRB, 11640.0_JPRB, 11650.0_JPRB, 11660.0_JPRB, 11670.0_JPRB, 11680.0_JPRB, 11690.0_JPRB,  &
 11700.0_JPRB, 11710.0_JPRB, 11720.0_JPRB, 11730.0_JPRB, 11740.0_JPRB, 11750.0_JPRB, 11760.0_JPRB,  &
 11770.0_JPRB, 11780.0_JPRB, 11790.0_JPRB, 11800.0_JPRB, 11810.0_JPRB, 11820.0_JPRB, 11830.0_JPRB,  &
 11840.0_JPRB, 11850.0_JPRB, 11860.0_JPRB, 11870.0_JPRB, 11880.0_JPRB, 11890.0_JPRB, 11900.0_JPRB,  &
 11910.0_JPRB, 11920.0_JPRB, 11930.0_JPRB, 11940.0_JPRB, 11950.0_JPRB, 11960.0_JPRB, 11970.0_JPRB,  &
 11980.0_JPRB, 11990.0_JPRB, 12000.0_JPRB, 12010.0_JPRB, 12020.0_JPRB, 12030.0_JPRB, 12040.0_JPRB,  &
 12050.0_JPRB, 12060.0_JPRB, 12070.0_JPRB, 12080.0_JPRB, 12090.0_JPRB, 12100.0_JPRB, 12110.0_JPRB,  &
 12120.0_JPRB, 12130.0_JPRB, 12140.0_JPRB, 12150.0_JPRB, 12160.0_JPRB, 12170.0_JPRB, 12180.0_JPRB,  &
 12190.0_JPRB, 12200.0_JPRB, 12210.0_JPRB, 12220.0_JPRB, 12230.0_JPRB, 12240.0_JPRB, 12250.0_JPRB,  &
 12260.0_JPRB, 12270.0_JPRB, 12280.0_JPRB, 12290.0_JPRB, 12300.0_JPRB, 12310.0_JPRB, 12320.0_JPRB,  &
 12330.0_JPRB, 12340.0_JPRB, 12350.0_JPRB, 12360.0_JPRB, 12370.0_JPRB, 12380.0_JPRB, 12390.0_JPRB,  &
 12400.0_JPRB, 12410.0_JPRB, 12420.0_JPRB, 12430.0_JPRB, 12440.0_JPRB, 12450.0_JPRB, 12460.0_JPRB,  &
 12470.0_JPRB, 12480.0_JPRB, 12490.0_JPRB, 12500.0_JPRB, 12510.0_JPRB, 12520.0_JPRB, 12530.0_JPRB,  &
 12540.0_JPRB, 12550.0_JPRB, 12560.0_JPRB, 12570.0_JPRB, 12580.0_JPRB, 12590.0_JPRB, 12600.0_JPRB,  &
 12610.0_JPRB, 12620.0_JPRB, 12630.0_JPRB, 12640.0_JPRB, 12650.0_JPRB, 12660.0_JPRB, 12670.0_JPRB,  &
 12680.0_JPRB, 12690.0_JPRB, 12700.0_JPRB, 12710.0_JPRB, 12720.0_JPRB, 12730.0_JPRB, 12740.0_JPRB,  &
 12750.0_JPRB, 12760.0_JPRB, 12770.0_JPRB, 12780.0_JPRB, 12790.0_JPRB, 12800.0_JPRB, 12810.0_JPRB,  &
 12820.0_JPRB, 12830.0_JPRB, 12840.0_JPRB, 12850.0_JPRB, 12860.0_JPRB, 12870.0_JPRB, 12880.0_JPRB,  &
 12890.0_JPRB, 12900.0_JPRB, 12910.0_JPRB, 12920.0_JPRB, 12930.0_JPRB, 12940.0_JPRB, 12950.0_JPRB,  &
 12960.0_JPRB, 12970.0_JPRB, 12980.0_JPRB, 12990.0_JPRB, 13000.0_JPRB, 13010.0_JPRB, 13020.0_JPRB,  &
 13030.0_JPRB, 13040.0_JPRB, 13050.0_JPRB, 13060.0_JPRB, 13070.0_JPRB, 13080.0_JPRB, 13090.0_JPRB,  &
 13100.0_JPRB, 13110.0_JPRB, 13120.0_JPRB, 13130.0_JPRB, 13140.0_JPRB, 13150.0_JPRB, 13160.0_JPRB,  &
 13170.0_JPRB, 13180.0_JPRB, 13190.0_JPRB, 13200.0_JPRB, 13210.0_JPRB, 13220.0_JPRB, 13230.0_JPRB,  &
 13240.0_JPRB, 13250.0_JPRB, 13260.0_JPRB, 13270.0_JPRB, 13280.0_JPRB, 13290.0_JPRB, 13300.0_JPRB,  &
 13310.0_JPRB, 13320.0_JPRB, 13330.0_JPRB, 13340.0_JPRB, 13350.0_JPRB, 13360.0_JPRB, 13370.0_JPRB,  &
 13380.0_JPRB, 13390.0_JPRB, 13400.0_JPRB, 13410.0_JPRB, 13420.0_JPRB, 13430.0_JPRB, 13440.0_JPRB,  &
 13450.0_JPRB, 13460.0_JPRB, 13470.0_JPRB, 13480.0_JPRB, 13490.0_JPRB, 13500.0_JPRB, 13510.0_JPRB,  &
 13520.0_JPRB, 13530.0_JPRB, 13540.0_JPRB, 13550.0_JPRB, 13560.0_JPRB, 13570.0_JPRB, 13580.0_JPRB,  &
 13590.0_JPRB, 13600.0_JPRB, 13610.0_JPRB, 13620.0_JPRB, 13630.0_JPRB, 13640.0_JPRB, 13650.0_JPRB,  &
 13660.0_JPRB, 13670.0_JPRB, 13680.0_JPRB, 13690.0_JPRB, 13700.0_JPRB, 13710.0_JPRB, 13720.0_JPRB,  &
 13730.0_JPRB, 13740.0_JPRB, 13750.0_JPRB, 13760.0_JPRB, 13770.0_JPRB, 13780.0_JPRB, 13790.0_JPRB,  &
 13800.0_JPRB, 13810.0_JPRB, 13820.0_JPRB, 13830.0_JPRB, 13840.0_JPRB, 13850.0_JPRB, 13860.0_JPRB,  &
 13870.0_JPRB, 13880.0_JPRB, 13890.0_JPRB, 13900.0_JPRB, 13910.0_JPRB, 13920.0_JPRB, 13930.0_JPRB,  &
 13940.0_JPRB, 13950.0_JPRB, 13960.0_JPRB, 13970.0_JPRB, 13980.0_JPRB, 13990.0_JPRB, 14000.0_JPRB,  &
 14010.0_JPRB, 14020.0_JPRB, 14030.0_JPRB, 14040.0_JPRB, 14050.0_JPRB, 14060.0_JPRB, 14070.0_JPRB,  &
 14080.0_JPRB, 14090.0_JPRB, 14100.0_JPRB, 14110.0_JPRB, 14120.0_JPRB, 14130.0_JPRB, 14140.0_JPRB,  &
 14150.0_JPRB, 14160.0_JPRB, 14170.0_JPRB, 14180.0_JPRB, 14190.0_JPRB, 14200.0_JPRB, 14210.0_JPRB,  &
 14220.0_JPRB, 14230.0_JPRB, 14240.0_JPRB, 14250.0_JPRB, 14260.0_JPRB, 14270.0_JPRB, 14280.0_JPRB,  &
 14290.0_JPRB, 14300.0_JPRB, 14310.0_JPRB, 14320.0_JPRB, 14330.0_JPRB, 14340.0_JPRB, 14350.0_JPRB,  &
 14360.0_JPRB, 14370.0_JPRB, 14380.0_JPRB, 14390.0_JPRB, 14400.0_JPRB, 14410.0_JPRB, 14420.0_JPRB,  &
 14430.0_JPRB, 14440.0_JPRB, 14450.0_JPRB, 14460.0_JPRB, 14470.0_JPRB, 14480.0_JPRB, 14490.0_JPRB,  &
 14500.0_JPRB, 14510.0_JPRB, 14520.0_JPRB, 14530.0_JPRB, 14540.0_JPRB, 14550.0_JPRB, 14560.0_JPRB,  &
 14570.0_JPRB, 14580.0_JPRB, 14590.0_JPRB, 14600.0_JPRB, 14610.0_JPRB, 14620.0_JPRB, 14630.0_JPRB,  &
 14640.0_JPRB, 14650.0_JPRB, 14660.0_JPRB, 14670.0_JPRB, 14680.0_JPRB, 14690.0_JPRB, 14700.0_JPRB,  &
 14710.0_JPRB, 14720.0_JPRB, 14730.0_JPRB, 14740.0_JPRB, 14750.0_JPRB, 14760.0_JPRB, 14770.0_JPRB,  &
 14780.0_JPRB, 14790.0_JPRB, 14800.0_JPRB, 14810.0_JPRB, 14820.0_JPRB, 14830.0_JPRB, 14840.0_JPRB,  &
 14850.0_JPRB, 14860.0_JPRB, 14870.0_JPRB, 14880.0_JPRB, 14890.0_JPRB, 14900.0_JPRB, 14910.0_JPRB,  &
 14920.0_JPRB, 14930.0_JPRB, 14940.0_JPRB, 14950.0_JPRB, 14960.0_JPRB, 14970.0_JPRB, 14980.0_JPRB,  &
 14990.0_JPRB, 15000.0_JPRB, 15010.0_JPRB, 15020.0_JPRB, 15030.0_JPRB, 15040.0_JPRB, 15050.0_JPRB,  &
 15060.0_JPRB, 15070.0_JPRB, 15080.0_JPRB, 15090.0_JPRB, 15100.0_JPRB, 15110.0_JPRB, 15120.0_JPRB,  &
 15130.0_JPRB, 15140.0_JPRB, 15150.0_JPRB, 15160.0_JPRB, 15170.0_JPRB, 15180.0_JPRB, 15190.0_JPRB,  &
 15200.0_JPRB, 15210.0_JPRB, 15220.0_JPRB, 15230.0_JPRB, 15240.0_JPRB, 15250.0_JPRB, 15260.0_JPRB,  &
 15270.0_JPRB, 15280.0_JPRB, 15290.0_JPRB, 15300.0_JPRB, 15310.0_JPRB, 15320.0_JPRB, 15330.0_JPRB,  &
 15340.0_JPRB, 15350.0_JPRB, 15360.0_JPRB, 15370.0_JPRB, 15380.0_JPRB, 15390.0_JPRB, 15400.0_JPRB,  &
 15410.0_JPRB, 15420.0_JPRB, 15430.0_JPRB, 15440.0_JPRB, 15450.0_JPRB, 15460.0_JPRB, 15470.0_JPRB,  &
 15480.0_JPRB, 15490.0_JPRB, 15500.0_JPRB, 15510.0_JPRB, 15520.0_JPRB, 15530.0_JPRB, 15540.0_JPRB,  &
 15550.0_JPRB, 15560.0_JPRB, 15570.0_JPRB, 15580.0_JPRB, 15590.0_JPRB, 15600.0_JPRB, 15610.0_JPRB,  &
 15620.0_JPRB, 15630.0_JPRB, 15640.0_JPRB, 15650.0_JPRB, 15660.0_JPRB, 15670.0_JPRB, 15680.0_JPRB,  &
 15690.0_JPRB, 15700.0_JPRB, 15710.0_JPRB, 15720.0_JPRB, 15730.0_JPRB, 15740.0_JPRB, 15750.0_JPRB,  &
 15760.0_JPRB, 15770.0_JPRB, 15780.0_JPRB, 15790.0_JPRB, 15800.0_JPRB, 15810.0_JPRB, 15820.0_JPRB,  &
 15830.0_JPRB, 15840.0_JPRB, 15850.0_JPRB, 15860.0_JPRB, 15870.0_JPRB, 15880.0_JPRB, 15890.0_JPRB,  &
 15900.0_JPRB, 15910.0_JPRB, 15920.0_JPRB, 15930.0_JPRB, 15940.0_JPRB, 15950.0_JPRB, 15960.0_JPRB,  &
 15970.0_JPRB, 15980.0_JPRB, 15990.0_JPRB, 16000.0_JPRB, 16010.0_JPRB, 16020.0_JPRB, 16030.0_JPRB,  &
 16040.0_JPRB, 16050.0_JPRB, 16060.0_JPRB, 16070.0_JPRB, 16080.0_JPRB, 16090.0_JPRB, 16100.0_JPRB,  &
 16110.0_JPRB, 16120.0_JPRB, 16130.0_JPRB, 16140.0_JPRB, 16150.0_JPRB, 16160.0_JPRB, 16170.0_JPRB,  &
 16180.0_JPRB, 16190.0_JPRB, 16200.0_JPRB, 16210.0_JPRB, 16220.0_JPRB, 16230.0_JPRB, 16240.0_JPRB,  &
 16250.0_JPRB, 16260.0_JPRB, 16270.0_JPRB, 16280.0_JPRB, 16290.0_JPRB, 16300.0_JPRB, 16310.0_JPRB,  &
 16320.0_JPRB, 16330.0_JPRB, 16340.0_JPRB, 16350.0_JPRB, 16360.0_JPRB, 16370.0_JPRB, 16380.0_JPRB,  &
 16390.0_JPRB, 16400.0_JPRB, 16410.0_JPRB, 16420.0_JPRB, 16430.0_JPRB, 16440.0_JPRB, 16450.0_JPRB,  &
 16460.0_JPRB, 16470.0_JPRB, 16480.0_JPRB, 16490.0_JPRB, 16500.0_JPRB, 16510.0_JPRB, 16520.0_JPRB,  &
 16530.0_JPRB, 16540.0_JPRB, 16550.0_JPRB, 16560.0_JPRB, 16570.0_JPRB, 16580.0_JPRB, 16590.0_JPRB,  &
 16600.0_JPRB, 16610.0_JPRB, 16620.0_JPRB, 16630.0_JPRB, 16640.0_JPRB, 16650.0_JPRB, 16660.0_JPRB,  &
 16670.0_JPRB, 16680.0_JPRB, 16690.0_JPRB, 16700.0_JPRB, 16710.0_JPRB, 16720.0_JPRB, 16730.0_JPRB,  &
 16740.0_JPRB, 16750.0_JPRB, 16760.0_JPRB, 16770.0_JPRB, 16780.0_JPRB, 16790.0_JPRB, 16800.0_JPRB,  &
 16810.0_JPRB, 16820.0_JPRB, 16830.0_JPRB, 16840.0_JPRB, 16850.0_JPRB, 16860.0_JPRB, 16870.0_JPRB,  &
 16880.0_JPRB, 16890.0_JPRB, 16900.0_JPRB, 16910.0_JPRB, 16920.0_JPRB, 16930.0_JPRB, 16940.0_JPRB,  &
 16950.0_JPRB, 16960.0_JPRB, 16970.0_JPRB, 16980.0_JPRB, 16990.0_JPRB, 17000.0_JPRB, 17010.0_JPRB,  &
 17020.0_JPRB, 17030.0_JPRB, 17040.0_JPRB, 17050.0_JPRB, 17060.0_JPRB, 17070.0_JPRB, 17080.0_JPRB,  &
 17090.0_JPRB, 17100.0_JPRB, 17110.0_JPRB, 17120.0_JPRB, 17130.0_JPRB, 17140.0_JPRB, 17150.0_JPRB,  &
 17160.0_JPRB, 17170.0_JPRB, 17180.0_JPRB, 17190.0_JPRB, 17200.0_JPRB, 17210.0_JPRB, 17220.0_JPRB,  &
 17230.0_JPRB, 17240.0_JPRB, 17250.0_JPRB, 17260.0_JPRB, 17270.0_JPRB, 17280.0_JPRB, 17290.0_JPRB,  &
 17300.0_JPRB, 17310.0_JPRB, 17320.0_JPRB, 17330.0_JPRB, 17340.0_JPRB, 17350.0_JPRB, 17360.0_JPRB,  &
 17370.0_JPRB, 17380.0_JPRB, 17390.0_JPRB, 17400.0_JPRB, 17410.0_JPRB, 17420.0_JPRB, 17430.0_JPRB,  &
 17440.0_JPRB, 17450.0_JPRB, 17460.0_JPRB, 17470.0_JPRB, 17480.0_JPRB, 17490.0_JPRB, 17500.0_JPRB,  &
 17510.0_JPRB, 17520.0_JPRB, 17530.0_JPRB, 17540.0_JPRB, 17550.0_JPRB, 17560.0_JPRB, 17570.0_JPRB,  &
 17580.0_JPRB, 17590.0_JPRB, 17600.0_JPRB, 17610.0_JPRB, 17620.0_JPRB, 17630.0_JPRB, 17640.0_JPRB,  &
 17650.0_JPRB, 17660.0_JPRB, 17670.0_JPRB, 17680.0_JPRB, 17690.0_JPRB, 17700.0_JPRB, 17710.0_JPRB,  &
 17720.0_JPRB, 17730.0_JPRB, 17740.0_JPRB, 17750.0_JPRB, 17760.0_JPRB, 17770.0_JPRB, 17780.0_JPRB,  &
 17790.0_JPRB, 17800.0_JPRB, 17810.0_JPRB, 17820.0_JPRB, 17830.0_JPRB, 17840.0_JPRB, 17850.0_JPRB,  &
 17860.0_JPRB, 17870.0_JPRB, 17880.0_JPRB, 17890.0_JPRB, 17900.0_JPRB, 17910.0_JPRB, 17920.0_JPRB,  &
 17930.0_JPRB, 17940.0_JPRB, 17950.0_JPRB, 17960.0_JPRB, 17970.0_JPRB, 17980.0_JPRB, 17990.0_JPRB,  &
 18000.0_JPRB, 18010.0_JPRB, 18020.0_JPRB, 18030.0_JPRB, 18040.0_JPRB, 18050.0_JPRB, 18060.0_JPRB,  &
 18070.0_JPRB, 18080.0_JPRB, 18090.0_JPRB, 18100.0_JPRB, 18110.0_JPRB, 18120.0_JPRB, 18130.0_JPRB,  &
 18140.0_JPRB, 18150.0_JPRB, 18160.0_JPRB, 18170.0_JPRB, 18180.0_JPRB, 18190.0_JPRB, 18200.0_JPRB,  &
 18210.0_JPRB, 18220.0_JPRB, 18230.0_JPRB, 18240.0_JPRB, 18250.0_JPRB, 18260.0_JPRB, 18270.0_JPRB,  &
 18280.0_JPRB, 18290.0_JPRB, 18300.0_JPRB, 18310.0_JPRB, 18320.0_JPRB, 18330.0_JPRB, 18340.0_JPRB,  &
 18350.0_JPRB, 18360.0_JPRB, 18370.0_JPRB, 18380.0_JPRB, 18390.0_JPRB, 18400.0_JPRB, 18410.0_JPRB,  &
 18420.0_JPRB, 18430.0_JPRB, 18440.0_JPRB, 18450.0_JPRB, 18460.0_JPRB, 18470.0_JPRB, 18480.0_JPRB,  &
 18490.0_JPRB, 18500.0_JPRB, 18510.0_JPRB, 18520.0_JPRB, 18530.0_JPRB, 18540.0_JPRB, 18550.0_JPRB,  &
 18560.0_JPRB, 18570.0_JPRB, 18580.0_JPRB, 18590.0_JPRB, 18600.0_JPRB, 18610.0_JPRB, 18620.0_JPRB,  &
 18630.0_JPRB, 18640.0_JPRB, 18650.0_JPRB, 18660.0_JPRB, 18670.0_JPRB, 18680.0_JPRB, 18690.0_JPRB,  &
 18700.0_JPRB, 18710.0_JPRB, 18720.0_JPRB, 18730.0_JPRB, 18740.0_JPRB, 18750.0_JPRB, 18760.0_JPRB,  &
 18770.0_JPRB, 18780.0_JPRB, 18790.0_JPRB, 18800.0_JPRB, 18810.0_JPRB, 18820.0_JPRB, 18830.0_JPRB,  &
 18840.0_JPRB, 18850.0_JPRB, 18860.0_JPRB, 18870.0_JPRB, 18880.0_JPRB, 18890.0_JPRB, 18900.0_JPRB,  &
 18910.0_JPRB, 18920.0_JPRB, 18930.0_JPRB, 18940.0_JPRB, 18950.0_JPRB, 18960.0_JPRB, 18970.0_JPRB,  &
 18980.0_JPRB, 18990.0_JPRB, 19000.0_JPRB, 19010.0_JPRB, 19020.0_JPRB, 19030.0_JPRB, 19040.0_JPRB,  &
 19050.0_JPRB, 19060.0_JPRB, 19070.0_JPRB, 19080.0_JPRB, 19090.0_JPRB, 19100.0_JPRB, 19110.0_JPRB,  &
 19120.0_JPRB, 19130.0_JPRB, 19140.0_JPRB, 19150.0_JPRB, 19160.0_JPRB, 19170.0_JPRB, 19180.0_JPRB,  &
 19190.0_JPRB, 19200.0_JPRB, 19210.0_JPRB, 19220.0_JPRB, 19230.0_JPRB, 19240.0_JPRB, 19250.0_JPRB,  &
 19260.0_JPRB, 19270.0_JPRB, 19280.0_JPRB, 19290.0_JPRB, 19300.0_JPRB, 19310.0_JPRB, 19320.0_JPRB,  &
 19330.0_JPRB, 19340.0_JPRB, 19350.0_JPRB, 19360.0_JPRB, 19370.0_JPRB, 19380.0_JPRB, 19390.0_JPRB,  &
 19400.0_JPRB, 19410.0_JPRB, 19420.0_JPRB, 19430.0_JPRB, 19440.0_JPRB, 19450.0_JPRB, 19460.0_JPRB,  &
 19470.0_JPRB, 19480.0_JPRB, 19490.0_JPRB, 19500.0_JPRB, 19510.0_JPRB, 19520.0_JPRB, 19530.0_JPRB,  &
 19540.0_JPRB, 19550.0_JPRB, 19560.0_JPRB, 19570.0_JPRB, 19580.0_JPRB, 19590.0_JPRB, 19600.0_JPRB,  &
 19610.0_JPRB, 19620.0_JPRB, 19630.0_JPRB, 19640.0_JPRB, 19650.0_JPRB, 19660.0_JPRB, 19670.0_JPRB,  &
 19680.0_JPRB, 19690.0_JPRB, 19700.0_JPRB, 19710.0_JPRB, 19720.0_JPRB, 19730.0_JPRB, 19740.0_JPRB,  &
 19750.0_JPRB, 19760.0_JPRB, 19770.0_JPRB, 19780.0_JPRB, 19790.0_JPRB, 19800.0_JPRB, 19810.0_JPRB,  &
 19820.0_JPRB, 19830.0_JPRB, 19840.0_JPRB, 19850.0_JPRB, 19860.0_JPRB, 19870.0_JPRB, 19880.0_JPRB,  &
 19890.0_JPRB, 19900.0_JPRB, 19910.0_JPRB, 19920.0_JPRB, 19930.0_JPRB, 19940.0_JPRB, 19950.0_JPRB,  &
 19960.0_JPRB, 19970.0_JPRB, 19980.0_JPRB, 19990.0_JPRB, 20000.0_JPRB, 20010.0_JPRB, 20020.0_JPRB,  &
 20030.0_JPRB, 20040.0_JPRB, 20050.0_JPRB, 20060.0_JPRB, 20070.0_JPRB, 20080.0_JPRB, 20090.0_JPRB,  &
 20100.0_JPRB, 20110.0_JPRB, 20120.0_JPRB, 20130.0_JPRB, 20140.0_JPRB, 20150.0_JPRB, 20160.0_JPRB,  &
 20170.0_JPRB, 20180.0_JPRB, 20190.0_JPRB, 20200.0_JPRB, 20210.0_JPRB, 20220.0_JPRB, 20230.0_JPRB,  &
 20240.0_JPRB, 20250.0_JPRB, 20260.0_JPRB, 20270.0_JPRB, 20280.0_JPRB, 20290.0_JPRB, 20300.0_JPRB,  &
 20310.0_JPRB, 20320.0_JPRB, 20330.0_JPRB, 20340.0_JPRB, 20350.0_JPRB, 20360.0_JPRB, 20370.0_JPRB,  &
 20380.0_JPRB, 20390.0_JPRB, 20400.0_JPRB, 20410.0_JPRB, 20420.0_JPRB, 20430.0_JPRB, 20440.0_JPRB,  &
 20450.0_JPRB, 20460.0_JPRB, 20470.0_JPRB, 20480.0_JPRB, 20490.0_JPRB, 20500.0_JPRB, 20510.0_JPRB,  &
 20520.0_JPRB, 20530.0_JPRB, 20540.0_JPRB, 20550.0_JPRB, 20560.0_JPRB, 20570.0_JPRB, 20580.0_JPRB,  &
 20590.0_JPRB, 20600.0_JPRB, 20610.0_JPRB, 20620.0_JPRB, 20630.0_JPRB, 20640.0_JPRB, 20650.0_JPRB,  &
 20660.0_JPRB, 20670.0_JPRB, 20680.0_JPRB, 20690.0_JPRB, 20700.0_JPRB, 20710.0_JPRB, 20720.0_JPRB,  &
 20730.0_JPRB, 20740.0_JPRB, 20750.0_JPRB, 20760.0_JPRB, 20770.0_JPRB, 20780.0_JPRB, 20790.0_JPRB,  &
 20800.0_JPRB, 20810.0_JPRB, 20820.0_JPRB, 20830.0_JPRB, 20840.0_JPRB, 20850.0_JPRB, 20860.0_JPRB,  &
 20870.0_JPRB, 20880.0_JPRB, 20890.0_JPRB, 20900.0_JPRB, 20910.0_JPRB, 20920.0_JPRB, 20930.0_JPRB,  &
 20940.0_JPRB, 20950.0_JPRB, 20960.0_JPRB, 20970.0_JPRB, 20980.0_JPRB, 20990.0_JPRB, 21000.0_JPRB,  &
 21010.0_JPRB, 21020.0_JPRB, 21030.0_JPRB, 21040.0_JPRB, 21050.0_JPRB, 21060.0_JPRB, 21070.0_JPRB,  &
 21080.0_JPRB, 21090.0_JPRB, 21100.0_JPRB, 21110.0_JPRB, 21120.0_JPRB, 21130.0_JPRB, 21140.0_JPRB,  &
 21150.0_JPRB, 21160.0_JPRB, 21170.0_JPRB, 21180.0_JPRB, 21190.0_JPRB, 21200.0_JPRB, 21210.0_JPRB,  &
 21220.0_JPRB, 21230.0_JPRB, 21240.0_JPRB, 21250.0_JPRB, 21260.0_JPRB, 21270.0_JPRB, 21280.0_JPRB,  &
 21290.0_JPRB, 21300.0_JPRB, 21310.0_JPRB, 21320.0_JPRB, 21330.0_JPRB, 21340.0_JPRB, 21350.0_JPRB,  &
 21360.0_JPRB, 21370.0_JPRB, 21380.0_JPRB, 21390.0_JPRB, 21400.0_JPRB, 21410.0_JPRB, 21420.0_JPRB,  &
 21430.0_JPRB, 21440.0_JPRB, 21450.0_JPRB, 21460.0_JPRB, 21470.0_JPRB, 21480.0_JPRB, 21490.0_JPRB,  &
 21500.0_JPRB, 21510.0_JPRB, 21520.0_JPRB, 21530.0_JPRB, 21540.0_JPRB, 21550.0_JPRB, 21560.0_JPRB,  &
 21570.0_JPRB, 21580.0_JPRB, 21590.0_JPRB, 21600.0_JPRB, 21610.0_JPRB, 21620.0_JPRB, 21630.0_JPRB,  &
 21640.0_JPRB, 21650.0_JPRB, 21660.0_JPRB, 21670.0_JPRB, 21680.0_JPRB, 21690.0_JPRB, 21700.0_JPRB,  &
 21710.0_JPRB, 21720.0_JPRB, 21730.0_JPRB, 21740.0_JPRB, 21750.0_JPRB, 21760.0_JPRB, 21770.0_JPRB,  &
 21780.0_JPRB, 21790.0_JPRB, 21800.0_JPRB, 21810.0_JPRB, 21820.0_JPRB, 21830.0_JPRB, 21840.0_JPRB,  &
 21850.0_JPRB, 21860.0_JPRB, 21870.0_JPRB, 21880.0_JPRB, 21890.0_JPRB, 21900.0_JPRB, 21910.0_JPRB,  &
 21920.0_JPRB, 21930.0_JPRB, 21940.0_JPRB, 21950.0_JPRB, 21960.0_JPRB, 21970.0_JPRB, 21980.0_JPRB,  &
 21990.0_JPRB, 22000.0_JPRB, 22010.0_JPRB, 22020.0_JPRB, 22030.0_JPRB, 22040.0_JPRB, 22050.0_JPRB,  &
 22060.0_JPRB, 22070.0_JPRB, 22080.0_JPRB, 22090.0_JPRB, 22100.0_JPRB, 22110.0_JPRB, 22120.0_JPRB,  &
 22130.0_JPRB, 22140.0_JPRB, 22150.0_JPRB, 22160.0_JPRB, 22170.0_JPRB, 22180.0_JPRB, 22190.0_JPRB,  &
 22200.0_JPRB, 22210.0_JPRB, 22220.0_JPRB, 22230.0_JPRB, 22240.0_JPRB, 22250.0_JPRB, 22260.0_JPRB,  &
 22270.0_JPRB, 22280.0_JPRB, 22290.0_JPRB, 22300.0_JPRB, 22310.0_JPRB, 22320.0_JPRB, 22330.0_JPRB,  &
 22340.0_JPRB, 22350.0_JPRB, 22360.0_JPRB, 22370.0_JPRB, 22380.0_JPRB, 22390.0_JPRB, 22400.0_JPRB,  &
 22410.0_JPRB, 22420.0_JPRB, 22430.0_JPRB, 22440.0_JPRB, 22450.0_JPRB, 22460.0_JPRB, 22470.0_JPRB,  &
 22480.0_JPRB, 22490.0_JPRB, 22500.0_JPRB, 22510.0_JPRB, 22520.0_JPRB, 22530.0_JPRB, 22540.0_JPRB,  &
 22550.0_JPRB, 22560.0_JPRB, 22570.0_JPRB, 22580.0_JPRB, 22590.0_JPRB, 22600.0_JPRB, 22610.0_JPRB,  &
 22620.0_JPRB, 22630.0_JPRB, 22640.0_JPRB, 22650.0_JPRB, 22660.0_JPRB, 22670.0_JPRB, 22680.0_JPRB,  &
 22690.0_JPRB, 22700.0_JPRB, 22710.0_JPRB, 22720.0_JPRB, 22730.0_JPRB, 22740.0_JPRB, 22750.0_JPRB,  &
 22760.0_JPRB, 22770.0_JPRB, 22780.0_JPRB, 22790.0_JPRB, 22800.0_JPRB, 22810.0_JPRB, 22820.0_JPRB,  &
 22830.0_JPRB, 22840.0_JPRB, 22850.0_JPRB, 22860.0_JPRB, 22870.0_JPRB, 22880.0_JPRB, 22890.0_JPRB,  &
 22900.0_JPRB, 22910.0_JPRB, 22920.0_JPRB, 22930.0_JPRB, 22940.0_JPRB, 22950.0_JPRB, 22960.0_JPRB,  &
 22970.0_JPRB, 22980.0_JPRB, 22990.0_JPRB, 23000.0_JPRB, 23010.0_JPRB, 23020.0_JPRB, 23030.0_JPRB,  &
 23040.0_JPRB, 23050.0_JPRB, 23060.0_JPRB, 23070.0_JPRB, 23080.0_JPRB, 23090.0_JPRB, 23100.0_JPRB,  &
 23110.0_JPRB, 23120.0_JPRB, 23130.0_JPRB, 23140.0_JPRB, 23150.0_JPRB, 23160.0_JPRB, 23170.0_JPRB,  &
 23180.0_JPRB, 23190.0_JPRB, 23200.0_JPRB, 23210.0_JPRB, 23220.0_JPRB, 23230.0_JPRB, 23240.0_JPRB,  &
 23250.0_JPRB, 23260.0_JPRB, 23270.0_JPRB, 23280.0_JPRB, 23290.0_JPRB, 23300.0_JPRB, 23310.0_JPRB,  &
 23320.0_JPRB, 23330.0_JPRB, 23340.0_JPRB, 23350.0_JPRB, 23360.0_JPRB, 23370.0_JPRB, 23380.0_JPRB,  &
 23390.0_JPRB, 23400.0_JPRB, 23410.0_JPRB, 23420.0_JPRB, 23430.0_JPRB, 23440.0_JPRB, 23450.0_JPRB,  &
 23460.0_JPRB, 23470.0_JPRB, 23480.0_JPRB, 23490.0_JPRB, 23500.0_JPRB, 23510.0_JPRB, 23520.0_JPRB,  &
 23530.0_JPRB, 23540.0_JPRB, 23550.0_JPRB, 23560.0_JPRB, 23570.0_JPRB, 23580.0_JPRB, 23590.0_JPRB,  &
 23600.0_JPRB, 23610.0_JPRB, 23620.0_JPRB, 23630.0_JPRB, 23640.0_JPRB, 23650.0_JPRB, 23660.0_JPRB,  &
 23670.0_JPRB, 23680.0_JPRB, 23690.0_JPRB, 23700.0_JPRB, 23710.0_JPRB, 23720.0_JPRB, 23730.0_JPRB,  &
 23740.0_JPRB, 23750.0_JPRB, 23760.0_JPRB, 23770.0_JPRB, 23780.0_JPRB, 23790.0_JPRB, 23800.0_JPRB,  &
 23810.0_JPRB, 23820.0_JPRB, 23830.0_JPRB, 23840.0_JPRB, 23850.0_JPRB, 23860.0_JPRB, 23870.0_JPRB,  &
 23880.0_JPRB, 23890.0_JPRB, 23900.0_JPRB, 23910.0_JPRB, 23920.0_JPRB, 23930.0_JPRB, 23940.0_JPRB,  &
 23950.0_JPRB, 23960.0_JPRB, 23970.0_JPRB, 23980.0_JPRB, 23990.0_JPRB, 24000.0_JPRB, 24010.0_JPRB,  &
 24020.0_JPRB, 24030.0_JPRB, 24040.0_JPRB, 24050.0_JPRB, 24060.0_JPRB, 24070.0_JPRB, 24080.0_JPRB,  &
 24090.0_JPRB, 24100.0_JPRB, 24110.0_JPRB, 24120.0_JPRB, 24130.0_JPRB, 24140.0_JPRB, 24150.0_JPRB,  &
 24160.0_JPRB, 24170.0_JPRB, 24180.0_JPRB, 24190.0_JPRB, 24200.0_JPRB, 24210.0_JPRB, 24220.0_JPRB,  &
 24230.0_JPRB, 24240.0_JPRB, 24250.0_JPRB, 24260.0_JPRB, 24270.0_JPRB, 24280.0_JPRB, 24290.0_JPRB,  &
 24300.0_JPRB, 24310.0_JPRB, 24320.0_JPRB, 24330.0_JPRB, 24340.0_JPRB, 24350.0_JPRB, 24360.0_JPRB,  &
 24370.0_JPRB, 24380.0_JPRB, 24390.0_JPRB, 24400.0_JPRB, 24410.0_JPRB, 24420.0_JPRB, 24430.0_JPRB,  &
 24440.0_JPRB, 24450.0_JPRB, 24460.0_JPRB, 24470.0_JPRB, 24480.0_JPRB, 24490.0_JPRB, 24500.0_JPRB,  &
 24510.0_JPRB, 24520.0_JPRB, 24530.0_JPRB, 24540.0_JPRB, 24550.0_JPRB, 24560.0_JPRB, 24570.0_JPRB,  &
 24580.0_JPRB, 24590.0_JPRB, 24600.0_JPRB, 24610.0_JPRB, 24620.0_JPRB, 24630.0_JPRB, 24640.0_JPRB,  &
 24650.0_JPRB, 24660.0_JPRB, 24670.0_JPRB, 24680.0_JPRB, 24690.0_JPRB, 24700.0_JPRB, 24710.0_JPRB,  &
 24720.0_JPRB, 24730.0_JPRB, 24740.0_JPRB, 24750.0_JPRB, 24760.0_JPRB, 24770.0_JPRB, 24780.0_JPRB,  &
 24790.0_JPRB, 24800.0_JPRB, 24810.0_JPRB, 24820.0_JPRB, 24830.0_JPRB, 24840.0_JPRB, 24850.0_JPRB,  &
 24860.0_JPRB, 24870.0_JPRB, 24880.0_JPRB, 24890.0_JPRB, 24900.0_JPRB, 24910.0_JPRB, 24920.0_JPRB,  &
 24930.0_JPRB, 24940.0_JPRB, 24950.0_JPRB, 24960.0_JPRB, 24970.0_JPRB, 24980.0_JPRB, 24990.0_JPRB,  &
 25000.0_JPRB /

  DATA (pcm(i), i=1, 1000) / &
  24.8277779_JPRB,  25.0408955_JPRB,  25.3756046_JPRB,  25.7037067_JPRB,  26.0310898_JPRB,  &
  26.3604908_JPRB,  26.7347832_JPRB,  27.1387291_JPRB,  27.5377216_JPRB,  27.9030590_JPRB,  &
  28.2610722_JPRB,  28.6590042_JPRB,  29.0834980_JPRB,  29.4976254_JPRB,  29.8795204_JPRB,  &
  30.1842937_JPRB,  30.4282646_JPRB,  30.6161823_JPRB,  30.7734871_JPRB,  30.9610100_JPRB,  &
  31.2550640_JPRB,  31.5511723_JPRB,  31.6682377_JPRB,  31.6455383_JPRB,  31.3637829_JPRB,  &
  31.0997677_JPRB,  30.9445992_JPRB,  30.9195175_JPRB,  31.0114574_JPRB,  31.1538925_JPRB,  &
  31.3271103_JPRB,  31.5918617_JPRB,  31.9364128_JPRB,  32.5347023_JPRB,  33.1924286_JPRB,  &
  33.7807274_JPRB,  34.3048897_JPRB,  34.8109322_JPRB,  35.2768326_JPRB,  35.6295013_JPRB,  &
  35.8979759_JPRB,  36.0899544_JPRB,  36.2127190_JPRB,  36.3239937_JPRB,  36.4798775_JPRB,  &
  36.7381706_JPRB,  37.0174065_JPRB,  37.2550430_JPRB,  37.3141747_JPRB,  37.1758308_JPRB,  &
  36.7321091_JPRB,  36.0366364_JPRB,  35.4615135_JPRB,  35.1427155_JPRB,  35.2301254_JPRB,  &
  35.6141052_JPRB,  35.9188957_JPRB,  36.1277504_JPRB,  36.3305817_JPRB,  36.5077209_JPRB,  &
  36.6811256_JPRB,  36.8497658_JPRB,  37.0669022_JPRB,  37.4394073_JPRB,  37.8410454_JPRB,  &
  38.2695389_JPRB,  38.6767044_JPRB,  38.9945450_JPRB,  39.2578239_JPRB,  39.4313698_JPRB,  &
  39.5493889_JPRB,  39.6527290_JPRB,  39.7408524_JPRB,  39.8403931_JPRB,  39.9821548_JPRB,  &
  40.1356277_JPRB,  40.2465286_JPRB,  40.3448677_JPRB,  40.4148369_JPRB,  40.4683533_JPRB,  &
  40.5087280_JPRB,  40.5428581_JPRB,  40.5748482_JPRB,  40.5822258_JPRB,  40.5920639_JPRB,  &
  40.5929642_JPRB,  40.5849724_JPRB,  40.5702782_JPRB,  40.5456543_JPRB,  40.5149002_JPRB,  &
  40.4415970_JPRB,  40.3617287_JPRB,  40.2473755_JPRB,  40.1151505_JPRB,  39.9589920_JPRB,  &
  39.7548714_JPRB,  39.5531769_JPRB,  39.3287430_JPRB,  39.0908279_JPRB,  38.8686562_JPRB,  &
  38.6769829_JPRB,  38.4995613_JPRB,  38.3691139_JPRB,  38.2617950_JPRB,  38.1423302_JPRB,  &
  38.0185242_JPRB,  37.8886223_JPRB,  37.7509346_JPRB,  37.6172295_JPRB,  37.4692764_JPRB,  &
  37.3169022_JPRB,  37.1636925_JPRB,  37.0170670_JPRB,  36.8825645_JPRB,  36.7567787_JPRB,  &
  36.6402168_JPRB,  36.5216675_JPRB,  36.3995590_JPRB,  36.2845993_JPRB,  36.1637955_JPRB,  &
  36.0548248_JPRB,  35.9530907_JPRB,  35.8764610_JPRB,  36.0405540_JPRB,  36.3078537_JPRB,  &
  36.6830826_JPRB,  37.5555115_JPRB,  38.4996796_JPRB,  39.5082893_JPRB,  40.6039925_JPRB,  &
  41.6787682_JPRB,  42.6532631_JPRB,  43.5256271_JPRB,  44.3674774_JPRB,  45.0891762_JPRB,  &
  45.7204056_JPRB,  46.3044815_JPRB,  46.6977043_JPRB,  47.0319710_JPRB,  47.3322449_JPRB,  &
  47.5031433_JPRB,  47.6438637_JPRB,  47.7786484_JPRB,  47.8841782_JPRB,  47.9842186_JPRB,  &
  48.0831642_JPRB,  48.1158714_JPRB,  48.1428604_JPRB,  48.1731644_JPRB,  48.2052116_JPRB,  &
  48.2262611_JPRB,  48.2345009_JPRB,  48.2307587_JPRB,  48.2203102_JPRB,  48.1980476_JPRB,  &
  48.1617775_JPRB,  48.1188393_JPRB,  48.0772324_JPRB,  48.0221825_JPRB,  47.9725037_JPRB,  &
  47.8647423_JPRB,  47.7546425_JPRB,  47.6628494_JPRB,  47.6102982_JPRB,  47.5757103_JPRB,  &
  47.5605965_JPRB,  47.5695610_JPRB,  47.5893745_JPRB,  47.6571159_JPRB,  47.7293015_JPRB,  &
  47.8073463_JPRB,  47.8942604_JPRB,  47.9880371_JPRB,  48.0909271_JPRB,  48.1990356_JPRB,  &
  48.3001366_JPRB,  48.3927689_JPRB,  48.4773979_JPRB,  48.5763664_JPRB,  48.6816711_JPRB,  &
  48.7880669_JPRB,  48.8990593_JPRB,  49.0132561_JPRB,  49.1233025_JPRB,  49.2419930_JPRB,  &
  49.3548508_JPRB,  49.4367981_JPRB,  49.5028229_JPRB,  49.5608177_JPRB,  49.6137123_JPRB,  &
  49.6512108_JPRB,  49.6875305_JPRB,  49.7134285_JPRB,  49.7348328_JPRB,  49.7571220_JPRB,  &
  49.7906685_JPRB,  49.8226013_JPRB,  49.8518486_JPRB,  49.8824654_JPRB,  49.9062805_JPRB,  &
  49.9288368_JPRB,  49.9525146_JPRB,  49.9775429_JPRB,  49.9752960_JPRB,  49.9725151_JPRB,  &
  49.9668655_JPRB,  49.9510574_JPRB,  49.9350204_JPRB,  49.9062233_JPRB,  49.8749199_JPRB,  &
  49.8329430_JPRB,  49.7877769_JPRB,  49.7458458_JPRB,  49.7070007_JPRB,  49.6658325_JPRB,  &
  49.6287918_JPRB,  49.5949745_JPRB,  49.5527306_JPRB,  49.5147476_JPRB,  49.4577179_JPRB,  &
  49.4103966_JPRB,  49.3577080_JPRB,  49.3042564_JPRB,  49.2495346_JPRB,  49.1900711_JPRB,  &
  49.1349220_JPRB,  49.0783005_JPRB,  49.0104485_JPRB,  48.9424515_JPRB,  48.8712959_JPRB,  &
  48.7976608_JPRB,  48.7201958_JPRB,  48.6492691_JPRB,  48.5846863_JPRB,  48.5148926_JPRB,  &
  48.4375496_JPRB,  48.3640823_JPRB,  48.2843971_JPRB,  48.2191467_JPRB,  48.1516533_JPRB,  &
  48.0809593_JPRB,  48.0056038_JPRB,  47.9305763_JPRB,  47.8318100_JPRB,  47.7247353_JPRB,  &
  47.6254768_JPRB,  47.5232315_JPRB,  47.4170990_JPRB,  47.3067856_JPRB,  47.1959763_JPRB,  &
  47.1025696_JPRB,  47.0158081_JPRB,  46.9364243_JPRB,  46.8515549_JPRB,  46.7669792_JPRB,  &
  46.6809807_JPRB,  46.5889931_JPRB,  46.4778671_JPRB,  46.3589363_JPRB,  46.2331390_JPRB,  &
  46.1061630_JPRB,  45.9807281_JPRB,  45.8536797_JPRB,  45.7196045_JPRB,  45.5729408_JPRB,  &
  45.4256516_JPRB,  45.2711411_JPRB,  45.1085663_JPRB,  44.9622421_JPRB,  44.8323746_JPRB,  &
  44.6884499_JPRB,  44.5197906_JPRB,  44.3317146_JPRB,  44.0937538_JPRB,  43.8393288_JPRB,  &
  43.5828819_JPRB,  43.3454704_JPRB,  43.2157364_JPRB,  43.1692314_JPRB,  43.1494408_JPRB,  &
  43.1466866_JPRB,  43.1388931_JPRB,  43.0929909_JPRB,  43.0425720_JPRB,  42.9774513_JPRB,  &
  42.9205513_JPRB,  42.8812256_JPRB,  42.8628807_JPRB,  42.8499832_JPRB,  42.8666153_JPRB,  &
  42.8766289_JPRB,  42.8651009_JPRB,  42.8294296_JPRB,  42.7809982_JPRB,  42.6996040_JPRB,  &
  42.6092682_JPRB,  42.4630814_JPRB,  42.2992706_JPRB,  42.1220589_JPRB,  41.9337540_JPRB,  &
  41.7471809_JPRB,  41.5761147_JPRB,  41.4013367_JPRB,  41.2029266_JPRB,  40.9854431_JPRB,  &
  40.7861519_JPRB,  40.6723328_JPRB,  40.6026917_JPRB,  40.7962875_JPRB,  41.2006149_JPRB,  &
  41.6306000_JPRB,  42.0552292_JPRB,  42.4793282_JPRB,  42.8819809_JPRB,  43.2628098_JPRB,  &
  43.6393890_JPRB,  43.9990768_JPRB,  44.3819275_JPRB,  44.8599586_JPRB,  45.4965630_JPRB,  &
  46.1477737_JPRB,  46.7114563_JPRB,  47.2458572_JPRB,  47.7549171_JPRB,  48.1832123_JPRB,  &
  48.5828934_JPRB,  48.9394531_JPRB,  49.2532997_JPRB,  49.5449371_JPRB,  49.7617188_JPRB,  &
  49.9556465_JPRB,  50.1164284_JPRB,  50.2043686_JPRB,  50.2867813_JPRB,  50.3782043_JPRB,  &
  50.4792099_JPRB,  50.5746956_JPRB,  50.6758156_JPRB,  50.7880707_JPRB,  50.8856621_JPRB,  &
  50.9989624_JPRB,  51.1025085_JPRB,  51.2065086_JPRB,  51.3081245_JPRB,  51.4042892_JPRB,  &
  51.4893990_JPRB,  51.5658188_JPRB,  51.6339455_JPRB,  51.7216644_JPRB,  51.8101959_JPRB,  &
  51.8976097_JPRB,  51.9952774_JPRB,  52.0927658_JPRB,  52.1877747_JPRB,  52.2839622_JPRB,  &
  52.3732262_JPRB,  52.4584694_JPRB,  52.5331650_JPRB,  52.6083336_JPRB,  52.6867561_JPRB,  &
  52.7561340_JPRB,  52.8231392_JPRB,  52.8769035_JPRB,  52.9340477_JPRB,  52.9769707_JPRB,  &
  53.0189934_JPRB,  53.0571861_JPRB,  53.1026077_JPRB,  53.1264534_JPRB,  53.1512184_JPRB,  &
  53.1757889_JPRB,  53.1957359_JPRB,  53.2113342_JPRB,  53.2238350_JPRB,  53.2309532_JPRB,  &
  53.2288017_JPRB,  53.2214279_JPRB,  53.2135010_JPRB,  53.2051048_JPRB,  53.1969490_JPRB,  &
  53.1745415_JPRB,  53.1487122_JPRB,  53.1208115_JPRB,  53.0874443_JPRB,  53.0537109_JPRB,  &
  53.0197678_JPRB,  52.9902916_JPRB,  52.9605255_JPRB,  52.9354286_JPRB,  52.9177933_JPRB,  &
  52.9032173_JPRB,  52.8898621_JPRB,  52.8802643_JPRB,  52.8731766_JPRB,  52.8647499_JPRB,  &
  52.8530045_JPRB,  52.8380737_JPRB,  52.8203926_JPRB,  52.8020592_JPRB,  52.7679634_JPRB,  &
  52.7333336_JPRB,  52.7019920_JPRB,  52.6730728_JPRB,  52.6412010_JPRB,  52.6088791_JPRB,  &
  52.5843277_JPRB,  52.5531616_JPRB,  52.5271034_JPRB,  52.4952164_JPRB,  52.4646378_JPRB,  &
  52.4378052_JPRB,  52.4118347_JPRB,  52.3874474_JPRB,  52.3652267_JPRB,  52.3381195_JPRB,  &
  52.3142014_JPRB,  52.2914925_JPRB,  52.2736092_JPRB,  52.2451439_JPRB,  52.2222404_JPRB,  &
  52.1886978_JPRB,  52.1623955_JPRB,  52.1310196_JPRB,  52.1028595_JPRB,  52.0730438_JPRB,  &
  52.0445671_JPRB,  52.0113258_JPRB,  51.9817390_JPRB,  51.9564095_JPRB,  51.9228745_JPRB,  &
  51.8925934_JPRB,  51.8659782_JPRB,  51.8516502_JPRB,  51.8358459_JPRB,  51.8187065_JPRB,  &
  51.8114624_JPRB,  51.8038597_JPRB,  51.7975960_JPRB,  51.7957802_JPRB,  51.7862206_JPRB,  &
  51.7749062_JPRB,  51.7648163_JPRB,  51.7499771_JPRB,  51.7300301_JPRB,  51.7053070_JPRB,  &
  51.6875572_JPRB,  51.6764526_JPRB,  51.6724510_JPRB,  51.6580887_JPRB,  51.6512489_JPRB,  &
  51.6510277_JPRB,  51.6447945_JPRB,  51.6359177_JPRB,  51.6288338_JPRB,  51.6105728_JPRB,  &
  51.5943718_JPRB,  51.5859909_JPRB,  51.5827827_JPRB,  51.5809479_JPRB,  51.5743904_JPRB,  &
  51.5832405_JPRB,  51.5928879_JPRB,  51.6025124_JPRB,  51.6131096_JPRB,  51.6350632_JPRB,  &
  51.6606941_JPRB,  51.6791229_JPRB,  51.7093048_JPRB,  51.7432480_JPRB,  51.7822037_JPRB,  &
  51.8244705_JPRB,  51.8735008_JPRB,  51.9337273_JPRB,  52.0021095_JPRB,  52.0662193_JPRB,  &
  52.1219902_JPRB,  52.1815948_JPRB,  52.2372551_JPRB,  52.2917328_JPRB,  52.3450737_JPRB,  &
  52.3958054_JPRB,  52.4485970_JPRB,  52.4963264_JPRB,  52.5285416_JPRB,  52.5456848_JPRB,  &
  52.5682220_JPRB,  52.5838470_JPRB,  52.5915985_JPRB,  52.6025658_JPRB,  52.6067963_JPRB,  &
  52.6079178_JPRB,  52.6029167_JPRB,  52.5942574_JPRB,  52.5815468_JPRB,  52.5732155_JPRB,  &
  52.5633888_JPRB,  52.5515327_JPRB,  52.5406914_JPRB,  52.5288696_JPRB,  52.5188370_JPRB,  &
  52.5111122_JPRB,  52.5029030_JPRB,  52.4930000_JPRB,  52.4852066_JPRB,  52.4742317_JPRB,  &
  52.4616852_JPRB,  52.4498520_JPRB,  52.4321480_JPRB,  52.4159622_JPRB,  52.3998108_JPRB,  &
  52.3804893_JPRB,  52.3675957_JPRB,  52.3594437_JPRB,  52.3474426_JPRB,  52.3394012_JPRB,  &
  52.3224907_JPRB,  52.3066368_JPRB,  52.2871361_JPRB,  52.2686691_JPRB,  52.2519913_JPRB,  &
  52.2347488_JPRB,  52.2113647_JPRB,  52.1953735_JPRB,  52.1752243_JPRB,  52.1597404_JPRB,  &
  52.1403046_JPRB,  52.1238899_JPRB,  52.1051445_JPRB,  52.0915756_JPRB,  52.0763893_JPRB,  &
  52.0601921_JPRB,  52.0443954_JPRB,  52.0285988_JPRB,  52.0057831_JPRB,  51.9835510_JPRB,  &
  51.9618835_JPRB,  51.9426765_JPRB,  51.9239616_JPRB,  51.9005165_JPRB,  51.8794785_JPRB,  &
  51.8587990_JPRB,  51.8369713_JPRB,  51.8125839_JPRB,  51.7946358_JPRB,  51.7740822_JPRB,  &
  51.7539864_JPRB,  51.7282219_JPRB,  51.7064743_JPRB,  51.6843719_JPRB,  51.6619110_JPRB,  &
  51.6393089_JPRB,  51.6160889_JPRB,  51.5870743_JPRB,  51.5575371_JPRB,  51.5361748_JPRB,  &
  51.5095062_JPRB,  51.4877815_JPRB,  51.4607735_JPRB,  51.4419670_JPRB,  51.4202919_JPRB,  &
  51.4010620_JPRB,  51.3809471_JPRB,  51.3631134_JPRB,  51.3402901_JPRB,  51.3124466_JPRB,  &
  51.2802010_JPRB,  51.2503929_JPRB,  51.2237434_JPRB,  51.1952133_JPRB,  51.1688309_JPRB,  &
  51.1414261_JPRB,  51.1203613_JPRB,  51.0967331_JPRB,  51.0638733_JPRB,  51.0370522_JPRB,  &
  51.0052795_JPRB,  50.9706154_JPRB,  50.9383621_JPRB,  50.9037666_JPRB,  50.8628426_JPRB,  &
  50.8253746_JPRB,  50.7880478_JPRB,  50.7480202_JPRB,  50.7141953_JPRB,  50.6797256_JPRB,  &
  50.6510429_JPRB,  50.6217155_JPRB,  50.5882530_JPRB,  50.5581360_JPRB,  50.5372543_JPRB,  &
  50.5106659_JPRB,  50.4889908_JPRB,  50.4598312_JPRB,  50.4428482_JPRB,  50.4221764_JPRB,  &
  50.3889351_JPRB,  50.3576393_JPRB,  50.3225365_JPRB,  50.2929764_JPRB,  50.2579727_JPRB,  &
  50.2222214_JPRB,  50.1976433_JPRB,  50.1746941_JPRB,  50.1478615_JPRB,  50.1312180_JPRB,  &
  50.1038666_JPRB,  50.0832787_JPRB,  50.0704613_JPRB,  50.0520058_JPRB,  50.0398445_JPRB,  &
  50.0267334_JPRB,  50.0163155_JPRB,  50.0069122_JPRB,  49.9936562_JPRB,  49.9829483_JPRB,  &
  49.9692192_JPRB,  49.9593430_JPRB,  49.9469185_JPRB,  49.9347458_JPRB,  49.9212189_JPRB,  &
  49.9120140_JPRB,  49.9000587_JPRB,  49.8828354_JPRB,  49.8688316_JPRB,  49.8582878_JPRB,  &
  49.8416824_JPRB,  49.8188858_JPRB,  49.8003883_JPRB,  49.7844124_JPRB,  49.7631149_JPRB,  &
  49.7411499_JPRB,  49.7240791_JPRB,  49.7277603_JPRB,  49.7255249_JPRB,  49.7225990_JPRB,  &
  49.7235184_JPRB,  49.7244034_JPRB,  49.7272987_JPRB,  49.7290535_JPRB,  49.7340431_JPRB,  &
  49.7380943_JPRB,  49.7374840_JPRB,  49.7377243_JPRB,  49.7411957_JPRB,  49.7446899_JPRB,  &
  49.7512856_JPRB,  49.7585487_JPRB,  49.7601433_JPRB,  49.7730370_JPRB,  49.7852173_JPRB,  &
  49.7938004_JPRB,  49.8022423_JPRB,  49.8105507_JPRB,  49.8190346_JPRB,  49.8340187_JPRB,  &
  49.8599854_JPRB,  49.8791466_JPRB,  49.8962135_JPRB,  49.9132423_JPRB,  49.9302406_JPRB,  &
  49.9448166_JPRB,  49.9586143_JPRB,  49.9748116_JPRB,  49.9914742_JPRB,  50.0030594_JPRB,  &
  50.0160561_JPRB,  50.0209503_JPRB,  50.0245743_JPRB,  50.0328445_JPRB,  50.0443077_JPRB,  &
  50.0511284_JPRB,  50.0550766_JPRB,  50.0624504_JPRB,  50.0567436_JPRB,  50.0464134_JPRB,  &
  50.0464554_JPRB,  50.0418129_JPRB,  50.0353546_JPRB,  50.0275078_JPRB,  50.0182991_JPRB,  &
  50.0115814_JPRB,  49.9986649_JPRB,  49.9969940_JPRB,  49.9932785_JPRB,  49.9854317_JPRB,  &
  49.9782829_JPRB,  49.9752884_JPRB,  49.9710770_JPRB,  49.9630280_JPRB,  49.9549904_JPRB,  &
  49.9537354_JPRB,  49.9515114_JPRB,  49.9452438_JPRB,  49.9397163_JPRB,  49.9324188_JPRB,  &
  49.9341507_JPRB,  49.9386711_JPRB,  49.9422760_JPRB,  49.9458580_JPRB,  49.9523811_JPRB,  &
  49.9523811_JPRB,  49.9547119_JPRB,  49.9586601_JPRB,  49.9626236_JPRB,  49.9655609_JPRB,  &
  49.9674263_JPRB,  49.9661255_JPRB,  49.9658737_JPRB,  49.9669991_JPRB,  49.9691620_JPRB,  &
  49.9715080_JPRB,  49.9721489_JPRB,  49.9716530_JPRB,  49.9718475_JPRB,  49.9727516_JPRB,  &
  49.9733696_JPRB,  49.9708176_JPRB,  49.9688606_JPRB,  49.9712830_JPRB,  49.9716225_JPRB,  &
  49.9664421_JPRB,  49.9607506_JPRB,  49.9544907_JPRB,  49.9515419_JPRB,  49.9500313_JPRB,  &
  49.9477959_JPRB,  49.9434090_JPRB,  49.9423103_JPRB,  49.9394264_JPRB,  49.9348564_JPRB,  &
  49.9301224_JPRB,  49.9295044_JPRB,  49.9240608_JPRB,  49.9237061_JPRB,  49.9297562_JPRB,  &
  49.9353065_JPRB,  49.9357147_JPRB,  49.9364967_JPRB,  49.9389839_JPRB,  49.9436493_JPRB,  &
  49.9436493_JPRB,  49.9500465_JPRB,  49.9561234_JPRB,  49.9587288_JPRB,  49.9587288_JPRB,  &
  49.9617653_JPRB,  49.9663429_JPRB,  49.9707603_JPRB,  49.9713631_JPRB,  49.9714279_JPRB,  &
  49.9734993_JPRB,  49.9766426_JPRB,  49.9784393_JPRB,  49.9791260_JPRB,  49.9800758_JPRB,  &
  49.9815826_JPRB,  49.9839630_JPRB,  49.9846802_JPRB,  49.9863243_JPRB,  49.9880943_JPRB,  &
  49.9880943_JPRB,  49.9896965_JPRB,  49.9894905_JPRB,  49.9895668_JPRB,  49.9966049_JPRB,  &
  49.9956169_JPRB,  49.9964828_JPRB,  50.0000000_JPRB,  50.0000000_JPRB,  49.9968452_JPRB,  &
  49.9953918_JPRB,  49.9976425_JPRB,  49.9982185_JPRB,  49.9976425_JPRB,  49.9966583_JPRB,  &
  49.9960823_JPRB,  49.9860840_JPRB,  49.9757996_JPRB,  49.9660645_JPRB,  49.9563522_JPRB,  &
  49.9436417_JPRB,  49.9303856_JPRB,  49.9178734_JPRB,  49.9114265_JPRB,  49.9006920_JPRB,  &
  49.8869209_JPRB,  49.8750648_JPRB,  49.8820724_JPRB,  49.8952179_JPRB,  49.9087067_JPRB,  &
  49.9226913_JPRB,  49.9384460_JPRB,  49.9551392_JPRB,  49.9662971_JPRB,  49.9789619_JPRB,  &
  49.9923134_JPRB,  50.0056419_JPRB,  50.0198593_JPRB,  50.0290070_JPRB,  50.0301132_JPRB,  &
  50.0306892_JPRB,  50.0315247_JPRB,  50.0325394_JPRB,  50.0325394_JPRB,  50.0366821_JPRB,  &
  50.0424576_JPRB,  50.0446434_JPRB,  50.0444450_JPRB,  50.0435028_JPRB,  50.0429611_JPRB,  &
  50.0410995_JPRB,  50.0381851_JPRB,  50.0338593_JPRB,  50.0290680_JPRB,  50.0268478_JPRB,  &
  50.0284653_JPRB,  50.0255661_JPRB,  50.0255165_JPRB,  50.0292664_JPRB,  50.0288467_JPRB,  &
  50.0309105_JPRB,  50.0357094_JPRB,  50.0378380_JPRB,  50.0396957_JPRB,  50.0412788_JPRB,  &
  50.0423393_JPRB,  50.0442009_JPRB,  50.0468445_JPRB,  50.0500031_JPRB,  50.0515862_JPRB,  &
  50.0515862_JPRB,  50.0526237_JPRB,  50.0519371_JPRB,  50.0493240_JPRB,  50.0447273_JPRB,  &
  50.0386925_JPRB,  50.0308914_JPRB,  50.0273399_JPRB,  50.0226707_JPRB,  50.0154266_JPRB,  &
  50.0147209_JPRB,  50.0150795_JPRB,  50.0150795_JPRB,  50.0154724_JPRB,  50.0163231_JPRB,  &
  50.0183716_JPRB,  50.0180206_JPRB,  50.0166054_JPRB,  50.0160980_JPRB,  50.0155869_JPRB,  &
  50.0150795_JPRB,  50.0094986_JPRB,  50.0067902_JPRB,  50.0078011_JPRB,  50.0048714_JPRB,  &
  50.0004501_JPRB,  49.9939041_JPRB,  49.9873734_JPRB,  49.9813309_JPRB,  49.9783249_JPRB,  &
  49.9720001_JPRB,  49.9630051_JPRB,  49.9660683_JPRB,  49.9688110_JPRB,  49.9703026_JPRB,  &
  49.9748764_JPRB,  49.9796295_JPRB,  49.9811134_JPRB,  49.9752426_JPRB,  49.9638939_JPRB,  &
  49.9525757_JPRB,  49.9417839_JPRB,  49.9319649_JPRB,  49.9225235_JPRB,  49.9138374_JPRB,  &
  49.9099312_JPRB,  49.9088898_JPRB,  49.9108391_JPRB,  49.9056740_JPRB,  49.8997993_JPRB,  &
  49.8954391_JPRB,  49.8925095_JPRB,  49.8905754_JPRB,  49.8859024_JPRB,  49.8807907_JPRB,  &
  49.8750191_JPRB,  49.8676910_JPRB,  49.8595352_JPRB,  49.8509216_JPRB,  49.8418274_JPRB,  &
  49.8318062_JPRB,  49.8179283_JPRB,  49.8022957_JPRB,  49.7942238_JPRB,  49.7859993_JPRB,  &
  49.7774734_JPRB,  49.7667694_JPRB,  49.7549629_JPRB,  49.7385483_JPRB,  49.7234764_JPRB,  &
  49.7117271_JPRB,  49.7011414_JPRB,  49.6912956_JPRB,  49.6767616_JPRB,  49.6633415_JPRB,  &
  49.6554146_JPRB,  49.6472588_JPRB,  49.6388893_JPRB,  49.6279030_JPRB,  49.6166306_JPRB,  &
  49.6083031_JPRB,  49.5962639_JPRB,  49.5782700_JPRB,  49.5603027_JPRB,  49.5423546_JPRB,  &
  49.5231552_JPRB,  49.5037918_JPRB,  49.4841080_JPRB,  49.4644470_JPRB,  49.4448128_JPRB,  &
  49.4255524_JPRB,  49.4064255_JPRB,  49.3850861_JPRB,  49.3621750_JPRB,  49.3345413_JPRB,  &
  49.3073959_JPRB,  49.2807388_JPRB,  49.2501335_JPRB,  49.2181396_JPRB,  49.1827126_JPRB,  &
  49.1461792_JPRB,  49.1058388_JPRB,  49.0671272_JPRB,  49.0304718_JPRB,  48.9971504_JPRB,  &
  48.9654961_JPRB,  48.9351006_JPRB,  48.9032974_JPRB,  48.8576202_JPRB,  48.8112907_JPRB,  &
  48.7635269_JPRB,  48.7192497_JPRB,  48.6782303_JPRB,  48.6360245_JPRB,  48.5933533_JPRB,  &
  48.5451736_JPRB,  48.4958916_JPRB,  48.4416351_JPRB,  48.3870430_JPRB,  48.3316231_JPRB,  &
  48.2760696_JPRB,  48.2203636_JPRB,  48.1568756_JPRB,  48.0891685_JPRB,  48.0233879_JPRB,  &
  47.9580269_JPRB,  47.8983803_JPRB,  47.8384514_JPRB,  47.7764587_JPRB,  47.7113800_JPRB,  &
  47.6392517_JPRB,  47.5664330_JPRB,  47.4927902_JPRB,  47.4156647_JPRB,  47.3362427_JPRB,  &
  47.2579308_JPRB,  47.1800041_JPRB,  47.1000214_JPRB,  47.0197868_JPRB,  46.9422836_JPRB,  &
  46.8630714_JPRB,  46.7731094_JPRB,  46.6837540_JPRB,  46.5961456_JPRB,  46.5114517_JPRB,  &
  46.4316444_JPRB,  46.3515358_JPRB,  46.2711067_JPRB,  46.1839142_JPRB,  46.0920143_JPRB,  &
  46.0002861_JPRB,  45.9086380_JPRB,  45.8165321_JPRB,  45.7243080_JPRB,  45.6323013_JPRB,  &
  45.5403290_JPRB,  45.4414749_JPRB,  45.3423462_JPRB,  45.2438774_JPRB,  45.1460304_JPRB,  &
  45.0531769_JPRB,  44.9611778_JPRB,  44.8730888_JPRB,  44.7848625_JPRB,  44.6962013_JPRB,  &
  44.6074944_JPRB,  44.5186729_JPRB,  44.4296494_JPRB,  44.3402634_JPRB,  44.2506561_JPRB /
  
  DATA (pcm(i), i=1001, 2101) / &
  44.1607094_JPRB,  44.0731812_JPRB,  43.9887390_JPRB,  43.9055557_JPRB,  43.8237686_JPRB,  &
  43.7400627_JPRB,  43.6544876_JPRB,  43.5656395_JPRB,  43.4738922_JPRB,  43.3855782_JPRB,  &
  43.3000793_JPRB,  43.2144966_JPRB,  43.1288414_JPRB,  43.0444603_JPRB,  42.9610329_JPRB,  &
  42.8788757_JPRB,  42.7976685_JPRB,  42.7186127_JPRB,  42.6411896_JPRB,  42.5643272_JPRB,  &
  42.4879112_JPRB,  42.4143944_JPRB,  42.3432999_JPRB,  42.2731361_JPRB,  42.2038078_JPRB,  &
  42.1262093_JPRB,  42.0402870_JPRB,  41.9597473_JPRB,  41.8853607_JPRB,  41.8125687_JPRB,  &
  41.7418900_JPRB,  41.6727829_JPRB,  41.6061783_JPRB,  41.5405464_JPRB,  41.4768372_JPRB,  &
  41.4141502_JPRB,  41.3540878_JPRB,  41.2948189_JPRB,  41.2383842_JPRB,  41.1822052_JPRB,  &
  41.1274643_JPRB,  41.0734787_JPRB,  41.0284386_JPRB,  40.9833984_JPRB,  40.9400063_JPRB,  &
  40.8966179_JPRB,  40.8550797_JPRB,  40.8137131_JPRB,  40.7774200_JPRB,  40.7422295_JPRB,  &
  40.7087517_JPRB,  40.6759262_JPRB,  40.6424599_JPRB,  40.6086006_JPRB,  40.5786285_JPRB,  &
  40.5523720_JPRB,  40.5258446_JPRB,  40.4989166_JPRB,  40.4718018_JPRB,  40.4442024_JPRB,  &
  40.4175949_JPRB,  40.3967705_JPRB,  40.3760033_JPRB,  40.3600616_JPRB,  40.3441162_JPRB,  &
  40.3246727_JPRB,  40.3047104_JPRB,  40.2845230_JPRB,  40.2642479_JPRB,  40.2461090_JPRB,  &
  40.2295685_JPRB,  40.2119942_JPRB,  40.1929398_JPRB,  40.1752472_JPRB,  40.1617279_JPRB,  &
  40.1482124_JPRB,  40.1347351_JPRB,  40.1212540_JPRB,  40.1068344_JPRB,  40.0923080_JPRB,  &
  40.0796089_JPRB,  40.0676575_JPRB,  40.0519829_JPRB,  40.0328445_JPRB,  40.0155716_JPRB,  &
  40.0022507_JPRB,  39.9889793_JPRB,  39.9760551_JPRB,  39.9631348_JPRB,  39.9518661_JPRB,  &
  39.9407730_JPRB,  39.9260254_JPRB,  39.9096107_JPRB,  39.8932228_JPRB,  39.8768578_JPRB,  &
  39.8610001_JPRB,  39.8468132_JPRB,  39.8326263_JPRB,  39.8258629_JPRB,  39.8191414_JPRB,  &
  39.8177261_JPRB,  39.8180771_JPRB,  39.8131714_JPRB,  39.8029785_JPRB,  39.7918663_JPRB,  &
  39.7778473_JPRB,  39.7638283_JPRB,  39.7553101_JPRB,  39.7469215_JPRB,  39.7385559_JPRB,  &
  39.7301941_JPRB,  39.7209587_JPRB,  39.7105408_JPRB,  39.6998329_JPRB,  39.6873703_JPRB,  &
  39.6749039_JPRB,  39.6656952_JPRB,  39.6570625_JPRB,  39.6512833_JPRB,  39.6478424_JPRB,  &
  39.6415672_JPRB,  39.6264725_JPRB,  39.6113739_JPRB,  39.5991821_JPRB,  39.5872116_JPRB,  &
  39.5752602_JPRB,  39.5633278_JPRB,  39.5532265_JPRB,  39.5477867_JPRB,  39.5423470_JPRB,  &
  39.5340309_JPRB,  39.5255585_JPRB,  39.5175095_JPRB,  39.5097389_JPRB,  39.5021439_JPRB,  &
  39.4950714_JPRB,  39.4879951_JPRB,  39.4809418_JPRB,  39.4738884_JPRB,  39.4654083_JPRB,  &
  39.4556961_JPRB,  39.4457703_JPRB,  39.4347534_JPRB,  39.4237404_JPRB,  39.4153824_JPRB,  &
  39.4077301_JPRB,  39.4006233_JPRB,  39.3943214_JPRB,  39.3879433_JPRB,  39.3757095_JPRB,  &
  39.3634720_JPRB,  39.3536835_JPRB,  39.3454399_JPRB,  39.3374062_JPRB,  39.3301773_JPRB,  &
  39.3229485_JPRB,  39.3178062_JPRB,  39.3132172_JPRB,  39.3081551_JPRB,  39.3022766_JPRB,  &
  39.2963982_JPRB,  39.2932854_JPRB,  39.2903557_JPRB,  39.2858047_JPRB,  39.2796402_JPRB,  &
  39.2733307_JPRB,  39.2645950_JPRB,  39.2558594_JPRB,  39.2483025_JPRB,  39.2415314_JPRB,  &
  39.2345772_JPRB,  39.2265396_JPRB,  39.2185020_JPRB,  39.2100487_JPRB,  39.2013931_JPRB,  &
  39.1928711_JPRB,  39.1848831_JPRB,  39.1768951_JPRB,  39.1696091_JPRB,  39.1626015_JPRB,  &
  39.1544037_JPRB,  39.1420212_JPRB,  39.1296387_JPRB,  39.1166000_JPRB,  39.1033058_JPRB,  &
  39.0901566_JPRB,  39.0775375_JPRB,  39.0649185_JPRB,  39.0551796_JPRB,  39.0466881_JPRB,  &
  39.0380936_JPRB,  39.0289993_JPRB,  39.0199089_JPRB,  39.0114403_JPRB,  39.0033150_JPRB,  &
  38.9953842_JPRB,  38.9891548_JPRB,  38.9829254_JPRB,  38.9763603_JPRB,  38.9695282_JPRB,  &
  38.9626999_JPRB,  38.9484596_JPRB,  38.9342232_JPRB,  38.9233665_JPRB,  38.9168854_JPRB,  &
  38.9104080_JPRB,  38.9007607_JPRB,  38.8906097_JPRB,  38.8807297_JPRB,  38.8715324_JPRB,  &
  38.8623390_JPRB,  38.8516769_JPRB,  38.8403702_JPRB,  38.8293648_JPRB,  38.8211403_JPRB,  &
  38.8129158_JPRB,  38.8053017_JPRB,  38.7983170_JPRB,  38.7913361_JPRB,  38.7822495_JPRB,  &
  38.7728691_JPRB,  38.7637253_JPRB,  38.7552795_JPRB,  38.7468300_JPRB,  38.7378387_JPRB,  &
  38.7285194_JPRB,  38.7192001_JPRB,  38.7160988_JPRB,  38.7131042_JPRB,  38.7093086_JPRB,  &
  38.7039299_JPRB,  38.6985550_JPRB,  38.6919746_JPRB,  38.6848297_JPRB,  38.6776543_JPRB,  &
  38.6681557_JPRB,  38.6586609_JPRB,  38.6500778_JPRB,  38.6432724_JPRB,  38.6364708_JPRB,  &
  38.6283264_JPRB,  38.6194801_JPRB,  38.6106377_JPRB,  38.6043701_JPRB,  38.5981979_JPRB,  &
  38.5919533_JPRB,  38.5855103_JPRB,  38.5790672_JPRB,  38.5736046_JPRB,  38.5689354_JPRB,  &
  38.5642624_JPRB,  38.5462418_JPRB,  38.5255814_JPRB,  38.5055542_JPRB,  38.4907646_JPRB,  &
  38.4759712_JPRB,  38.4659157_JPRB,  38.4638939_JPRB,  38.4618683_JPRB,  38.4560661_JPRB,  &
  38.4480019_JPRB,  38.4399338_JPRB,  38.4268913_JPRB,  38.4131088_JPRB,  38.3994522_JPRB,  &
  38.3868599_JPRB,  38.3742676_JPRB,  38.3613091_JPRB,  38.3476181_JPRB,  38.3339272_JPRB,  &
  38.3180656_JPRB,  38.3004417_JPRB,  38.2828140_JPRB,  38.2699661_JPRB,  38.2586327_JPRB,  &
  38.2472992_JPRB,  38.2403336_JPRB,  38.2335548_JPRB,  38.2267380_JPRB,  38.2197037_JPRB,  &
  38.2126656_JPRB,  38.2033386_JPRB,  38.1893158_JPRB,  38.1752892_JPRB,  38.1601753_JPRB,  &
  38.1439590_JPRB,  38.1277466_JPRB,  38.1148758_JPRB,  38.1037292_JPRB,  38.0925865_JPRB,  &
  38.0762939_JPRB,  38.0587997_JPRB,  38.0413055_JPRB,  38.0249214_JPRB,  38.0085945_JPRB,  &
  37.9921532_JPRB,  37.9745026_JPRB,  37.9568520_JPRB,  37.9401245_JPRB,  37.9269295_JPRB,  &
  37.9137383_JPRB,  37.9009972_JPRB,  37.8892174_JPRB,  37.8774376_JPRB,  37.8631363_JPRB,  &
  37.8453941_JPRB,  37.8276482_JPRB,  37.8100739_JPRB,  37.7926598_JPRB,  37.7752495_JPRB,  &
  37.7593307_JPRB,  37.7444191_JPRB,  37.7295074_JPRB,  37.7139015_JPRB,  37.6979599_JPRB,  &
  37.6820183_JPRB,  37.6686821_JPRB,  37.6562958_JPRB,  37.6439095_JPRB,  37.6292305_JPRB,  &
  37.6139336_JPRB,  37.5986366_JPRB,  37.5867119_JPRB,  37.5754814_JPRB,  37.5642471_JPRB,  &
  37.5502930_JPRB,  37.5359001_JPRB,  37.5215073_JPRB,  37.5048103_JPRB,  37.4878120_JPRB,  &
  37.4708138_JPRB,  37.4529190_JPRB,  37.4349174_JPRB,  37.4169197_JPRB,  37.4013443_JPRB,  &
  37.3860474_JPRB,  37.3707466_JPRB,  37.3536263_JPRB,  37.3362770_JPRB,  37.3189278_JPRB,  &
  37.3034554_JPRB,  37.2882614_JPRB,  37.2730713_JPRB,  37.2579231_JPRB,  37.2427826_JPRB,  &
  37.2276421_JPRB,  37.2088013_JPRB,  37.1890335_JPRB,  37.1692619_JPRB,  37.1514854_JPRB,  &
  37.1343765_JPRB,  37.1172676_JPRB,  37.1028595_JPRB,  37.0896873_JPRB,  37.0765114_JPRB,  &
  37.0611534_JPRB,  37.0444221_JPRB,  37.0276909_JPRB,  37.0130348_JPRB,  37.0002098_JPRB,  &
  36.9873848_JPRB,  36.9724579_JPRB,  36.9548225_JPRB,  36.9371872_JPRB,  36.9202461_JPRB,  &
  36.9047089_JPRB,  36.8891716_JPRB,  36.8728256_JPRB,  36.8535385_JPRB,  36.8342476_JPRB,  &
  36.8150330_JPRB,  36.7965698_JPRB,  36.7781105_JPRB,  36.7596474_JPRB,  36.7407684_JPRB,  &
  36.7218666_JPRB,  36.7029648_JPRB,  36.6841202_JPRB,  36.6652870_JPRB,  36.6464500_JPRB,  &
  36.6266975_JPRB,  36.6064301_JPRB,  36.5861626_JPRB,  36.5639420_JPRB,  36.5395088_JPRB,  &
  36.5150719_JPRB,  36.4915161_JPRB,  36.4701500_JPRB,  36.4487839_JPRB,  36.4279404_JPRB,  &
  36.4123459_JPRB,  36.3967514_JPRB,  36.3811531_JPRB,  36.3616753_JPRB,  36.3416977_JPRB,  &
  36.3217163_JPRB,  36.2965317_JPRB,  36.2687569_JPRB,  36.2409821_JPRB,  36.2143250_JPRB,  &
  36.1891022_JPRB,  36.1638756_JPRB,  36.1390076_JPRB,  36.1155815_JPRB,  36.0921555_JPRB,  &
  36.0687294_JPRB,  36.0453835_JPRB,  36.0220413_JPRB,  35.9986992_JPRB,  35.9755783_JPRB,  &
  35.9525642_JPRB,  35.9295502_JPRB,  35.9059792_JPRB,  35.8815994_JPRB,  35.8572197_JPRB,  &
  35.8324814_JPRB,  35.8050613_JPRB,  35.7776413_JPRB,  35.7502251_JPRB,  35.7244530_JPRB,  &
  35.6990509_JPRB,  35.6736488_JPRB,  35.6487770_JPRB,  35.6244240_JPRB,  35.6000710_JPRB,  &
  35.5758171_JPRB,  35.5520287_JPRB,  35.5282364_JPRB,  35.5044479_JPRB,  35.4815331_JPRB,  &
  35.4587784_JPRB,  35.4360237_JPRB,  35.4116631_JPRB,  35.3856850_JPRB,  35.3597031_JPRB,  &
  35.3340645_JPRB,  35.3105316_JPRB,  35.2870026_JPRB,  35.2634735_JPRB,  35.2387543_JPRB,  &
  35.2136688_JPRB,  35.1885834_JPRB,  35.1640663_JPRB,  35.1404762_JPRB,  35.1168823_JPRB,  &
  35.0932922_JPRB,  35.0679588_JPRB,  35.0425949_JPRB,  35.0172272_JPRB,  34.9915161_JPRB,  &
  34.9655495_JPRB,  34.9395828_JPRB,  34.9134903_JPRB,  34.8866959_JPRB,  34.8598976_JPRB,  &
  34.8331032_JPRB,  34.8068695_JPRB,  34.7808609_JPRB,  34.7548561_JPRB,  34.7286949_JPRB,  &
  34.7020950_JPRB,  34.6754990_JPRB,  34.6489029_JPRB,  34.6229324_JPRB,  34.5971184_JPRB,  &
  34.5713043_JPRB,  34.5450020_JPRB,  34.5176888_JPRB,  34.4903793_JPRB,  34.4630699_JPRB,  &
  34.4367981_JPRB,  34.4107246_JPRB,  34.3846474_JPRB,  34.3579903_JPRB,  34.3302078_JPRB,  &
  34.3024216_JPRB,  34.2746353_JPRB,  34.2480545_JPRB,  34.2217255_JPRB,  34.1953964_JPRB,  &
  34.1690941_JPRB,  34.1428642_JPRB,  34.1166306_JPRB,  34.0904007_JPRB,  34.0639038_JPRB,  &
  34.0373268_JPRB,  34.0107460_JPRB,  33.9838638_JPRB,  33.9558258_JPRB,  33.9277878_JPRB,  &
  33.8997536_JPRB,  33.8666687_JPRB,  33.8307571_JPRB,  33.7948456_JPRB,  33.7593575_JPRB,  &
  33.7304268_JPRB,  33.7014999_JPRB,  33.6725693_JPRB,  33.6459503_JPRB,  33.6219673_JPRB,  &
  33.5979881_JPRB,  33.5740089_JPRB,  33.5482216_JPRB,  33.5221443_JPRB,  33.4960632_JPRB,  &
  33.4684982_JPRB,  33.4361839_JPRB,  33.4038696_JPRB,  33.3715553_JPRB,  33.3411522_JPRB,  &
  33.3120079_JPRB,  33.2828636_JPRB,  33.2537193_JPRB,  33.2248878_JPRB,  33.1960678_JPRB,  &
  33.1672516_JPRB,  33.1380882_JPRB,  33.1080856_JPRB,  33.0780830_JPRB,  33.0480843_JPRB,  &
  33.0174904_JPRB,  32.9865303_JPRB,  32.9555664_JPRB,  32.9246025_JPRB,  32.8961258_JPRB,  &
  32.8678513_JPRB,  32.8395767_JPRB,  32.8113747_JPRB,  32.7834244_JPRB,  32.7554741_JPRB,  &
  32.7275200_JPRB,  32.6999435_JPRB,  32.6727371_JPRB,  32.6455307_JPRB,  32.6183243_JPRB,  &
  32.5931396_JPRB,  32.5685806_JPRB,  32.5440216_JPRB,  32.5194664_JPRB,  32.4952126_JPRB,  &
  32.4709587_JPRB,  32.4467049_JPRB,  32.4211998_JPRB,  32.3919983_JPRB,  32.3627968_JPRB,  &
  32.3335953_JPRB,  32.3058357_JPRB,  32.2796783_JPRB,  32.2535172_JPRB,  32.2273598_JPRB,  &
  32.2004166_JPRB,  32.1731071_JPRB,  32.1458015_JPRB,  32.1184921_JPRB,  32.0921860_JPRB,  &
  32.0660248_JPRB,  32.0398598_JPRB,  32.0139084_JPRB,  31.9917793_JPRB,  31.9696484_JPRB,  &
  31.9475174_JPRB,  31.9254494_JPRB,  31.9036102_JPRB,  31.8817711_JPRB,  31.8599319_JPRB,  &
  31.8373051_JPRB,  31.8132954_JPRB,  31.7892838_JPRB,  31.7652740_JPRB,  31.7417164_JPRB,  &
  31.7186184_JPRB,  31.6955204_JPRB,  31.6724224_JPRB,  31.6495037_JPRB,  31.6266994_JPRB,  &
  31.6038971_JPRB,  31.5810928_JPRB,  31.5579185_JPRB,  31.5345955_JPRB,  31.5112743_JPRB,  &
  31.4879513_JPRB,  31.4651890_JPRB,  31.4425659_JPRB,  31.4199409_JPRB,  31.3973179_JPRB,  &
  31.3761768_JPRB,  31.3552513_JPRB,  31.3343277_JPRB,  31.3134022_JPRB,  31.2921829_JPRB,  &
  31.2709408_JPRB,  31.2496967_JPRB,  31.2284546_JPRB,  31.2088394_JPRB,  31.1892796_JPRB,  &
  31.1697178_JPRB,  31.1501560_JPRB,  31.1285019_JPRB,  31.1068325_JPRB,  31.0851631_JPRB,  &
  31.0634918_JPRB,  31.0434933_JPRB,  31.0234928_JPRB,  31.0034924_JPRB,  30.9834938_JPRB,  &
  30.9657249_JPRB,  30.9479752_JPRB,  30.9302254_JPRB,  30.9124756_JPRB,  30.8942242_JPRB,  &
  30.8759556_JPRB,  30.8576870_JPRB,  30.8394184_JPRB,  30.8213997_JPRB,  30.8034000_JPRB,  &
  30.7854004_JPRB,  30.7674026_JPRB,  30.7479382_JPRB,  30.7282581_JPRB,  30.7085762_JPRB,  &
  30.6888962_JPRB,  30.6694336_JPRB,  30.6500263_JPRB,  30.6306190_JPRB,  30.6112118_JPRB,  &
  30.5930939_JPRB,  30.5755043_JPRB,  30.5579147_JPRB,  30.5403271_JPRB,  30.5217361_JPRB,  &
  30.5024853_JPRB,  30.4832344_JPRB,  30.4639854_JPRB,  30.4447708_JPRB,  30.4256001_JPRB,  &
  30.4064274_JPRB,  30.3872547_JPRB,  30.3688240_JPRB,  30.3518295_JPRB,  30.3348351_JPRB,  &
  30.3178406_JPRB,  30.3011398_JPRB,  30.2857361_JPRB,  30.2703323_JPRB,  30.2549286_JPRB,  &
  30.2395267_JPRB,  30.2243767_JPRB,  30.2092247_JPRB,  30.1940746_JPRB,  30.1789227_JPRB,  &
  30.1636677_JPRB,  30.1483898_JPRB,  30.1331120_JPRB,  30.1178341_JPRB,  30.1022491_JPRB,  &
  30.0864716_JPRB,  30.0706921_JPRB,  30.0549126_JPRB,  30.0396652_JPRB,  30.0252609_JPRB,  &
  30.0108566_JPRB,  29.9964523_JPRB,  29.9821110_JPRB,  29.9681377_JPRB,  29.9541645_JPRB,  &
  29.9401932_JPRB,  29.9262199_JPRB,  29.9103298_JPRB,  29.8941879_JPRB,  29.8780460_JPRB,  &
  29.8619061_JPRB,  29.8450203_JPRB,  29.8276520_JPRB,  29.8102837_JPRB,  29.7929153_JPRB,  &
  29.7757988_JPRB,  29.7592392_JPRB,  29.7426777_JPRB,  29.7261181_JPRB,  29.7095585_JPRB,  &
  29.6996498_JPRB,  29.6897545_JPRB,  29.6798592_JPRB,  29.6699657_JPRB,  29.6582737_JPRB,  &
  29.6456814_JPRB,  29.6330910_JPRB,  29.6205006_JPRB,  29.6072922_JPRB,  29.5927544_JPRB,  &
  29.5782166_JPRB,  29.5636806_JPRB,  29.5491428_JPRB,  29.5346622_JPRB,  29.5201855_JPRB,  &
  29.5057106_JPRB,  29.4912338_JPRB,  29.4785175_JPRB,  29.4671650_JPRB,  29.4558125_JPRB,  &
  29.4444599_JPRB,  29.4330292_JPRB,  29.4211864_JPRB,  29.4093437_JPRB,  29.3974991_JPRB,  &
  29.3856564_JPRB,  29.3738499_JPRB,  29.3620567_JPRB,  29.3502617_JPRB,  29.3384686_JPRB,  &
  29.3269520_JPRB,  29.3160992_JPRB,  29.3052444_JPRB,  29.2943897_JPRB,  29.2835350_JPRB,  &
  29.2743473_JPRB,  29.2654877_JPRB,  29.2566280_JPRB,  29.2477703_JPRB,  29.2384205_JPRB,  &
  29.2281876_JPRB,  29.2179546_JPRB,  29.2077217_JPRB,  29.1974869_JPRB,  29.1856327_JPRB,  &
  29.1735115_JPRB,  29.1613884_JPRB,  29.1492653_JPRB,  29.1374588_JPRB,  29.1262627_JPRB,  &
  29.1150646_JPRB,  29.1038685_JPRB,  29.0926723_JPRB,  29.0812340_JPRB,  29.0697365_JPRB,  &
  29.0582390_JPRB,  29.0467434_JPRB,  29.0351295_JPRB,  29.0231609_JPRB,  29.0111923_JPRB,  &
  28.9992237_JPRB,  28.9872551_JPRB,  28.9762497_JPRB,  28.9657135_JPRB,  28.9551773_JPRB,  &
  28.9446430_JPRB,  28.9341106_JPRB,  28.9236202_JPRB,  28.9131298_JPRB,  28.9026394_JPRB,  &
  28.8921471_JPRB,  28.8813610_JPRB,  28.8702316_JPRB,  28.8591003_JPRB,  28.8479710_JPRB,  &
  28.8368397_JPRB,  28.8234921_JPRB,  28.8096809_JPRB,  28.7958698_JPRB,  28.7820587_JPRB,  &
  28.7683182_JPRB,  28.7549057_JPRB,  28.7414951_JPRB,  28.7280827_JPRB,  28.7146721_JPRB,  &
  28.7011204_JPRB,  28.6874294_JPRB,  28.6737366_JPRB,  28.6600456_JPRB,  28.6463528_JPRB,  &
  28.6348705_JPRB,  28.6239319_JPRB,  28.6129913_JPRB,  28.6020527_JPRB,  28.5911160_JPRB,  &
  28.5802250_JPRB,  28.5693321_JPRB,  28.5584393_JPRB,  28.5475483_JPRB,  28.5352097_JPRB,  &
  28.5201931_JPRB,  28.5051765_JPRB,  28.4901619_JPRB,  28.4751453_JPRB,  28.4604645_JPRB,  &
  28.4460125_JPRB,  28.4315605_JPRB,  28.4171085_JPRB,  28.4026566_JPRB,  28.3889313_JPRB,  &
  28.3753681_JPRB,  28.3618069_JPRB,  28.3482456_JPRB,  28.3346806_JPRB,  28.3210144_JPRB,  &
  28.3073463_JPRB,  28.2936783_JPRB,  28.2800121_JPRB,  28.2669716_JPRB,  28.2564793_JPRB,  &
  28.2459869_JPRB,  28.2354946_JPRB,  28.2250023_JPRB,  28.2137108_JPRB,  28.2009792_JPRB,  &
  28.1882477_JPRB,  28.1755161_JPRB,  28.1627846_JPRB,  28.1492786_JPRB,  28.1349773_JPRB,  &
  28.1206779_JPRB,  28.1063766_JPRB,  28.0920753_JPRB,  28.0768261_JPRB,  28.0609722_JPRB,  &
  28.0451164_JPRB,  28.0292606_JPRB,  28.0134068_JPRB,  27.9966908_JPRB,  27.9796181_JPRB,  &
  27.9625454_JPRB,  27.9454708_JPRB,  27.9283981_JPRB,  27.9122562_JPRB,  27.8963795_JPRB,  &
  27.8805046_JPRB,  27.8646297_JPRB,  27.8487549_JPRB,  27.8316154_JPRB,  27.8142128_JPRB,  &
  27.7968121_JPRB,  27.7794113_JPRB,  27.7620106_JPRB,  27.7442665_JPRB,  27.7264652_JPRB,  &
  27.7086658_JPRB,  27.6908646_JPRB,  27.6730633_JPRB,  27.6553326_JPRB,  27.6376114_JPRB,  &
  27.6198902_JPRB,  27.6021690_JPRB,  27.5844479_JPRB,  27.5658588_JPRB,  27.5471153_JPRB,  &
  27.5283699_JPRB,  27.5096264_JPRB,  27.4908829_JPRB,  27.4715710_JPRB,  27.4521275_JPRB,  &
  27.4326820_JPRB,  27.4132385_JPRB,  27.3937931_JPRB,  27.3746490_JPRB,  27.3556042_JPRB,  &
  27.3365593_JPRB,  27.3175144_JPRB,  27.2984695_JPRB,  27.2805119_JPRB,  27.2631073_JPRB,  &
  27.2457027_JPRB,  27.2282982_JPRB,  27.2108917_JPRB,  27.1937027_JPRB,  27.1766872_JPRB,  &
  27.1596699_JPRB,  27.1426525_JPRB,  27.1256351_JPRB,  27.1083889_JPRB,  27.0908337_JPRB,  &
  27.0732765_JPRB,  27.0557213_JPRB,  27.0381641_JPRB,  27.0203800_JPRB,  27.0019836_JPRB,  &
  26.9835873_JPRB,  26.9651909_JPRB,  26.9467945_JPRB,  26.9284649_JPRB,  26.9107628_JPRB,  &
  26.8930607_JPRB,  26.8753586_JPRB,  26.8576565_JPRB,  26.8399544_JPRB,  26.8213730_JPRB,  &
  26.8026886_JPRB,  26.7840023_JPRB,  26.7653179_JPRB,  26.7466335_JPRB,  26.7292213_JPRB,  &
  26.7124367_JPRB,  26.6956520_JPRB,  26.6788673_JPRB,  26.6620827_JPRB,  26.6450748_JPRB,  &
  26.6277657_JPRB,  26.6104546_JPRB,  26.5931454_JPRB,  26.5758343_JPRB,  26.5585136_JPRB,  &
  26.5411339_JPRB,  26.5237522_JPRB,  26.5063725_JPRB,  26.4889908_JPRB,  26.4716110_JPRB,  &
  26.4569683_JPRB,  26.4428005_JPRB,  26.4286308_JPRB,  26.4144630_JPRB,  26.4002934_JPRB,  &
  26.3871841_JPRB,  26.3750114_JPRB,  26.3628368_JPRB,  26.3506641_JPRB,  26.3384914_JPRB,  &
  26.3263550_JPRB,  26.3143845_JPRB,  26.3024158_JPRB,  26.2904472_JPRB,  26.2784767_JPRB,  &
  26.2665081_JPRB,  26.2512283_JPRB,  26.2351971_JPRB,  26.2191639_JPRB,  26.2031307_JPRB,  &
  26.1870995_JPRB,  26.1714039_JPRB,  26.1561775_JPRB,  26.1409512_JPRB,  26.1257267_JPRB,  &
  26.1105003_JPRB,  26.0952740_JPRB,  26.0812817_JPRB,  26.0672913_JPRB,  26.0533028_JPRB,  &
  26.0393124_JPRB,  26.0253239_JPRB,  26.0126495_JPRB,  26.0010452_JPRB,  25.9894428_JPRB,  &
  25.9778385_JPRB,  25.9662361_JPRB,  25.9545670_JPRB,  25.9421520_JPRB,  25.9297352_JPRB,  &
  25.9173203_JPRB,  25.9049053_JPRB,  25.8924885_JPRB,  25.8806934_JPRB,  25.8693409_JPRB,  &
  25.8579903_JPRB,  25.8466377_JPRB,  25.8352871_JPRB,  25.8239040_JPRB,  25.8120346_JPRB,  &
  25.8001652_JPRB,  25.7882938_JPRB,  25.7764244_JPRB,  25.7645531_JPRB,  25.7520580_JPRB,  &
  25.7389641_JPRB,  25.7258682_JPRB,  25.7127724_JPRB,  25.6996765_JPRB,  25.6865807_JPRB,  &
  25.6744690_JPRB,  25.6624279_JPRB,  25.6503849_JPRB,  25.6383438_JPRB,  25.6263008_JPRB,  &
  25.6137161_JPRB,  25.6000385_JPRB,  25.5863628_JPRB,  25.5726852_JPRB,  25.5590076_JPRB,  &
  25.5453300_JPRB,  25.5319958_JPRB,  25.5188046_JPRB,  25.5056133_JPRB,  25.4924221_JPRB,  &
  25.4792290_JPRB,  25.4660549_JPRB,  25.4532051_JPRB,  25.4403553_JPRB,  25.4275074_JPRB,  &
  25.4146576_JPRB,  25.4018078_JPRB,  25.3888283_JPRB,  25.3756237_JPRB,  25.3624191_JPRB,  &
  25.3492126_JPRB,  25.3360081_JPRB,  25.3228035_JPRB,  25.3096390_JPRB,  25.2964973_JPRB,  &
  25.2833557_JPRB,  25.2702122_JPRB,  25.2570705_JPRB,  25.2439289_JPRB,  25.2310963_JPRB,  &
  25.2182941_JPRB,  25.2054901_JPRB,  25.1926861_JPRB,  25.1798820_JPRB,  25.1671085_JPRB,  &
  25.1545029_JPRB,  25.1418972_JPRB,  25.1292915_JPRB,  25.1166859_JPRB,  25.1040802_JPRB,  &
  25.0913010_JPRB,  25.0782108_JPRB,  25.0651207_JPRB,  25.0520287_JPRB,  25.0389385_JPRB,  &
  25.0258484_JPRB,  25.0122795_JPRB,  24.9983025_JPRB,  24.9843254_JPRB,  24.9703484_JPRB,  &
  24.9563713_JPRB,  24.9423923_JPRB,  24.9293041_JPRB,  24.9166107_JPRB,  24.9039154_JPRB,  &
  24.8912220_JPRB,  24.8785286_JPRB,  24.8658333_JPRB,  24.8535175_JPRB,  24.8412895_JPRB,  &
  24.8290596_JPRB,  24.8168297_JPRB,  24.8045998_JPRB,  24.7923717_JPRB,  24.7742329_JPRB,  &
  24.7555084_JPRB,  24.7367859_JPRB,  24.7180614_JPRB,  24.6993389_JPRB,  24.6806145_JPRB,  &
  24.6643181_JPRB,  24.6480808_JPRB,  24.6318455_JPRB,  24.6156082_JPRB,  24.5993710_JPRB,  &
  24.5831604_JPRB,  24.5689888_JPRB,  24.5548191_JPRB,  24.5406475_JPRB,  24.5264759_JPRB,  &
  24.5123062_JPRB,  24.4981022_JPRB,  24.4822884_JPRB,  24.4664726_JPRB,  24.4506588_JPRB,  &
  24.4348431_JPRB,  24.4190292_JPRB,  24.4032135_JPRB,  24.3889141_JPRB,  24.3746204_JPRB,  &
  24.3603249_JPRB,  24.3460312_JPRB,  24.3317356_JPRB,  24.3174419_JPRB,  24.3038292_JPRB,  &
  24.2902584_JPRB,  24.2766857_JPRB,  24.2631149_JPRB,  24.2495422_JPRB,  24.2359695_JPRB,  &
  24.2227917_JPRB,  24.2096767_JPRB,  24.1965618_JPRB,  24.1834450_JPRB,  24.1703300_JPRB,  &
  24.1572151_JPRB,  24.1433697_JPRB,  24.1292858_JPRB,  24.1152000_JPRB,  24.1011162_JPRB,  &
  24.0870323_JPRB,  24.0729465_JPRB,  24.0578842_JPRB,  24.0421982_JPRB,  24.0265121_JPRB,  &
  24.0108261_JPRB,  23.9951401_JPRB,  23.9794540_JPRB,  23.9644184_JPRB,  23.9502163_JPRB,  &
  23.9360161_JPRB,  23.9218140_JPRB,  23.9076138_JPRB,  23.8934116_JPRB,  23.8783321_JPRB,  &
  23.8603821_JPRB,  23.8424339_JPRB,  23.8244839_JPRB,  23.8065338_JPRB,  23.7885838_JPRB,  &
  23.7706356_JPRB /

  DATA snow_ref / &
  0.010_JPRB,  0.010_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.008_JPRB,  0.008_JPRB,  &
  0.008_JPRB,  0.008_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.008_JPRB,  0.008_JPRB,  &
  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.006_JPRB,  0.006_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.003_JPRB,  0.003_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  &
  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.009_JPRB,  &
  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  &
  0.009_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  &
  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  &
  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  &
  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  &
  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  &
  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  &
  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  &
  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  &
  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  &
  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  &
  0.010_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  &
  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  &
  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  &
  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  &
  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.003_JPRB,  0.003_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.003_JPRB,  &
  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  &
  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  &
  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  &
  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.003_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  &
  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.004_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  &
  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.005_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  &
  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.011_JPRB,  &
  0.011_JPRB,  0.011_JPRB,  0.012_JPRB,  0.012_JPRB,  0.013_JPRB,  0.013_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.015_JPRB,  0.015_JPRB,  0.016_JPRB,  0.016_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.019_JPRB,  0.019_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.023_JPRB,  &
  0.023_JPRB,  0.023_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  &
  0.024_JPRB,  0.024_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  &
  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  &
  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  &
  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.024_JPRB,  0.024_JPRB,  &
  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  &
  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  &
  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  &
  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  &
  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  &
  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  &
  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  &
  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  &
  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  &
  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  &
  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  &
  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  &
  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.013_JPRB,  &
  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  &
  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.012_JPRB,  0.012_JPRB,  &
  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  &
  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  &
  0.012_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  &
  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  &
  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  &
  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  0.009_JPRB,  0.009_JPRB,  &
  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  &
  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  &
  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  &
  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  &
  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.006_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  &
  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  0.007_JPRB,  &
  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.008_JPRB,  0.009_JPRB,  &
  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.009_JPRB,  0.010_JPRB,  0.010_JPRB,  0.010_JPRB,  &
  0.010_JPRB,  0.011_JPRB,  0.011_JPRB,  0.011_JPRB,  0.012_JPRB,  0.012_JPRB,  0.013_JPRB,  &
  0.013_JPRB,  0.013_JPRB,  0.014_JPRB,  0.014_JPRB,  0.015_JPRB,  0.015_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.017_JPRB,  0.017_JPRB,  0.018_JPRB,  0.018_JPRB,  0.019_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.021_JPRB,  0.022_JPRB,  0.022_JPRB,  0.023_JPRB,  0.024_JPRB,  0.025_JPRB,  &
  0.026_JPRB,  0.026_JPRB,  0.027_JPRB,  0.028_JPRB,  0.029_JPRB,  0.031_JPRB,  0.032_JPRB,  &
  0.033_JPRB,  0.034_JPRB,  0.035_JPRB,  0.037_JPRB,  0.038_JPRB,  0.040_JPRB,  0.041_JPRB,  &
  0.043_JPRB,  0.045_JPRB,  0.047_JPRB,  0.049_JPRB,  0.051_JPRB,  0.053_JPRB,  0.055_JPRB,  &
  0.057_JPRB,  0.060_JPRB,  0.062_JPRB,  0.065_JPRB,  0.067_JPRB,  0.070_JPRB,  0.072_JPRB,  &
  0.075_JPRB,  0.078_JPRB,  0.080_JPRB,  0.083_JPRB,  0.085_JPRB,  0.088_JPRB,  0.090_JPRB,  &
  0.093_JPRB,  0.095_JPRB,  0.097_JPRB,  0.099_JPRB,  0.101_JPRB,  0.103_JPRB,  0.105_JPRB,  &
  0.106_JPRB,  0.108_JPRB,  0.110_JPRB,  0.111_JPRB,  0.112_JPRB,  0.114_JPRB,  0.115_JPRB,  &
  0.116_JPRB,  0.117_JPRB,  0.118_JPRB,  0.119_JPRB,  0.120_JPRB,  0.121_JPRB,  0.122_JPRB,  &
  0.123_JPRB,  0.124_JPRB,  0.125_JPRB,  0.125_JPRB,  0.126_JPRB,  0.127_JPRB,  0.128_JPRB,  &
  0.129_JPRB,  0.130_JPRB,  0.130_JPRB,  0.131_JPRB,  0.132_JPRB,  0.133_JPRB,  0.134_JPRB,  &
  0.135_JPRB,  0.136_JPRB,  0.136_JPRB,  0.137_JPRB,  0.138_JPRB,  0.139_JPRB,  0.140_JPRB,  &
  0.141_JPRB,  0.142_JPRB,  0.143_JPRB,  0.144_JPRB,  0.145_JPRB,  0.146_JPRB,  0.147_JPRB,  &
  0.148_JPRB,  0.149_JPRB,  0.150_JPRB,  0.151_JPRB,  0.152_JPRB,  0.153_JPRB,  0.154_JPRB,  &
  0.155_JPRB,  0.155_JPRB,  0.156_JPRB,  0.157_JPRB,  0.158_JPRB,  0.159_JPRB,  0.160_JPRB,  &
  0.160_JPRB,  0.161_JPRB,  0.162_JPRB,  0.163_JPRB,  0.163_JPRB,  0.164_JPRB,  0.165_JPRB,  &
  0.165_JPRB,  0.166_JPRB,  0.167_JPRB,  0.167_JPRB,  0.168_JPRB,  0.168_JPRB,  0.169_JPRB,  &
  0.169_JPRB,  0.170_JPRB,  0.170_JPRB,  0.170_JPRB,  0.171_JPRB,  0.171_JPRB,  0.172_JPRB,  &
  0.172_JPRB,  0.172_JPRB,  0.173_JPRB,  0.173_JPRB,  0.173_JPRB,  0.173_JPRB,  0.174_JPRB,  &
  0.174_JPRB,  0.174_JPRB,  0.174_JPRB,  0.174_JPRB,  0.174_JPRB,  0.174_JPRB,  0.174_JPRB,  &
  0.174_JPRB,  0.174_JPRB,  0.174_JPRB,  0.173_JPRB,  0.173_JPRB,  0.173_JPRB,  0.173_JPRB,  &
  0.173_JPRB,  0.172_JPRB,  0.172_JPRB,  0.172_JPRB,  0.171_JPRB,  0.171_JPRB,  0.171_JPRB,  &
  0.171_JPRB,  0.170_JPRB,  0.170_JPRB,  0.170_JPRB,  0.170_JPRB,  0.169_JPRB,  0.169_JPRB,  &
  0.169_JPRB,  0.169_JPRB,  0.169_JPRB,  0.168_JPRB,  0.168_JPRB,  0.168_JPRB,  0.168_JPRB,  &
  0.168_JPRB,  0.168_JPRB,  0.168_JPRB,  0.168_JPRB,  0.168_JPRB,  0.168_JPRB,  0.168_JPRB,  &
  0.168_JPRB,  0.168_JPRB,  0.169_JPRB,  0.169_JPRB,  0.169_JPRB,  0.169_JPRB,  0.169_JPRB,  &
  0.170_JPRB,  0.170_JPRB,  0.170_JPRB,  0.171_JPRB,  0.171_JPRB,  0.172_JPRB,  0.172_JPRB,  &
  0.172_JPRB,  0.173_JPRB,  0.174_JPRB,  0.174_JPRB,  0.175_JPRB,  0.175_JPRB,  0.176_JPRB,  &
  0.177_JPRB,  0.178_JPRB,  0.178_JPRB,  0.179_JPRB,  0.180_JPRB,  0.181_JPRB,  0.182_JPRB,  &
  0.183_JPRB,  0.184_JPRB,  0.185_JPRB,  0.186_JPRB,  0.187_JPRB,  0.188_JPRB,  0.189_JPRB,  &
  0.191_JPRB,  0.192_JPRB,  0.193_JPRB,  0.195_JPRB,  0.196_JPRB,  0.197_JPRB,  0.199_JPRB,  &
  0.200_JPRB,  0.202_JPRB,  0.204_JPRB,  0.205_JPRB,  0.207_JPRB,  0.209_JPRB,  0.211_JPRB,  &
  0.213_JPRB,  0.215_JPRB,  0.217_JPRB,  0.219_JPRB,  0.221_JPRB,  0.223_JPRB,  0.225_JPRB,  &
  0.227_JPRB,  0.229_JPRB,  0.232_JPRB,  0.234_JPRB,  0.237_JPRB,  0.239_JPRB,  0.242_JPRB,  &
  0.244_JPRB,  0.247_JPRB,  0.250_JPRB,  0.252_JPRB,  0.255_JPRB,  0.258_JPRB,  0.260_JPRB,  &
  0.263_JPRB,  0.266_JPRB,  0.269_JPRB,  0.272_JPRB,  0.275_JPRB,  0.278_JPRB,  0.281_JPRB,  &
  0.284_JPRB,  0.287_JPRB,  0.290_JPRB,  0.293_JPRB,  0.296_JPRB,  0.299_JPRB,  0.302_JPRB,  &
  0.305_JPRB,  0.308_JPRB,  0.312_JPRB,  0.315_JPRB,  0.318_JPRB,  0.321_JPRB,  0.325_JPRB,  &
  0.328_JPRB,  0.331_JPRB,  0.335_JPRB,  0.338_JPRB,  0.342_JPRB,  0.346_JPRB,  0.350_JPRB,  &
  0.354_JPRB,  0.357_JPRB,  0.361_JPRB,  0.365_JPRB,  0.369_JPRB,  0.373_JPRB,  0.377_JPRB,  &
  0.381_JPRB,  0.386_JPRB,  0.390_JPRB,  0.394_JPRB,  0.397_JPRB,  0.401_JPRB,  0.405_JPRB,  &
  0.409_JPRB,  0.412_JPRB,  0.415_JPRB,  0.418_JPRB,  0.421_JPRB,  0.424_JPRB,  0.427_JPRB,  &
  0.429_JPRB,  0.431_JPRB,  0.434_JPRB,  0.436_JPRB,  0.437_JPRB,  0.439_JPRB,  0.441_JPRB,  &
  0.442_JPRB,  0.444_JPRB,  0.445_JPRB,  0.447_JPRB,  0.448_JPRB,  0.449_JPRB,  0.450_JPRB,  &
  0.451_JPRB,  0.453_JPRB,  0.454_JPRB,  0.455_JPRB,  0.456_JPRB,  0.457_JPRB,  0.458_JPRB,  &
  0.459_JPRB,  0.460_JPRB,  0.461_JPRB,  0.462_JPRB,  0.463_JPRB,  0.464_JPRB,  0.465_JPRB,  &
  0.466_JPRB,  0.467_JPRB,  0.468_JPRB,  0.468_JPRB,  0.469_JPRB,  0.470_JPRB,  0.471_JPRB,  &
  0.471_JPRB,  0.472_JPRB,  0.473_JPRB,  0.473_JPRB,  0.474_JPRB,  0.474_JPRB,  0.474_JPRB,  &
  0.475_JPRB,  0.475_JPRB,  0.475_JPRB,  0.475_JPRB,  0.475_JPRB,  0.474_JPRB,  0.474_JPRB,  &
  0.474_JPRB,  0.473_JPRB,  0.473_JPRB,  0.472_JPRB,  0.471_JPRB,  0.470_JPRB,  0.469_JPRB,  &
  0.468_JPRB,  0.467_JPRB,  0.466_JPRB,  0.464_JPRB,  0.463_JPRB,  0.461_JPRB,  0.460_JPRB,  &
  0.458_JPRB,  0.456_JPRB,  0.454_JPRB,  0.452_JPRB,  0.451_JPRB,  0.449_JPRB,  0.447_JPRB,  &
  0.445_JPRB,  0.443_JPRB,  0.441_JPRB,  0.439_JPRB,  0.437_JPRB,  0.434_JPRB,  0.432_JPRB,  &
  0.430_JPRB,  0.428_JPRB,  0.426_JPRB,  0.424_JPRB,  0.423_JPRB,  0.421_JPRB,  0.419_JPRB,  &
  0.417_JPRB,  0.415_JPRB,  0.414_JPRB,  0.412_JPRB,  0.411_JPRB,  0.409_JPRB,  0.408_JPRB,  &
  0.407_JPRB,  0.405_JPRB,  0.404_JPRB,  0.403_JPRB,  0.402_JPRB,  0.401_JPRB,  0.400_JPRB,  &
  0.400_JPRB,  0.399_JPRB,  0.398_JPRB,  0.398_JPRB,  0.397_JPRB,  0.397_JPRB,  0.397_JPRB,  &
  0.397_JPRB,  0.396_JPRB,  0.396_JPRB,  0.397_JPRB,  0.397_JPRB,  0.397_JPRB,  0.397_JPRB,  &
  0.398_JPRB,  0.398_JPRB,  0.399_JPRB,  0.399_JPRB,  0.400_JPRB,  0.401_JPRB,  0.401_JPRB,  &
  0.402_JPRB,  0.403_JPRB,  0.404_JPRB,  0.406_JPRB,  0.407_JPRB,  0.408_JPRB,  0.410_JPRB,  &
  0.411_JPRB,  0.413_JPRB,  0.414_JPRB,  0.416_JPRB,  0.417_JPRB,  0.419_JPRB,  0.421_JPRB,  &
  0.423_JPRB,  0.425_JPRB,  0.427_JPRB,  0.429_JPRB,  0.432_JPRB,  0.434_JPRB,  0.436_JPRB,  &
  0.438_JPRB,  0.440_JPRB,  0.443_JPRB,  0.445_JPRB,  0.448_JPRB,  0.450_JPRB,  0.452_JPRB,  &
  0.454_JPRB,  0.457_JPRB,  0.459_JPRB,  0.462_JPRB,  0.464_JPRB,  0.467_JPRB,  0.469_JPRB,  &
  0.471_JPRB,  0.473_JPRB,  0.476_JPRB,  0.478_JPRB,  0.480_JPRB,  0.483_JPRB,  0.485_JPRB,  &
  0.488_JPRB,  0.490_JPRB,  0.492_JPRB,  0.494_JPRB,  0.497_JPRB,  0.499_JPRB,  0.502_JPRB,  &
  0.505_JPRB,  0.508_JPRB,  0.510_JPRB,  0.513_JPRB,  0.516_JPRB,  0.519_JPRB,  0.523_JPRB,  &
  0.526_JPRB,  0.529_JPRB,  0.532_JPRB,  0.535_JPRB,  0.538_JPRB,  0.541_JPRB,  0.544_JPRB,  &
  0.546_JPRB,  0.549_JPRB,  0.552_JPRB,  0.554_JPRB,  0.556_JPRB,  0.558_JPRB,  0.560_JPRB,  &
  0.561_JPRB,  0.563_JPRB,  0.565_JPRB,  0.567_JPRB,  0.568_JPRB,  0.569_JPRB,  0.571_JPRB,  &
  0.572_JPRB,  0.573_JPRB,  0.574_JPRB,  0.575_JPRB,  0.576_JPRB,  0.577_JPRB,  0.578_JPRB,  &
  0.579_JPRB,  0.580_JPRB,  0.581_JPRB,  0.581_JPRB,  0.582_JPRB,  0.583_JPRB,  0.583_JPRB,  &
  0.584_JPRB,  0.584_JPRB,  0.585_JPRB,  0.585_JPRB,  0.586_JPRB,  0.586_JPRB,  0.586_JPRB,  &
  0.587_JPRB,  0.587_JPRB,  0.588_JPRB,  0.588_JPRB,  0.588_JPRB,  0.589_JPRB,  0.589_JPRB,  &
  0.590_JPRB,  0.590_JPRB,  0.591_JPRB,  0.592_JPRB,  0.592_JPRB,  0.593_JPRB,  0.594_JPRB,  &
  0.594_JPRB,  0.595_JPRB,  0.596_JPRB,  0.597_JPRB,  0.598_JPRB,  0.599_JPRB,  0.600_JPRB,  &
  0.601_JPRB,  0.602_JPRB,  0.603_JPRB,  0.604_JPRB,  0.606_JPRB,  0.607_JPRB,  0.608_JPRB,  &
  0.610_JPRB,  0.611_JPRB,  0.612_JPRB,  0.614_JPRB,  0.616_JPRB,  0.617_JPRB,  0.619_JPRB,  &
  0.620_JPRB,  0.622_JPRB,  0.623_JPRB,  0.625_JPRB,  0.626_JPRB,  0.628_JPRB,  0.629_JPRB,  &
  0.631_JPRB,  0.632_JPRB,  0.634_JPRB,  0.635_JPRB,  0.636_JPRB,  0.638_JPRB,  0.639_JPRB,  &
  0.640_JPRB,  0.642_JPRB,  0.643_JPRB,  0.644_JPRB,  0.645_JPRB,  0.646_JPRB,  0.647_JPRB,  &
  0.648_JPRB,  0.650_JPRB,  0.651_JPRB,  0.652_JPRB,  0.653_JPRB,  0.654_JPRB,  0.654_JPRB,  &
  0.655_JPRB,  0.656_JPRB,  0.657_JPRB,  0.658_JPRB,  0.658_JPRB,  0.659_JPRB,  0.659_JPRB,  &
  0.660_JPRB,  0.660_JPRB,  0.661_JPRB,  0.661_JPRB,  0.661_JPRB,  0.662_JPRB,  0.662_JPRB,  &
  0.662_JPRB,  0.663_JPRB,  0.663_JPRB,  0.663_JPRB,  0.663_JPRB,  0.663_JPRB,  0.664_JPRB,  &
  0.664_JPRB,  0.664_JPRB,  0.664_JPRB,  0.664_JPRB,  0.664_JPRB,  0.664_JPRB,  0.664_JPRB,  &
  0.664_JPRB,  0.664_JPRB,  0.664_JPRB,  0.664_JPRB,  0.664_JPRB,  0.664_JPRB,  0.663_JPRB,  &
  0.663_JPRB,  0.663_JPRB,  0.663_JPRB,  0.663_JPRB,  0.663_JPRB,  0.663_JPRB,  0.663_JPRB,  &
  0.662_JPRB,  0.662_JPRB,  0.662_JPRB,  0.662_JPRB,  0.663_JPRB,  0.663_JPRB,  0.663_JPRB,  &
  0.663_JPRB,  0.663_JPRB,  0.663_JPRB,  0.663_JPRB,  0.664_JPRB,  0.664_JPRB,  0.664_JPRB,  &
  0.665_JPRB,  0.665_JPRB,  0.665_JPRB,  0.666_JPRB,  0.666_JPRB,  0.667_JPRB,  0.667_JPRB,  &
  0.668_JPRB,  0.668_JPRB,  0.669_JPRB,  0.670_JPRB,  0.670_JPRB,  0.671_JPRB,  0.672_JPRB,  &
  0.672_JPRB,  0.673_JPRB,  0.674_JPRB,  0.675_JPRB,  0.675_JPRB,  0.676_JPRB,  0.677_JPRB,  &
  0.678_JPRB,  0.678_JPRB,  0.679_JPRB,  0.680_JPRB,  0.681_JPRB,  0.681_JPRB,  0.682_JPRB,  &
  0.683_JPRB,  0.684_JPRB,  0.684_JPRB,  0.685_JPRB,  0.686_JPRB,  0.687_JPRB,  0.687_JPRB,  &
  0.688_JPRB,  0.689_JPRB,  0.689_JPRB,  0.690_JPRB,  0.690_JPRB,  0.691_JPRB,  0.691_JPRB,  &
  0.692_JPRB,  0.692_JPRB,  0.693_JPRB,  0.693_JPRB,  0.694_JPRB,  0.694_JPRB,  0.695_JPRB,  &
  0.695_JPRB,  0.695_JPRB,  0.696_JPRB,  0.696_JPRB,  0.696_JPRB,  0.697_JPRB,  0.697_JPRB,  &
  0.698_JPRB,  0.698_JPRB,  0.698_JPRB,  0.699_JPRB,  0.699_JPRB,  0.700_JPRB,  0.700_JPRB,  &
  0.701_JPRB,  0.701_JPRB,  0.702_JPRB,  0.702_JPRB,  0.702_JPRB,  0.703_JPRB,  0.703_JPRB,  &
  0.703_JPRB,  0.704_JPRB,  0.704_JPRB,  0.704_JPRB,  0.704_JPRB,  0.705_JPRB,  0.705_JPRB,  &
  0.705_JPRB,  0.706_JPRB,  0.706_JPRB,  0.706_JPRB,  0.706_JPRB,  0.707_JPRB,  0.707_JPRB,  &
  0.707_JPRB,  0.708_JPRB,  0.708_JPRB,  0.708_JPRB,  0.708_JPRB,  0.709_JPRB,  0.709_JPRB,  &
  0.709_JPRB,  0.710_JPRB,  0.710_JPRB,  0.710_JPRB,  0.710_JPRB,  0.710_JPRB,  0.711_JPRB,  &
  0.711_JPRB,  0.711_JPRB,  0.711_JPRB,  0.711_JPRB,  0.711_JPRB,  0.711_JPRB,  0.711_JPRB,  &
  0.711_JPRB,  0.711_JPRB,  0.712_JPRB,  0.712_JPRB,  0.712_JPRB,  0.712_JPRB,  0.712_JPRB,  &
  0.712_JPRB,  0.712_JPRB,  0.712_JPRB,  0.712_JPRB,  0.712_JPRB,  0.712_JPRB,  0.713_JPRB,  &
  0.713_JPRB,  0.713_JPRB,  0.713_JPRB,  0.713_JPRB,  0.713_JPRB,  0.713_JPRB,  0.713_JPRB,  &
  0.713_JPRB,  0.713_JPRB,  0.714_JPRB,  0.714_JPRB,  0.714_JPRB,  0.714_JPRB,  0.714_JPRB,  &
  0.714_JPRB,  0.715_JPRB,  0.715_JPRB,  0.715_JPRB,  0.715_JPRB,  0.715_JPRB,  0.715_JPRB,  &
  0.715_JPRB,  0.716_JPRB,  0.716_JPRB,  0.716_JPRB,  0.716_JPRB,  0.716_JPRB,  0.716_JPRB,  &
  0.716_JPRB,  0.717_JPRB,  0.717_JPRB,  0.717_JPRB,  0.717_JPRB,  0.717_JPRB,  0.717_JPRB,  &
  0.718_JPRB,  0.718_JPRB,  0.718_JPRB,  0.718_JPRB,  0.718_JPRB,  0.718_JPRB,  0.718_JPRB,  &
  0.718_JPRB,  0.719_JPRB,  0.719_JPRB,  0.719_JPRB,  0.719_JPRB,  0.719_JPRB,  0.719_JPRB,  &
  0.719_JPRB,  0.719_JPRB,  0.719_JPRB,  0.720_JPRB,  0.720_JPRB,  0.720_JPRB,  0.720_JPRB,  &
  0.720_JPRB,  0.720_JPRB,  0.720_JPRB,  0.720_JPRB,  0.720_JPRB,  0.720_JPRB,  0.721_JPRB,  &
  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  &
  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  &
  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  &
  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  &
  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  &
  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  &
  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  &
  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  &
  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  &
  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  &
  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  &
  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  &
  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  &
  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  0.723_JPRB,  &
  0.723_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  &
  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  &
  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  0.722_JPRB,  &
  0.722_JPRB,  0.722_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  &
  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  &
  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  &
  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  &
  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  &
  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  &
  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  &
  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  &
  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  &
  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.721_JPRB,  0.720_JPRB,  0.720_JPRB,  0.720_JPRB,  &
  0.720_JPRB,  0.720_JPRB,  0.720_JPRB,  0.720_JPRB,  0.720_JPRB,  0.720_JPRB,  0.720_JPRB,  &
  0.719_JPRB,  0.719_JPRB,  0.719_JPRB,  0.719_JPRB,  0.719_JPRB,  0.720_JPRB,  0.719_JPRB,  &
  0.719_JPRB,  0.719_JPRB,  0.718_JPRB,  0.718_JPRB,  0.718_JPRB,  0.718_JPRB,  0.718_JPRB,  &
  0.718_JPRB /

  DATA coastal_waters_ref / &
  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  &
  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  &
  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  &
  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  &
  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  &
  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  &
  0.022_JPRB,  0.022_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  &
  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  &
  0.024_JPRB,  0.024_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  &
  0.025_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  &
  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  &
  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  &
  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  &
  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  &
  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  &
  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  &
  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  &
  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  &
  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  &
  0.025_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  &
  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  &
  0.026_JPRB,  0.026_JPRB,  0.027_JPRB,  0.027_JPRB,  0.027_JPRB,  0.027_JPRB,  0.027_JPRB,  &
  0.027_JPRB,  0.028_JPRB,  0.028_JPRB,  0.028_JPRB,  0.028_JPRB,  0.029_JPRB,  0.029_JPRB,  &
  0.029_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  &
  0.032_JPRB,  0.032_JPRB,  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  0.034_JPRB,  0.034_JPRB,  &
  0.034_JPRB,  0.035_JPRB,  0.035_JPRB,  0.036_JPRB,  0.036_JPRB,  0.036_JPRB,  0.037_JPRB,  &
  0.037_JPRB,  0.037_JPRB,  0.037_JPRB,  0.038_JPRB,  0.038_JPRB,  0.038_JPRB,  0.038_JPRB,  &
  0.038_JPRB,  0.039_JPRB,  0.039_JPRB,  0.039_JPRB,  0.039_JPRB,  0.039_JPRB,  0.039_JPRB,  &
  0.039_JPRB,  0.039_JPRB,  0.039_JPRB,  0.039_JPRB,  0.039_JPRB,  0.039_JPRB,  0.038_JPRB,  &
  0.038_JPRB,  0.038_JPRB,  0.038_JPRB,  0.038_JPRB,  0.038_JPRB,  0.038_JPRB,  0.038_JPRB,  &
  0.037_JPRB,  0.037_JPRB,  0.037_JPRB,  0.037_JPRB,  0.037_JPRB,  0.037_JPRB,  0.037_JPRB,  &
  0.037_JPRB,  0.036_JPRB,  0.036_JPRB,  0.036_JPRB,  0.036_JPRB,  0.036_JPRB,  0.036_JPRB,  &
  0.036_JPRB,  0.036_JPRB,  0.035_JPRB,  0.035_JPRB,  0.035_JPRB,  0.035_JPRB,  0.035_JPRB,  &
  0.035_JPRB,  0.035_JPRB,  0.035_JPRB,  0.035_JPRB,  0.034_JPRB,  0.034_JPRB,  0.034_JPRB,  &
  0.034_JPRB,  0.034_JPRB,  0.034_JPRB,  0.034_JPRB,  0.034_JPRB,  0.034_JPRB,  0.034_JPRB,  &
  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  &
  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  &
  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  0.033_JPRB,  0.032_JPRB,  0.032_JPRB,  &
  0.032_JPRB,  0.032_JPRB,  0.032_JPRB,  0.032_JPRB,  0.032_JPRB,  0.032_JPRB,  0.032_JPRB,  &
  0.032_JPRB,  0.032_JPRB,  0.032_JPRB,  0.032_JPRB,  0.032_JPRB,  0.032_JPRB,  0.032_JPRB,  &
  0.032_JPRB,  0.032_JPRB,  0.032_JPRB,  0.032_JPRB,  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  &
  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  &
  0.031_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  &
  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  &
  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  &
  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  &
  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  &
  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  0.030_JPRB,  0.030_JPRB,  &
  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  &
  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.031_JPRB,  0.031_JPRB,  &
  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  &
  0.031_JPRB /

  DATA ocean_waters_ref / &
  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  &
  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.012_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  &
  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  &
  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  &
  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.013_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  &
  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.014_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  &
  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.015_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  &
  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.016_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  0.017_JPRB,  &
  0.017_JPRB,  0.017_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  &
  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.018_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  0.019_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  &
  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.020_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  &
  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.021_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  &
  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  &
  0.022_JPRB,  0.022_JPRB,  0.022_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  &
  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.023_JPRB,  0.024_JPRB,  &
  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  0.024_JPRB,  &
  0.024_JPRB,  0.024_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  &
  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  0.025_JPRB,  &
  0.025_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  &
  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.026_JPRB,  0.027_JPRB,  0.027_JPRB,  0.027_JPRB,  &
  0.027_JPRB,  0.027_JPRB,  0.027_JPRB,  0.027_JPRB,  0.027_JPRB,  0.027_JPRB,  0.028_JPRB,  &
  0.028_JPRB,  0.028_JPRB,  0.028_JPRB,  0.028_JPRB,  0.028_JPRB,  0.028_JPRB,  0.028_JPRB,  &
  0.028_JPRB,  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  &
  0.029_JPRB,  0.029_JPRB,  0.029_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  0.030_JPRB,  &
  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  0.031_JPRB,  0.032_JPRB,  0.032_JPRB,  0.032_JPRB,  &
  0.033_JPRB,  0.033_JPRB,  0.034_JPRB,  0.034_JPRB,  0.034_JPRB,  0.035_JPRB,  0.035_JPRB,  &
  0.036_JPRB,  0.036_JPRB,  0.036_JPRB,  0.037_JPRB,  0.037_JPRB,  0.037_JPRB,  0.038_JPRB,  &
  0.038_JPRB,  0.038_JPRB,  0.039_JPRB,  0.039_JPRB,  0.039_JPRB,  0.040_JPRB,  0.040_JPRB,  &
  0.040_JPRB,  0.040_JPRB,  0.041_JPRB,  0.041_JPRB,  0.041_JPRB,  0.041_JPRB,  0.041_JPRB,  &
  0.041_JPRB,  0.042_JPRB,  0.042_JPRB,  0.042_JPRB,  0.042_JPRB,  0.042_JPRB,  0.042_JPRB,  &
  0.042_JPRB,  0.043_JPRB,  0.043_JPRB,  0.043_JPRB,  0.043_JPRB,  0.043_JPRB,  0.043_JPRB,  &
  0.043_JPRB,  0.043_JPRB,  0.043_JPRB,  0.043_JPRB,  0.043_JPRB,  0.043_JPRB,  0.043_JPRB,  &
  0.043_JPRB,  0.043_JPRB,  0.044_JPRB,  0.044_JPRB,  0.044_JPRB,  0.044_JPRB,  0.044_JPRB,  &
  0.044_JPRB,  0.044_JPRB,  0.044_JPRB,  0.044_JPRB,  0.045_JPRB,  0.045_JPRB,  0.045_JPRB,  &
  0.045_JPRB,  0.045_JPRB,  0.045_JPRB,  0.046_JPRB,  0.046_JPRB,  0.046_JPRB,  0.046_JPRB,  &
  0.046_JPRB,  0.047_JPRB,  0.047_JPRB,  0.047_JPRB,  0.047_JPRB,  0.047_JPRB,  0.047_JPRB,  &
  0.047_JPRB,  0.048_JPRB,  0.048_JPRB,  0.048_JPRB,  0.048_JPRB,  0.048_JPRB,  0.048_JPRB,  &
  0.048_JPRB,  0.048_JPRB,  0.048_JPRB,  0.048_JPRB,  0.048_JPRB,  0.049_JPRB,  0.049_JPRB,  &
  0.049_JPRB,  0.049_JPRB,  0.049_JPRB,  0.049_JPRB,  0.049_JPRB,  0.049_JPRB,  0.049_JPRB,  &
  0.049_JPRB,  0.049_JPRB,  0.049_JPRB,  0.049_JPRB,  0.048_JPRB,  0.048_JPRB,  0.048_JPRB,  &
  0.048_JPRB /


CONTAINS

!------------------------------------------
! Routines for initialising database
!------------------------------------------
  
  SUBROUTINE rttov_visnirbrdf_init(     &
        &             path,             &! in
        &             imonth,           &! in
        &             verbose,          &! in
        &             err,              &! out
        &             instr_wavenum     )! in, optional
          
    ! Description:
    ! initialize the rttov vis/nir brdf algorithm by (1) reading in the MODIS VIS/NIR Global 
    ! BRDF kernel parameters data and (2) the eigenvectors of the laboratory spectra, and (2) make some
    ! precalculations for the PCA regression
    ! 
    ! History:
    ! Version   Date     Comment
    ! -------   ----     -------
    !  1.0     03/01/2012 original code J. Vidot
    !
    ! Code Description:
    !   Language:           Fortran 90.
    !   Software Standards: "European Standards for Writing and
    !     Documenting Exchangeable Fortran 90 Code".
    !
    ! Declarations:
    ! Modules used:
    
    Use rttov_const, Only :      &
            errorstatus_success, &
            errorstatus_fatal
                      
    Implicit None

    Character (len=*),  Intent(in)  :: path
    Integer(Kind=jpim), Intent(in)  :: imonth
    Logical(Kind=jplm), Intent(in)  :: verbose
    Integer(Kind=jpim), Intent(out) :: err
    
    Real(Kind=jprb),    Intent(in), Optional :: instr_wavenum(:)
    
    Integer(Kind=jpim) :: i,info
    Integer(Kind=jpim) :: ipiv(numpcs)
    Integer(Kind=jpim), Parameter :: nmonth=12_jpim

    Real(Kind=jprb)  :: A(numpcs,hngpnts),AT(hngpnts,numpcs)
    Double Precision :: B(numpcs,numpcs),C(numpcs,numpcs)
    
    Character (len=300) :: fn
    Character (len=4) :: cyear
    Character (len=2) :: cmonth

    Logical(Kind=jplm) :: file_exists
    
    TRY
    
    err = errorstatus_success
    Write(cyear,'(i4)') db_ver_year
    Write(cmonth,'(i2.2)') imonth

    Allocate(brdfmodis_flag  (nb_lats,nb_lons))
    Allocate(brdfmodis_lut   (nb_lats,nb_lons))
    Allocate(brdfmodis       (3,nb_pack,hngpnts))
    
    !-------------------------------------------------------------------
    !  reading the eigienvectors of the 126 selected laboratory spectra 
    !-------------------------------------------------------------------
    fn=TRIM(path)//'CMSsolarbrdf_labeigvects.nc'
    Inquire(FILE=fn, EXIST=file_exists)
    If (.not. file_exists) Then
      err = errorstatus_fatal
      THROWM(err .ne. errorstatus_success, 'BRDF eigenvector file not found: '//Trim(fn))
    End If

    Call rttov_brdf_read_labeigvects(Trim(fn))

    !----------------------------------------------------------------------------
    ! reading the 0.1 degree resolution MODIS BRDF kernel parameters 
    !----------------------------------------------------------------------------
    fn=TRIM(path)//'CMSsolarbrdf_V1.0_0.1deg_'//cyear//cmonth//'_mask.nc'
    Inquire(FILE=fn, EXIST=file_exists)
    If (.not. file_exists) Then
      err = errorstatus_fatal
      THROWM(err .ne. errorstatus_success, 'BRDF data file not found: '//Trim(fn))
    End If
    
    If (verbose) INFO('Using BRDF coefs: '//Trim(fn))
    Call rttov_brdf_read_brdf(Trim(fn))

          
    !---------------------------------------------------
    ! compute D matrix for HSR brdf construction
    !---------------------------------------------------
  
    ! define matrix A
    
    A=pcu_modres(1:numpcs,1:hngpnts)
    
    ! calculate transpose matrix A : AT [6X4]         
    
    AT=transpose(A) 
  
    !  B=A*A' [4X6]X[6X4]   first: row second: column
  
    B(:,:)=0.0_jprb
    B=MATMUL(A,AT)
  
    ! computing inverse of B
  
    C(:,:)=0.0_jprb
    Do i=1,numpcs
      C(i,i)=1.0_jprb
    Enddo
    
    Call DGESV( numpcs, numpcs, B, numpcs, IPIV, C, numpcs, info )
                  
    ! compute D=A'*inv(A*A')=A'*inv(B)= AT * C [6X4] * [4X4]
  
    D=MATMUL(AT,C)

    If (PRESENT(instr_wavenum)) Then
      ! If a channel wavenumber list is supplied for a particular instrument
      ! we can retrieve BRDFs much faster.

      ! Interpolate hsr data onto the channel wavenumbers
      ncoefchans = SIZE(instr_wavenum)
      Allocate(pcu_int(numpcs,ncoefchans),         &
               pcm_int(ncoefchans),                &
               snow_ref_int(ncoefchans),           &
               coastal_waters_ref_int(ncoefchans), &
               ocean_waters_ref_int(ncoefchans))
      Call rttov_brdf_hsr_interp(instr_wavenum(:))

    EndIf

    CATCH
  END SUBROUTINE rttov_visnirbrdf_init
  
  SUBROUTINE rttov_brdf_read_brdf( &
        &                      fn)
  
    ! Description:
    ! read the 0.1 degree resolution MODIS BRDF kernel parameters data   
    ! from the netCDF file into memory. 
    ! 
    ! History:
    ! Version   Date     Comment
    ! -------   ----     -------
    !  1.0      03/01/2012 original code J. Vidot 
    !
    ! Code Description:
    !   Language:           Fortran 90.
    !   Software Standards: "European Standards for Writing and
    !     Documenting Exchangeable Fortran 90 Code".
    !
    ! Declarations:
    ! Modules used:
  
    Implicit None
  
    ! Define Variables.
  
    Character(len=*), Intent(in) :: fn    ! filename including full path?


    Integer(Kind=ncint32) :: nvars     ! number of variables
    Integer(Kind=ncint32) :: ndims     ! number of dimensions
    Integer(Kind=ncint32) :: errstat   ! error code
    Integer(Kind=ncint32) :: recdim    ! record dimension
    Integer(Kind=ncint32) :: nc_dim(4) ! hng_pnt, lats, lons, pack_len
  
    Integer(Kind=jpim)    :: i,j,k
    Integer(Kind=ncint32) :: ncid, ngatts, nrecs, varid
  
    Integer(Kind=ncint16), Allocatable, Dimension(:) :: brdf_ch
  
    Character (len=1024) :: strbuf ! string buffer for var
    Character (len=6)    :: cfld 
    Integer(Kind=jpim)   :: indexlut

    ! Open netCDF file.
    errstat = nf_open(Trim(fn),nf_nowrite,ncid)
    
    ! Get info on the record dimension for this file. 
    errstat = nf_inq(ncid,ndims,nvars,ngatts,recdim)

    Do recdim=1,ndims
      errstat = nf_inq_dim(ncid,recdim,strbuf,nrecs)
      nc_dim(recdim) = nrecs
    End Do

    Allocate(brdf_ch(nc_dim(4)))

    ! Retrieve brdf database flag value
    errstat = nf_inq_varid (ncid, 'brdf_flag', varid)
    errstat = nf_get_var_int2(ncid,varid,brdfmodis_flag)
  
    ! Generate the look-up table into the emissivity data
    brdfmodis_lut(:,:) = -1_jpim
    indexlut = 1_jpim
    Do i=1,nc_dim(3)
      Do j=1,nc_dim(2)
        If (brdfmodis_flag(j,i) > 0) then
          brdfmodis_lut(j,i) = indexlut
          indexlut = indexlut + 1_jpim
        End If
      End Do
    End Do
    
    ! Retrieve database of 7 MODIS hinge-point FISO values
    Do k=1,nc_dim(1)
      write(cfld,'("fiso",i1," ")') k
      errstat = nf_inq_varid (ncid,cfld,varid)
      errstat = nf_get_var_int2(ncid,varid,brdf_ch)
      errstat = nf_get_att_real(ncid,varid,'scale_factor',sfac_fiso(k))
      errstat = nf_get_att_real(ncid,varid,'add_offset',offs_fiso(k))
      brdfmodis(1,:,k) = brdf_ch
    End Do

    ! Retrieve database of 7 MODIS hinge-point FVOL values
    Do k=1,nc_dim(1)
      write(cfld,'("fvol",i1," ")') k
      errstat = nf_inq_varid (ncid,cfld,varid)
      errstat = nf_get_var_int2(ncid,varid,brdf_ch)
      errstat = nf_get_att_real(ncid,varid,'scale_factor',sfac_fvol(k))
      errstat = nf_get_att_real(ncid,varid,'add_offset',offs_fvol(k))
      brdfmodis(2,:,k) = brdf_ch
    End Do

    ! Retrieve database of 7 MODIS hinge-point FGEO values
    Do k=1,nc_dim(1)
      write(cfld,'("fgeo",i1," ")') k
      errstat = nf_inq_varid (ncid,cfld,varid)
      errstat = nf_get_var_int2(ncid,varid,brdf_ch)
      errstat = nf_get_att_real(ncid,varid,'scale_factor',sfac_fgeo(k))
      errstat = nf_get_att_real(ncid,varid,'add_offset',offs_fgeo(k))
      brdfmodis(3,:,k) = brdf_ch
    End Do

    Deallocate(brdf_ch)

    errstat = nf_close(ncid)
  
  END SUBROUTINE rttov_brdf_read_brdf
  
  SUBROUTINE rttov_brdf_read_labeigvects( &
        &                             fn) !in
  
    ! Description:
    ! read the eigenvectors of the 126 selected laboratory spectra 
    !(created by E borbas) from the netCDF file into memory. 
    ! 
    ! History:
    ! Version   Date     Comment
    ! -------   ----     -------
    !  1.0     03/01/2012 original code J. Vidot 
    !
    ! Code Description:
    !   Language:           Fortran 90.
    !   Software Standards: "European Standards for Writing and
    !     Documenting Exchangeable Fortran 90 Code".
    !
    ! Declarations:
    ! Modules used:
  
    Implicit None
  
    ! Define Variables.
    Character(len=*), Intent(in) :: fn    ! filename including full path?

    Integer(Kind=ncint32) :: errstat   ! error code
    Integer(Kind=ncint32) :: ncid, varid

    ! Open netCDF file.
    errstat = nf_open(Trim(fn),nf_nowrite,ncid)

    ! Read the laboratory eigenvalues into array pcu
    errstat = nf_inq_varid (ncid, 'PC_scores', varid)

    ! Specify integer kinds for compatibility
    errstat = nf_get_vara_real(ncid, varid, INT((/1,1/), KIND=ncint32), INT((/numpcs,numwave/), KIND=ncint32), pcu)

    errstat = nf_close(ncid)
  
  END SUBROUTINE rttov_brdf_read_labeigvects
  

!------------------------------------------
! Routines for returning emissivity values
!------------------------------------------
  
  SUBROUTINE rttov_visnirbrdf( &
        & nchs,              &! in 
        & lat,               &! in 
        & lon,               &! in
        & surfacetype,       &! in
        & watertype,         &! in
        & vzangle,           &! in
        & vaangle,           &! in
        & szangle,           &! in
        & saangle,           &! in
        & instr_wavenum,     &! in 
        & channels,          &! in
        & instr_brdf,        &! out 
        & instr_brdf_flag,   &! out 
        & instr_bh_albedo)    ! out, optional
  
    ! Description:
    ! To compute BRDF for a given location, frequency and geometry 
    ! from the 0.1 degree resolution MODIS BRDF kernel parameters data 
    ! (at 7 hinge points) 
    ! and labratory measurements using principal component analyses
    !
    !
    ! History:
    ! Version   Date     Comment
    ! -------   ----     -------
    !  1.0     03/01/2012 original code J. Vidot 
    !
    ! Code Description:
    !   Language:           Fortran 90.
    !   Software Standards: "European Standards for Writing and
    !     Documenting Exchangeable Fortran 90 Code".
    !
    ! Declarations:
    ! Modules used:
  
    Use rttov_const, Only : pi, surftype_land, surftype_sea, watertype_fresh_water !need to be verify
  
    Implicit None
  
    Integer(Kind=jpim), intent(in)  :: nchs
    Integer(Kind=jpim), intent(in)  :: surfacetype
    Integer(Kind=jpim), intent(in)  :: watertype
    Integer(Kind=jpim), Intent(out) :: instr_brdf_flag
  
    Real(Kind=jprb),    Intent(in)  :: lat, lon, vzangle, vaangle, szangle, saangle
    Real(Kind=jprb),    Intent(in)  :: instr_wavenum(nchs)
    Integer(Kind=jpim), Intent(in)  :: channels(nchs)
    Real(Kind=jprb),    Intent(out) :: instr_brdf(nchs)
    Real(Kind=jprb),    Intent(out), Optional :: instr_bh_albedo(nchs)
  
    Real(Kind=jprb) :: modisfiso(hngpnts),modisfvol(hngpnts),modisfgeo(hngpnts)
    Real(Kind=jprb) :: modisbrdf(hngpnts),modisalbedo(hngpnts)
    Real(Kind=jprb) :: hsrbrdf(numwave),albedo(numwave)
    
    Real(Kind=jprb) :: long
    Integer(Kind=jpim) :: gridy, gridx, ilat, ilon
          
    Real(Kind=jprb) :: hkod = -999._JPRB
        
    instr_brdf(:) = hkod
    instr_brdf_flag = hkod
   
    If ( surfacetype == surftype_land ) Then
        
    !------------------------------------------------------------
    ! find the closest grid point from the MODIS BRDF kernel parameters database
    !------------------------------------------------------------
      
      long = modulo(lon,360.0_jprb)
      If (long > 180.0) Then
        long = long - 360.0_jprb
      End If
    
      ilat = NInt(lat*1000._jprb,Kind=jpim)
      ilon = NInt(long*1000._jprb,Kind=jpim)

      gridy = NInt(Abs(brdfmodis_ygrid1-ilat)*1._jprb/brdfmodis_gridres,Kind=jpim)+1_jpim
      gridx = NInt(Abs(brdfmodis_xgrid1-ilon)*1._jprb/brdfmodis_gridres,Kind=jpim)+1_jpim

      instr_brdf_flag = brdfmodis_flag(gridy,gridx)
      
      If ( instr_brdf_flag == 6 ) Return  ! No BRDF data for this location

    !------------------------------
    ! check if it is a land pixel
    !------------------------------

      If ( instr_brdf_flag > 0 ) Then
        
        ! Find the brdf
        
        If (brdfmodis_lut(gridy,gridx) > 0) Then
          modisfiso(:) = Real(brdfmodis(1,brdfmodis_lut(gridy,gridx),:),Kind=JPRB)*sfac_fiso(:) + offs_fiso(:)
          modisfvol(:) = Real(brdfmodis(2,brdfmodis_lut(gridy,gridx),:),Kind=JPRB)*sfac_fvol(:) + offs_fvol(:)
          modisfgeo(:) = Real(brdfmodis(3,brdfmodis_lut(gridy,gridx),:),Kind=JPRB)*sfac_fgeo(:) + offs_fgeo(:)
        Else
          modisfiso(:) = 0.0_jprb
          modisfvol(:) = 0.0_jprb
          modisfgeo(:) = 0.0_jprb
        End If


        Call rttov_recon_modisbrdf(&
          & vzangle,                            &! in 
          & vaangle,                            &! in 
          & szangle,                            &! in 
          & saangle,                            &! in
          & modisfiso,                          &! in
          & modisfvol,                          &! in
          & modisfgeo,                          &! in
          & modisbrdf,                          &! out
          & modisalbedo,                        &! out
          & Present(instr_bh_albedo))            ! in

        If (brdf_atlas_single_inst) Then

          !-----------------------------------------------
          ! compute the brdf at the instrument wavenumbers
          !-----------------------------------------------

          Call rttov_brdf_recon_brdf( &
            & modisbrdf,              &! in
            & channels,               &! in
            & instr_brdf)              ! out

          Where (instr_wavenum < hsr_wavenum(1) .or. &
                 instr_wavenum > hsr_wavenum(numwave))
            instr_brdf = -999._jprb
          End Where

          If (Present(instr_bh_albedo)) Then
            Call rttov_brdf_recon_brdf( &
              & modisalbedo,            &! in
              & channels,               &! in
              & instr_bh_albedo)         ! out

            Where (instr_wavenum < hsr_wavenum(1) .or. &
                   instr_wavenum > hsr_wavenum(numwave))
              instr_bh_albedo = -999._jprb
            End Where
          End If

        Else

          !------------------------------------------------------------------------------------------
          ! compute the hsr brdf spectra at 2101 wavenumber points from the 7 MODIS BRDF hinge points
          !------------------------------------------------------------------------------------------

          Call rttov_brdf_recon_hsrbrdf( &
            & modisbrdf,                 &! in
            & hsrbrdf)                    ! out

          If (Present(instr_bh_albedo)) Then
            Call rttov_brdf_recon_hsrbrdf( &
              & modisalbedo,               &! in
              & albedo)                     ! out
          End If

          !--------------------------------------------------------------------------------
          ! create instrument specific brdf by finding the closest wavenumber value
          !--------------------------------------------------------------------------------

          Call rttov_brdf_select_wavenum( &
            & hsrbrdf,                    &! in
            & nchs,                       &! in
            & instr_wavenum(1:nchs),      &! in
            & instr_brdf)                  ! out

          !--------------------------------------------------------------------------------
          ! create instrument specific bi-hemispherical albedo by finding the closest wavenumber value
          !--------------------------------------------------------------------------------

          If (Present(instr_bh_albedo)) Then
            Call rttov_brdf_select_wavenum( &
              & albedo,                     &! in
              & nchs,                       &! in
              & instr_wavenum(1:nchs),      &! in
              & instr_bh_albedo)             ! out
          End If

        End If ! single_inst

      End If  ! brdf flag = 0

    End If ! surftype_land

    !--------------------------------------------------------------------------------
    ! if water surface, then used default spectra for open ocean or coastal waters
    !--------------------------------------------------------------------------------

    If ( surfacetype == surftype_sea ) Then

      If (brdf_atlas_single_inst) Then

        If ( watertype == watertype_fresh_water ) Then
          instr_brdf = coastal_waters_ref_int(channels(:))
        Else
          instr_brdf = ocean_waters_ref_int(channels(:))
        End If

        Where (instr_wavenum < hsr_wavenum(1) .or. &
               instr_wavenum > hsr_wavenum(numwave))
          instr_brdf = -999._jprb
        End Where

      Else

        If ( watertype == watertype_fresh_water ) Then
          hsrbrdf = coastal_waters_ref
        Else
          hsrbrdf = ocean_waters_ref
        End If

        Call rttov_brdf_select_wavenum( &
            & hsrbrdf,                  &! in
            & nchs,                     &! in
            & instr_wavenum(1:nchs),    &! in
            & instr_brdf)                ! out

      End If

    End If

    ! Cap the final BRDFs here for consistency between single-
    ! and multi-instrument initialisation.
    WHERE (instr_brdf > -999._jprb)
      instr_brdf = MAX(instr_brdf, 0._JPRB)
      instr_brdf = MIN(instr_brdf, 1._JPRB)

      ! The atlas gives BRFs, but we want the output to be a BRDF so normalise by pi
      instr_brdf = instr_brdf / pi
    END WHERE

    IF (PRESENT(instr_bh_albedo)) THEN
      WHERE (instr_bh_albedo > -999._jprb)
        instr_bh_albedo = MAX(instr_bh_albedo, 0._JPRB)
        instr_bh_albedo = MIN(instr_bh_albedo, 1._JPRB)
      ENDWHERE
    END IF

  END SUBROUTINE rttov_visnirbrdf

  SUBROUTINE rttov_recon_modisbrdf( &
          & vzangle,                            &! in 
          & vaangle,                            &! in 
          & szangle,                            &! in 
          & saangle,                            &! in
          & modisfiso,                          &! in
          & modisfvol,                          &! in
          & modisfgeo,                          &! in
          & modisbrdf,                          &! out
          & modisalbedo,                        &! out
          & calc_albedo)                         ! in

    ! Description:
    !   Compute the brdf and the bi-hemispherical albedo from the semi
    !   empirical model of Ross-Li from the three kernel parameters
    !   following the paper of Lucht et al.(2000) Eq 37 for BRDF
    !   and Eq. 46 for albedo.
    !
    ! History:
    ! Version   Date     Comment
    ! -------   ----     -------
    !  1.0     03/01/2012 original code J. Vidot 
    !
    ! Code Description:
    !   Language:           Fortran 90.
    !   Software Standards: "European Standards for Writing and
    !     Documenting Exchangeable Fortran 90 Code".
    !
    ! Declarations:
    ! Modules used:
          
  USE rttov_const, ONLY :      &
          deg2rad,             &
          pi
  
  
    Implicit None

    Real(Kind=jprb), Intent(in)  :: vzangle, vaangle, szangle, saangle
    Real(Kind=jprb), Intent(in)  :: modisfiso(hngpnts)
    Real(Kind=jprb), Intent(in)  :: modisfvol(hngpnts)
    Real(Kind=jprb), Intent(in)  :: modisfgeo(hngpnts)
    Real(Kind=jprb), Intent(out) :: modisbrdf(hngpnts)
    Real(Kind=jprb), Intent(out) :: modisalbedo(hngpnts)
    Logical(Kind=jplm), Intent(in) :: calc_albedo

    Integer(Kind=jpim) :: j
    Real(Kind=jprb) :: cossz, cosvz, tansz, tanvz, cosva_sa
    Real(Kind=jprb) :: cos_ksi,sin_ksi,ksi
    Real(Kind=jprb) :: kernel_vol,kernel_geo
    Real(Kind=jprb) :: delta_kgeo,sec_szangle,sec_vzangle,sec,cos_t,sin_t,t_kgeo

    Real(Kind=jprb) :: bh_alb_param(6)

    !--------------------------------------------------------------------------------
    ! see Lucht et al., 2000, IEEE vol 38, no 2, pp 977-998 for equations and 
    ! for coefficients to compute bi-hemispherical (or black-sky) albedo in Table 1
    !--------------------------------------------------------------------------------

    DATA bh_alb_param / &
    -0.007574_JPRB, -0.070987_JPRB, 0.307588_JPRB, -1.284909_JPRB, -0.166314_JPRB, 0.041840_JPRB /

    cossz = cos(szangle*deg2rad)
    cosvz = cos(vzangle*deg2rad)
    tansz = tan(szangle*deg2rad)
    tanvz = tan(vzangle*deg2rad)
    cosva_sa = cos((vaangle-saangle)*deg2rad)

    cos_ksi = cossz*cosvz + sin(szangle*deg2rad)*sin(vzangle*deg2rad)*cosva_sa
    sin_ksi = sqrt(1._JPRB-cos_ksi**2_JPIM)
    ksi = acos(cos_ksi)
    sec_szangle = 1._JPRB/cossz
    sec_vzangle = 1._JPRB/cosvz
    sec = sec_szangle + sec_vzangle
    kernel_vol = (((pi/2._JPRB - ksi)*cos_ksi + sin_ksi) / &
        (cossz + cosvz)) - PI/4._JPRB  !! Eq 38

    delta_kgeo = sqrt(tansz**2_JPIM + tanvz**2_JPIM - 2._JPRB * tansz*tanvz*cosva_sa)

    cos_t = (2._JPRB*sqrt(delta_kgeo**2_JPIM + &
        (tansz*tanvz)**2_JPIM * (1._JPRB - cosva_sa**2_JPIM))/sec)

    if (cos_t < -1._JPRB) cos_t = -1._JPRB
    if (cos_t > 1._JPRB) cos_t = 1._JPRB

    sin_t = sqrt(1._JPRB - cos_t**2_JPIM)
    t_kgeo = acos(cos_t)

    kernel_geo =  ((t_kgeo - sin_t*cos_t)*sec/pi - sec) + (1._JPRB + cos_ksi)*sec_vzangle*sec_szangle/2._JPRB !!eq 39

    Do j=1,hngpnts
      modisbrdf(j) = modisfiso(j) + modisfvol(j) * kernel_vol + modisfgeo(j) * kernel_geo !! Eq 37
    End Do

    If (calc_albedo) Then
      Do j=1,hngpnts
        modisalbedo(j) = modisfiso(j) + &
          modisfvol(j) * ( bh_alb_param(1) + bh_alb_param(2) * ((szangle*deg2rad)**2_JPIM) + &
          bh_alb_param(3) * ((szangle*deg2rad)**3_JPIM)) + &
          modisfgeo(j) * ( bh_alb_param(4) + bh_alb_param(5) * ((szangle*deg2rad)**2_JPIM) + &
          bh_alb_param(6) * ((szangle*deg2rad)**3_JPIM))
      End Do
    End If

  END SUBROUTINE rttov_recon_modisbrdf


  SUBROUTINE rttov_brdf_hsr_interp(instr_wavenum)

    Real(Kind=jprb), Intent(in) :: instr_wavenum(:)

    Integer(Kind=jpim) :: j, k, nchs

    Real(Kind=jprb) :: dist(numwave)
    Real(Kind=jprb) :: mindist(SIZE(instr_wavenum))
    Integer(Kind=jpim) :: ind_mindist(SIZE(instr_wavenum))

    Real(Kind=jprb) :: dwvnum1,dwvnum2,dwvsum
    Real(Kind=jprb) :: pcu1(numpcs),pcu2(numpcs),pcm1,pcm2
    Real(Kind=jprb) :: snow_ref1, snow_ref2
    Real(Kind=jprb) :: coastal_waters_ref1, coastal_waters_ref2
    Real(Kind=jprb) :: ocean_waters_ref1, ocean_waters_ref2

    !---------------------------------------------------------------
    ! finding the closest frequency from the hsr emissivity spectr
    !--------------------------------------------------------------

    nchs = SIZE(instr_wavenum)

    Do j = 1, nchs

      If (instr_wavenum(j) <= hsr_wavenum(1)) Then

        pcu_int(:,j)              = REAL(pcu(:,1), KIND=JPRB)
        pcm_int(j)                = pcm(1)
        snow_ref_int(j)           = snow_ref(1)
        coastal_waters_ref_int(j) = coastal_waters_ref(1)
        ocean_waters_ref_int(j)   = ocean_waters_ref(1)

      Else If (instr_wavenum(j) >= hsr_wavenum(numwave)) Then

        pcu_int(:,j)              = REAL(pcu(:,numwave), KIND=JPRB)
        pcm_int(j)                = pcm(numwave)
        snow_ref_int(j)           = snow_ref(numwave)
        coastal_waters_ref_int(j) = coastal_waters_ref(numwave)
        ocean_waters_ref_int(j)   = ocean_waters_ref(numwave)

      Else ! within wavenumber compute range

        mindist(j) = 100._JPRB
        ind_mindist(j) = 100000_jpim

        Do k = 1, numwave

          ! calculate distances between the instr freq and hsr emissivities
          dist(k) = abs( instr_wavenum(j) - hsr_wavenum(k) )

          ! finding the closest frequency from the hsr emissivity
          If( dist(k) <=  mindist(j) ) Then

            mindist(j) = dist(k)
            ind_mindist(j) = k

          End If
        End do

  !--------------------------------
  ! Interpolate values
  !--------------------------------

  ! Bilinear mean of the two closest spectral points

        k = 1
        If ( instr_wavenum(j) <= hsr_wavenum(ind_mindist(j)) ) k = -1

        dwvnum1 = dist( ind_mindist(j) )
        dwvnum2 = dist( ind_mindist(j) + k )
        dwvsum = dwvnum1 + dwvnum2

        pcu1(:)   = dwvnum1 * REAL(pcu(:,ind_mindist(j)+k), KIND=JPRB)
        pcu2(:)   = dwvnum2 * REAL(pcu(:,ind_mindist(j)), KIND=JPRB)
        pcm1      = dwvnum1 * pcm(ind_mindist(j)+k)
        pcm2      = dwvnum2 * pcm(ind_mindist(j))
        snow_ref1 = dwvnum1 * snow_ref(ind_mindist(j)+k)
        snow_ref2 = dwvnum2 * snow_ref(ind_mindist(j))
        coastal_waters_ref1 = dwvnum1 * coastal_waters_ref(ind_mindist(j)+k)
        coastal_waters_ref2 = dwvnum2 * coastal_waters_ref(ind_mindist(j))
        ocean_waters_ref1   = dwvnum1 * ocean_waters_ref(ind_mindist(j)+k)
        ocean_waters_ref2   = dwvnum2 * ocean_waters_ref(ind_mindist(j))

        pcu_int(:,j)    = (pcu1(:) + pcu2(:)) / dwvsum
        pcm_int(j)      = (pcm1 + pcm2) / dwvsum
        snow_ref_int(j) = (snow_ref1 + snow_ref2) / dwvsum
        coastal_waters_ref_int(j) = (coastal_waters_ref1 + coastal_waters_ref2) / dwvsum
        ocean_waters_ref_int(j)   = (ocean_waters_ref1 + ocean_waters_ref2) / dwvsum

      End If

    End Do

  END SUBROUTINE rttov_brdf_hsr_interp

  SUBROUTINE rttov_brdf_recon_brdf( &
        & modisbrdf,                &! in
        & channels,                 &! in
        & brdf)                      ! out

    ! Description:
    ! Reconstruct BRDFs at instrument wavenumbers from interpolated PCs
    !
    !
    ! History:
    ! Version   Date     Comment
    ! -------   ----     -------
    !  1.0     03/01/2012 original code J. Vidot
    !
    ! Code Description:
    !   Language:           Fortran 90.
    !   Software Standards: "European Standards for Writing and
    !     Documenting Exchangeable Fortran 90 Code".
    !
    ! Declarations:
    ! Modules used:

    Implicit None

    Real(Kind=jprb),    Intent(in)  :: modisbrdf(hngpnts)
    Integer(Kind=jpim), Intent(in)  :: channels(:)
    Real(Kind=jprb),    Intent(out) :: brdf(SIZE(channels))

    Integer(Kind=jpim) :: k, nchn

    Real(Kind=jprb) :: col(hngpnts)
    Real(Kind=jprb) :: coef(numpcs)

    !-------------------------
    ! calculate the regcoef
    !-------------------------

    If ( ALL(modisbrdf(:) > 0._JPRB) ) Then

      col(:) = modisbrdf(:) * 100._JPRB - pcm_modres(:)

      Do k = 1, numpcs
        coef(k) = SUM(col(:) * D(:,k))
      End Do

    !-----------------------------------
    ! apply regcoef to get the hsr dataset
    !-----------------------------------

      nchn = SIZE(brdf)
      Do k = 1, nchn

        brdf(k) = SUM(coef(:) * REAL(pcu_int(:,channels(k)),KIND=JPRB)) + pcm_int(channels(k))
        brdf(k) = brdf(k) / 100._JPRB

        ! This is done in the calling subroutine
!         brdf(k) = MAX(brdf(k), 0._JPRB)
!         brdf(k) = MIN(brdf(k), 1._JPRB)

      End Do

    Else

      brdf = -999._JPRB

    End If

  END SUBROUTINE rttov_brdf_recon_brdf


  SUBROUTINE rttov_brdf_recon_hsrbrdf( &
        & modisbrdf,                   &! in 
        & hsrbrdf)                      ! out 
          
    ! Description:
    ! To creates high spectra resolution brdf at 2101 wavenumbers
    ! from the MODIS BRDF kernel parameters data (at 7 hinge points) 
    ! and labratory measurements using principal component analyses
    !
    !
    ! History:
    ! Version   Date     Comment
    ! -------   ----     -------
    !  1.0     03/01/2012 original code J. Vidot 
    !
    ! Code Description:
    !   Language:           Fortran 90.
    !   Software Standards: "European Standards for Writing and
    !     Documenting Exchangeable Fortran 90 Code".
    !
    ! Declarations:
    ! Modules used:

    Implicit None


    Real(Kind=jprb), Intent(in)  :: modisbrdf(hngpnts)
    Real(Kind=jprb), Intent(out) :: hsrbrdf(numwave)


    Integer(Kind=jpim) :: k

    Real(Kind=jprb) :: col(hngpnts)
    Real(Kind=jprb) :: coef(numpcs)

    !-------------------------
    ! calculate the regcoef
    !-------------------------

    If ( ALL(modisbrdf(:) > 0._JPRB) ) Then

      col(:) = modisbrdf(:) * 100._JPRB - pcm_modres(:)

      Do k = 1, numpcs
        coef(k) = SUM(col(:) * D(:,k))
      End Do

    !-----------------------------------
    ! apply regcoef to get the hsr dataset
    !-----------------------------------

      Do k = 1, numwave

        hsrbrdf(k) = SUM(coef(:) * REAL(pcu(:,k),KIND=JPRB)) + pcm(k)
        hsrbrdf(k) = hsrbrdf(k) / 100._JPRB

        ! This is done in the calling subroutine for consistency
        ! with the single-instrument initialisation
!         hsrbrdf(k) = MAX(hsrbrdf(k), 0._JPRB)
!         hsrbrdf(k) = MIN(hsrbrdf(k), 1._JPRB)

      End Do

    Else

      hsrbrdf = -999._JPRB

    End If

  END SUBROUTINE rttov_brdf_recon_hsrbrdf


  SUBROUTINE rttov_brdf_select_wavenum( &
        & hsrbrdf,                      &! in
        & nchs,                         &! in
        & instr_wavenum,                &! in
        & instr_brdf)                    ! out
  
    ! Description:
    ! subroutine to find the closest wavenumber from the HSR brdf spectra 
    ! for the instrument frequency and assign the instrument brdf y by choosing the 
    ! closest spectral point value or bilinear interpolating  between the two 
    ! closest spectral point values
    !
    !
    ! History:
    ! Version   Date     Comment
    ! -------   ----     -------
    !  1.0     03/01/2012 original code J. Vidot 
    !
    ! Code Description:
    !   Language:           Fortran 90.
    !   Software Standards: "European Standards for Writing and
    !     Documenting Exchangeable Fortran 90 Code".
    !
    ! Declarations:
    ! Modules used:
              
    Integer(Kind=jpim), Intent(in) :: nchs
  
    Real(Kind=jprb), Intent(in) :: hsrbrdf(numwave)
    Real(Kind=jprb), Intent(in) :: instr_wavenum(nchs)
  
    Real(Kind=jprb), Intent(out) :: instr_brdf(nchs)
  
    Integer(Kind=jpim) :: j,k
  
    Real(Kind=jprb) :: dist(numwave)
    Real(Kind=jprb) :: mindist(nchs)
    Integer(Kind=jpim) :: ind_mindist(nchs)
          
    Real(Kind=jprb) :: dwvnum1,dwvnum2,dwvsum
    Real(Kind=jprb) :: hsrbrdf1,hsrbrdf2
    Logical(Kind=jplm) :: lcpu_brdf
  
    Real(Kind=jprb) :: hkod = -999._JPRB
  
    ! initialize instrument brdf
    
    !---------------------------------------------------------------
    ! finindg the closest frequency from the hsr brdf spectr
    !--------------------------------------------------------------

    lcpu_brdf = .True.
    If ( All(hsrbrdf == hsrbrdf(1)) ) lcpu_brdf = .False.

    If (lcpu_brdf) Then
      instr_brdf(:)       = hkod
      Do j = 1, nchs

        If(instr_wavenum(j) >= hsr_wavenum(1) .and. &
           instr_wavenum(j) <= hsr_wavenum(numwave)) Then

          ! within wavenumber compute range

          mindist(j) = 100._JPRB
          ind_mindist(j) = 100000_jpim

          Do k = 1, numwave
  
            ! calucalte distances between the instr freq end hsr brdf
            dist(k) = abs( instr_wavenum(j) - hsr_wavenum(k) )
        
            ! finding the closest frequency from the hsr brdf 
            If( dist(k) <=  mindist(j) ) Then
        
              mindist(j) = dist(k)
              ind_mindist(j) = k
        
            End If
          End do
  
    !--------------------------------
    ! assign instrument brdf
    !-------------------------------- 
    !  closest spectral point
    !                       instr_brdf(j)=hsrbrdf(ind_mindist(j))
    
    ! or bilinear mean of the two closest spectral points
  
          k = 1
          If ( instr_wavenum(j) <= hsr_wavenum(ind_mindist(j)) ) k=-1

          dwvnum1 = dist( ind_mindist(j) )
          dwvnum2 = dist( ind_mindist(j) + k )
          dwvsum = dwvnum1 + dwvnum2

          If ( lcpu_brdf ) Then
            hsrbrdf1 = dwvnum1 * hsrbrdf( ind_mindist(j) + k )
            hsrbrdf2 = dwvnum2 * hsrbrdf( ind_mindist(j) )
            instr_brdf(j) = ( hsrbrdf1 + hsrbrdf2 ) / dwvsum
          Else
            instr_brdf(j) = hsrbrdf(1)
          End If
  
        End If    !==  (instr_wavenum(j) <= hsr_wavenum(1)) 
  
      End Do
  
    Else  ! all logical computes (lcpu_brdf) are false
      instr_brdf(:)=hsrbrdf(1)
    End If
  
  END SUBROUTINE rttov_brdf_select_wavenum
  
!------------------------------------
! Routine to deallocate atlas arrays
!------------------------------------
  SUBROUTINE rttov_visnirbrdf_close_atlas
    IF ( ALLOCATED(brdfmodis_flag)  ) DEALLOCATE(brdfmodis_flag)
    IF ( ALLOCATED(brdfmodis_lut)   ) DEALLOCATE(brdfmodis_lut)
    IF ( ALLOCATED(brdfmodis)       ) DEALLOCATE(brdfmodis)
    IF ( ALLOCATED(pcu_int)         ) DEALLOCATE(pcu_int)
    IF ( ALLOCATED(pcm_int)         ) DEALLOCATE(pcm_int)
    IF ( ALLOCATED(snow_ref_int)    ) DEALLOCATE(snow_ref_int)
    IF ( ALLOCATED(coastal_waters_ref_int) ) DEALLOCATE(coastal_waters_ref_int)
    IF ( ALLOCATED(ocean_waters_ref_int)   ) DEALLOCATE(ocean_waters_ref_int)
  END SUBROUTINE rttov_visnirbrdf_close_atlas
  
END MODULE mod_brdf_atlas
